@@ -2,6 +2,7 @@
  * Copyright (c) 2021 Mellanox Technologies. All rights reserved.
  * Copyright (c) 2022      Amazon.com, Inc. or its affiliates.
  *                         All Rights reserved.
+ * Copyright (c) 2022 NVIDIA Corporation. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,23 +30,63 @@ int mca_coll_ucc_init_query(bool enable_progress_threads, bool enable_mpi_thread
 
 static void mca_coll_ucc_module_clear(mca_coll_ucc_module_t *ucc_module)
 {
-    ucc_module->ucc_team             = NULL;
-    ucc_module->previous_allreduce   = NULL;
-    ucc_module->previous_iallreduce  = NULL;
-    ucc_module->previous_barrier     = NULL;
-    ucc_module->previous_ibarrier    = NULL;
-    ucc_module->previous_bcast       = NULL;
-    ucc_module->previous_ibcast      = NULL;
-    ucc_module->previous_alltoall    = NULL;
-    ucc_module->previous_ialltoall   = NULL;
-    ucc_module->previous_alltoallv   = NULL;
-    ucc_module->previous_ialltoallv  = NULL;
-    ucc_module->previous_allgather   = NULL;
-    ucc_module->previous_iallgather  = NULL;
-    ucc_module->previous_allgatherv  = NULL;
-    ucc_module->previous_iallgatherv = NULL;
-    ucc_module->previous_reduce      = NULL;
-    ucc_module->previous_ireduce     = NULL;
+    ucc_module->ucc_team                              = NULL;
+    ucc_module->previous_allreduce                    = NULL;
+    ucc_module->previous_allreduce_module             = NULL;
+    ucc_module->previous_iallreduce                   = NULL;
+    ucc_module->previous_iallreduce_module            = NULL;
+    ucc_module->previous_barrier                      = NULL;
+    ucc_module->previous_barrier_module               = NULL;
+    ucc_module->previous_ibarrier                     = NULL;
+    ucc_module->previous_ibarrier_module              = NULL;
+    ucc_module->previous_bcast                        = NULL;
+    ucc_module->previous_bcast_module                 = NULL;
+    ucc_module->previous_ibcast                       = NULL;
+    ucc_module->previous_ibcast_module                = NULL;
+    ucc_module->previous_alltoall                     = NULL;
+    ucc_module->previous_alltoall_module              = NULL;
+    ucc_module->previous_ialltoall                    = NULL;
+    ucc_module->previous_ialltoall_module             = NULL;
+    ucc_module->previous_alltoallv                    = NULL;
+    ucc_module->previous_alltoallv_module             = NULL;
+    ucc_module->previous_ialltoallv                   = NULL;
+    ucc_module->previous_ialltoallv_module            = NULL;
+    ucc_module->previous_allgather                    = NULL;
+    ucc_module->previous_allgather_module             = NULL;
+    ucc_module->previous_iallgather                   = NULL;
+    ucc_module->previous_iallgather_module            = NULL;
+    ucc_module->previous_allgatherv                   = NULL;
+    ucc_module->previous_allgatherv_module            = NULL;
+    ucc_module->previous_iallgatherv                  = NULL;
+    ucc_module->previous_iallgatherv_module           = NULL;
+    ucc_module->previous_reduce                       = NULL;
+    ucc_module->previous_reduce_module                = NULL;
+    ucc_module->previous_ireduce                      = NULL;
+    ucc_module->previous_ireduce_module               = NULL;
+    ucc_module->previous_gather                       = NULL;
+    ucc_module->previous_gather_module                = NULL;
+    ucc_module->previous_igather                      = NULL;
+    ucc_module->previous_igather_module               = NULL;
+    ucc_module->previous_gatherv                      = NULL;
+    ucc_module->previous_gatherv_module               = NULL;
+    ucc_module->previous_igatherv                     = NULL;
+    ucc_module->previous_igatherv_module              = NULL;
+    ucc_module->previous_reduce_scatter_block         = NULL;
+    ucc_module->previous_reduce_scatter_block_module  = NULL;
+    ucc_module->previous_ireduce_scatter_block        = NULL;
+    ucc_module->previous_ireduce_scatter_block_module = NULL;
+    ucc_module->previous_reduce_scatter               = NULL;
+    ucc_module->previous_reduce_scatter_module        = NULL;
+    ucc_module->previous_ireduce_scatter              = NULL;
+    ucc_module->previous_ireduce_scatter_module       = NULL;
+    ucc_module->previous_scatterv                     = NULL;
+    ucc_module->previous_scatterv_module              = NULL;
+    ucc_module->previous_iscatterv                    = NULL;
+    ucc_module->previous_iscatterv_module             = NULL;
+    ucc_module->previous_scatter                      = NULL;
+    ucc_module->previous_scatter_module               = NULL;
+    ucc_module->previous_iscatter                     = NULL;
+    ucc_module->previous_iscatter_module              = NULL;
 }
 
 static void mca_coll_ucc_module_construct(mca_coll_ucc_module_t *ucc_module)
@@ -53,7 +94,7 @@ static void mca_coll_ucc_module_construct(mca_coll_ucc_module_t *ucc_module)
     mca_coll_ucc_module_clear(ucc_module);
 }
 
-int mca_coll_ucc_progress(void)
+static int mca_coll_ucc_progress(void)
 {
     ucc_context_progress(mca_coll_ucc_component.ucc_context);
     return OPAL_SUCCESS;
@@ -82,19 +123,30 @@ static void mca_coll_ucc_module_destruct(mca_coll_ucc_module_t *ucc_module)
     OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_iallgatherv_module);
     OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_reduce_module);
     OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_ireduce_module);
+    OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_gather_module);
+    OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_igather_module);
+    OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_gatherv_module);
+    OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_igatherv_module);
+    OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_reduce_scatter_block_module);
+    OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_ireduce_scatter_block_module);
+    OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_reduce_scatter_module);
+    OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_ireduce_scatter_module);
+    OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_scatterv_module);
+    OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_iscatterv_module);
+    OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_scatter_module);
+    OBJ_RELEASE_IF_NOT_NULL(ucc_module->previous_iscatter_module);
     mca_coll_ucc_module_clear(ucc_module);
 }
 
 #define SAVE_PREV_COLL_API(__api) do {                                                       \
         ucc_module->previous_ ## __api            = comm->c_coll->coll_ ## __api;            \
         ucc_module->previous_ ## __api ## _module = comm->c_coll->coll_ ## __api ## _module; \
-        if (!comm->c_coll->coll_ ## __api || !comm->c_coll->coll_ ## __api ## _module) {     \
-            return OMPI_ERROR;                                                               \
+        if (comm->c_coll->coll_ ## __api && comm->c_coll->coll_ ## __api ## _module) {       \
+            OBJ_RETAIN(ucc_module->previous_ ## __api ## _module);                           \
         }                                                                                    \
-        OBJ_RETAIN(ucc_module->previous_ ## __api ## _module);                               \
     } while(0)
 
-static int mca_coll_ucc_save_coll_handlers(mca_coll_ucc_module_t *ucc_module)
+static void mca_coll_ucc_save_coll_handlers(mca_coll_ucc_module_t *ucc_module)
 {
     ompi_communicator_t *comm = ucc_module->comm;
     SAVE_PREV_COLL_API(allreduce);
@@ -113,7 +165,18 @@ static int mca_coll_ucc_save_coll_handlers(mca_coll_ucc_module_t *ucc_module)
     SAVE_PREV_COLL_API(iallgatherv);
     SAVE_PREV_COLL_API(reduce);
     SAVE_PREV_COLL_API(ireduce);
-    return OMPI_SUCCESS;
+    SAVE_PREV_COLL_API(gather);
+    SAVE_PREV_COLL_API(igather);
+    SAVE_PREV_COLL_API(gatherv);
+    SAVE_PREV_COLL_API(igatherv);
+    SAVE_PREV_COLL_API(reduce_scatter_block);
+    SAVE_PREV_COLL_API(ireduce_scatter_block);
+    SAVE_PREV_COLL_API(reduce_scatter);
+    SAVE_PREV_COLL_API(ireduce_scatter);
+    SAVE_PREV_COLL_API(scatterv);
+    SAVE_PREV_COLL_API(iscatterv);
+    SAVE_PREV_COLL_API(scatter);
+    SAVE_PREV_COLL_API(iscatter);
 }
 
 /*
@@ -299,8 +362,7 @@ static int mca_coll_ucc_init_ctx() {
     }
     ucc_context_config_release(ctx_config);
 
-    copy_fn.attr_communicator_copy_fn  = (MPI_Comm_internal_copy_attr_function*)
-        MPI_COMM_NULL_COPY_FN;
+    copy_fn.attr_communicator_copy_fn  = MPI_COMM_NULL_COPY_FN;
     del_fn.attr_communicator_delete_fn = ucc_comm_attr_del_fn;
     if (OMPI_SUCCESS != ompi_attr_create_keyval(COMM_ATTR, copy_fn, del_fn,
                                                 &ucc_comm_attr_keyval, NULL ,0, NULL)) {
@@ -340,7 +402,8 @@ uint64_t rank_map_cb(uint64_t ep, void *cb_ctx)
 static inline ucc_ep_map_t get_rank_map(struct ompi_communicator_t *comm)
 {
     ucc_ep_map_t map;
-    int64_t      r1, r2, stride, i;
+    int64_t      r1, r2, stride;
+    uint64_t     i;
     int          is_strided;
 
     map.ep_num = ompi_comm_size(comm);
@@ -399,15 +462,11 @@ static int mca_coll_ucc_module_enable(mca_coll_base_module_t *module,
         },
         .ep       = ompi_comm_rank(comm),
         .ep_range = UCC_COLLECTIVE_EP_RANGE_CONTIG,
-        .id       = comm->c_contextid
+        .id       = ompi_comm_get_local_cid(comm)
     };
-    UCC_VERBOSE(2,"creating ucc_team for comm %p, comm_id %d, comm_size %d",
-                 (void*)comm,comm->c_contextid,ompi_comm_size(comm));
-
-    if (OMPI_SUCCESS != mca_coll_ucc_save_coll_handlers(ucc_module)){
-        UCC_ERROR("mca_coll_ucc_save_coll_handlers failed");
-        goto err;
-    }
+    UCC_VERBOSE(2, "creating ucc_team for comm %p, comm_id %llu, comm_size %d",
+                (void*)comm, (long long unsigned)team_params.id,
+                ompi_comm_size(comm));
 
     if (UCC_OK != ucc_team_create_post(&cm->ucc_context, 1,
                                        &team_params, &ucc_module->ucc_team)) {
@@ -429,6 +488,9 @@ static int mca_coll_ucc_module_enable(mca_coll_base_module_t *module,
         UCC_ERROR("ucc ompi_attr_set_c failed");
         goto err;
     }
+
+    mca_coll_ucc_save_coll_handlers(ucc_module);
+
     return OMPI_SUCCESS;
 
 err:
@@ -491,14 +553,20 @@ mca_coll_ucc_comm_query(struct ompi_communicator_t *comm, int *priority)
     ucc_module->comm                     = comm;
     ucc_module->super.coll_module_enable = mca_coll_ucc_module_enable;
     *priority                            = cm->ucc_priority;
-    SET_COLL_PTR(ucc_module, BARRIER,    barrier);
-    SET_COLL_PTR(ucc_module, BCAST,      bcast);
-    SET_COLL_PTR(ucc_module, ALLREDUCE,  allreduce);
-    SET_COLL_PTR(ucc_module, ALLTOALL,   alltoall);
-    SET_COLL_PTR(ucc_module, ALLTOALLV,  alltoallv);
-    SET_COLL_PTR(ucc_module, REDUCE,     reduce);
-    SET_COLL_PTR(ucc_module, ALLGATHER,  allgather);
-    SET_COLL_PTR(ucc_module, ALLGATHERV, allgatherv);
+    SET_COLL_PTR(ucc_module, BARRIER,         barrier);
+    SET_COLL_PTR(ucc_module, BCAST,           bcast);
+    SET_COLL_PTR(ucc_module, ALLREDUCE,       allreduce);
+    SET_COLL_PTR(ucc_module, ALLTOALL,        alltoall);
+    SET_COLL_PTR(ucc_module, ALLTOALLV,       alltoallv);
+    SET_COLL_PTR(ucc_module, REDUCE,          reduce);
+    SET_COLL_PTR(ucc_module, ALLGATHER,       allgather);
+    SET_COLL_PTR(ucc_module, ALLGATHERV,      allgatherv);
+    SET_COLL_PTR(ucc_module, GATHER,          gather);
+    SET_COLL_PTR(ucc_module, GATHERV,         gatherv);
+    SET_COLL_PTR(ucc_module, REDUCE_SCATTER,  reduce_scatter_block);
+    SET_COLL_PTR(ucc_module, REDUCE_SCATTERV, reduce_scatter);
+    SET_COLL_PTR(ucc_module, SCATTERV,        scatterv);
+    SET_COLL_PTR(ucc_module, SCATTER,         scatter);
     return &ucc_module->super;
 }
 
