@@ -4,8 +4,8 @@
  *                         reserved.
  * Copyright (c) 2013-2018 Inria.  All rights reserved.
  * Copyright (c) 2015      Bull SAS.  All rights reserved.
- * Copyright (c) 2016-2017 Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016-2019 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -13,13 +13,13 @@
  * $HEADER$
  */
 
-#include <ompi_config.h>
+#include "ompi_config.h"
 #include "common_monitoring.h"
 #include "common_monitoring_coll.h"
-#include <ompi/constants.h>
-#include <ompi/communicator/communicator.h>
-#include <opal/mca/base/mca_base_component_repository.h>
-#include <opal/class/opal_hash_table.h>
+#include "ompi/constants.h"
+#include "ompi/communicator/communicator.h"
+#include "opal/mca/base/mca_base_component_repository.h"
+#include "opal/class/opal_hash_table.h"
 #include <assert.h>
 
 /*** Monitoring specific variables ***/
@@ -30,12 +30,12 @@ struct mca_monitoring_coll_data_t {
     int world_rank;
     int is_released;
     ompi_communicator_t*p_comm;
-    size_t o2a_count;
-    size_t o2a_size;
-    size_t a2o_count;
-    size_t a2o_size;
-    size_t a2a_count;
-    size_t a2a_size;
+    opal_atomic_size_t o2a_count;
+    opal_atomic_size_t o2a_size;
+    opal_atomic_size_t a2o_count;
+    opal_atomic_size_t a2o_size;
+    opal_atomic_size_t a2a_count;
+    opal_atomic_size_t a2a_size;
 };
 
 /* Collectives operation monitoring */
@@ -123,7 +123,7 @@ void mca_common_monitoring_coll_release(mca_monitoring_coll_data_t*data)
 {
 #if OPAL_ENABLE_DEBUG
     if( NULL == data ) {
-        OPAL_MONITORING_PRINT_ERR("coll: release: data structure empty or already desallocated");
+        OPAL_MONITORING_PRINT_ERR("coll: release: data structure empty or already deallocated");
         return;
     }
 #endif /* OPAL_ENABLE_DEBUG */
@@ -137,7 +137,7 @@ static void mca_common_monitoring_coll_cond_release(mca_monitoring_coll_data_t*d
 {
 #if OPAL_ENABLE_DEBUG
     if( NULL == data ) {
-        OPAL_MONITORING_PRINT_ERR("coll: release: data structure empty or already desallocated");
+        OPAL_MONITORING_PRINT_ERR("coll: release: data structure empty or already deallocated");
         return;
     }
 #endif /* OPAL_ENABLE_DEBUG */
