@@ -5,8 +5,8 @@
  * Copyright (c) 2014-2021 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2015-2016 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2018-2021 Amazon.com, Inc. or its affiliates.  All Rights reserved.
- * Copyright (c) 2020      Triad National Security, LLC. All rights
+ * Copyright (c) 2018-2022 Amazon.com, Inc. or its affiliates.  All Rights reserved.
+ * Copyright (c) 2020-2023 Triad National Security, LLC. All rights
  *                         reserved.
  * $COPYRIGHT$
  *
@@ -28,7 +28,8 @@ static int ompi_mtl_ofi_component_register(void);
 
 static mca_mtl_base_module_t*
 ompi_mtl_ofi_component_init(bool enable_progress_threads,
-                            bool enable_mpi_threads);
+                            bool enable_mpi_threads,
+                            bool *accelerator_support);
 
 static int param_priority;
 static int control_progress;
@@ -38,7 +39,6 @@ static int ofi_tag_mode;
 
 #if OPAL_HAVE_THREAD_LOCAL
     opal_thread_local int ompi_mtl_ofi_per_thread_ctx;
-    opal_thread_local struct fi_cq_tagged_entry ompi_mtl_ofi_wc[MTL_OFI_MAX_PROG_EVENT_COUNT];
 #endif
 
 /*
@@ -150,7 +150,7 @@ ompi_mtl_ofi_component_register(void)
     }
 
     ofi_tag_mode = MTL_OFI_TAG_AUTO;
-    asprintf(&desc, "Mode specifying how many bits to use for various MPI values in OFI/Libfabric"
+    opal_asprintf(&desc, "Mode specifying how many bits to use for various MPI values in OFI/Libfabric"
             " communications. Some Libfabric provider network types can support most of Open MPI"
             " needs; others can only supply a limited number of bits, which then must be split"
             " across the MPI communicator ID, MPI source rank, and MPI tag. Three different"
@@ -182,7 +182,7 @@ ompi_mtl_ofi_component_register(void)
     control_progress = MTL_OFI_PROG_UNSPEC;
     mca_base_component_var_register (&mca_mtl_ofi_component.super.mtl_version,
                                      "control_progress",
-                                     "Specify control progress model (default: unspecificed, use provider's default). Set to auto or manual for auto or manual progress respectively.",
+                                     "Specify control progress model (default: unspecified, use provider's default). Set to auto or manual for auto or manual progress respectively.",
                                      MCA_BASE_VAR_TYPE_INT, new_enum, 0, 0,
                                      OPAL_INFO_LVL_3,
                                      MCA_BASE_VAR_SCOPE_READONLY,
@@ -251,6 +251,15 @@ ompi_mtl_ofi_component_register(void)
                                     MCA_BASE_VAR_SCOPE_READONLY,
                                     &ompi_mtl_ofi.num_ofi_contexts);
 
+    ompi_mtl_ofi.disable_hmem = false;
+    mca_base_component_var_register(&mca_mtl_ofi_component.super.mtl_version,
+                                    "disable_hmem",
+                                    "Disable HMEM usage",
+                                    MCA_BASE_VAR_TYPE_BOOL, NULL, 0, 0,
+                                    OPAL_INFO_LVL_3,
+                                    MCA_BASE_VAR_SCOPE_READONLY,
+                                    &ompi_mtl_ofi.disable_hmem);
+
     return opal_common_ofi_mca_register(&mca_mtl_ofi_component.super.mtl_version);
 }
 
@@ -330,7 +339,7 @@ select_ofi_provider(struct fi_info *providers,
     }
 
     opal_output_verbose(1, opal_common_ofi.output,
-                        "%s:%d: mtl:ofi:prov: %s\n",
+                        "%s:%d: mtl:ofi:provider: %s\n",
                         __FILE__, __LINE__,
                         (prov ? prov->fabric_attr->prov_name : "none"));
 
@@ -346,19 +355,16 @@ select_ofi_provider(struct fi_info *providers,
       * however some may have multiple info objects with different
       * attributes for the same NIC. The initial provider attributes
       * are used to ensure that all NICs we return provide the same
-      * capabilities as the inital one.
+      * capabilities as the initial one.
       *
       * We use package rank to select between NICs of equal distance
       * if we cannot calculate a package_rank, we fall back to using the
       * process id.
       */
     if (NULL != prov) {
-        prov = opal_mca_common_ofi_select_provider(prov, ompi_process_info.num_local_peers,
-                                                   ompi_process_info.my_local_rank,
-                                                   ompi_process_info.cpuset,
-                                                   ompi_process_info.pid);
-        opal_output_verbose(1, ompi_mtl_base_framework.framework_output,
-                            "%s:%d: mtl:ofi:provider: %s\n",
+        prov = opal_common_ofi_select_provider(prov, &ompi_process_info);
+        opal_output_verbose(1, opal_common_ofi.output,
+                            "%s:%d: mtl:ofi:provider:domain: %s\n",
                             __FILE__, __LINE__,
                             (prov ? prov->domain_attr->name : "none"));
     }
@@ -472,7 +478,7 @@ static int ompi_mtl_ofi_init_sep(struct fi_info *prov, int universe_size)
 
     /*
      * If SEP supported and Thread Grouping feature enabled, use
-     * num_ofi_contexts + 2. Extra 2 items is to accomodate Open MPI contextid
+     * num_ofi_contexts + 2. Extra 2 items is to accommodate Open MPI contextid
      * numbering- COMM_WORLD is 0, COMM_SELF is 1. Other user created
      * Comm contextid values are assigned sequentially starting with 3.
      */
@@ -559,9 +565,10 @@ static int ompi_mtl_ofi_init_regular_ep(struct fi_info * prov, int universe_size
 
 static mca_mtl_base_module_t*
 ompi_mtl_ofi_component_init(bool enable_progress_threads,
-                            bool enable_mpi_threads)
+                            bool enable_mpi_threads,
+                            bool *accelerator_support)
 {
-    int ret, fi_version;
+    int ret, fi_primary_version, fi_alternate_version;
     int num_local_ranks, sep_support_in_provider, max_ofi_ctxts;
     int ofi_tag_leading_zeros, ofi_tag_bits_for_cid;
     char **include_list = NULL;
@@ -570,8 +577,8 @@ ompi_mtl_ofi_component_init(bool enable_progress_threads,
     struct fi_info *providers = NULL;
     struct fi_info *prov = NULL;
     struct fi_info *prov_cq_data = NULL;
-    char ep_name[FI_NAME_MAX] = {0};
-    size_t namelen;
+    void *ep_name = NULL;
+    size_t namelen = 0;
     int universe_size;
     char *univ_size_str;
 
@@ -588,6 +595,26 @@ ompi_mtl_ofi_component_init(bool enable_progress_threads,
         exclude_list = opal_argv_split(*opal_common_ofi.prov_exclude, ',');
     }
 
+    /**
+     * Note: API version 1.5 is the first version that supports
+     * FI_LOCAL_COMM / FI_REMOTE_COMM checking (and we definitely need
+     * that checking -- e.g., the shared memory provider supports
+     * intranode communication (FI_LOCAL_COMM), but not internode
+     * (FI_REMOTE_COMM), which is insufficient for MTL selection.
+     *
+     * Note: API version 1.9 is the first version that supports FI_HMEM
+     *
+     * Note: API version 1.18 is the first version that clearly define
+     * provider's behavior in making CUDA API calls that all provider
+     * by default is permitted to make CUDA calls if application uses >= 1.18 API.
+     *
+     * If application is using < 1.18 API, some provider will not claim support
+     * of FI_HMEM (even if they are capable of) because it does not know
+     * whether application permits it to make CUDA calls.
+     */
+    fi_primary_version = FI_VERSION(1, 18);
+    fi_alternate_version = FI_VERSION(1, 9);
+
     /**
      * Hints to filter providers
      * See man fi_getinfo for a list of all filters
@@ -605,15 +632,26 @@ ompi_mtl_ofi_component_init(bool enable_progress_threads,
                             __FILE__, __LINE__);
         goto error;
     }
+
+    /* Request device transfer capabilities */
+#if defined(FI_HMEM)
+    if (false == ompi_mtl_ofi.disable_hmem) {
+        hints->caps |= FI_HMEM;
+        hints->domain_attr->mr_mode |= FI_MR_HMEM | FI_MR_ALLOCATED;
+    }
+#endif
+
+no_hmem:
+
     /* Make sure to get a RDM provider that can do the tagged matching
        interface and local communication and remote communication. */
     hints->mode               = FI_CONTEXT | FI_CONTEXT2;
     hints->ep_attr->type      = FI_EP_RDM;
-    hints->caps               = FI_TAGGED | FI_LOCAL_COMM | FI_REMOTE_COMM | FI_DIRECTED_RECV;
+    hints->caps               |= FI_MSG | FI_TAGGED | FI_LOCAL_COMM | FI_REMOTE_COMM | FI_DIRECTED_RECV;
     hints->tx_attr->msg_order = FI_ORDER_SAS;
     hints->rx_attr->msg_order = FI_ORDER_SAS;
-    hints->rx_attr->op_flags = FI_COMPLETION;
-    hints->tx_attr->op_flags = FI_COMPLETION;
+    hints->rx_attr->op_flags  = FI_COMPLETION;
+    hints->tx_attr->op_flags  = FI_COMPLETION;
 
     if (enable_mpi_threads) {
         ompi_mtl_ofi.mpi_thread_multiple = true;
@@ -657,18 +695,6 @@ ompi_mtl_ofi_component_init(bool enable_progress_threads,
 
     hints->domain_attr->resource_mgmt    = FI_RM_ENABLED;
 
-    /**
-     * FI_VERSION provides binary backward and forward compatibility support
-     * Specify the version of OFI is coded to, the provider will select struct
-     * layouts that are compatible with this version.
-     *
-     * Note: API version 1.5 is the first version that supports
-     * FI_LOCAL_COMM / FI_REMOTE_COMM checking (and we definitely need
-     * that checking -- e.g., some providers are suitable for RXD or
-     * RXM, but can't provide local communication).
-     */
-    fi_version = FI_VERSION(1, 5);
-
     /**
      * The EFA provider in Libfabric versions prior to 1.10 contains a bug
      * where the FI_LOCAL_COMM and FI_REMOTE_COMM capabilities are not
@@ -688,7 +714,11 @@ ompi_mtl_ofi_component_init(bool enable_progress_threads,
         hints_dup->caps &= ~(FI_LOCAL_COMM | FI_REMOTE_COMM);
         hints_dup->fabric_attr->prov_name = strdup("efa");
 
-        ret = fi_getinfo(fi_version, NULL, NULL, 0ULL, hints_dup, &providers);
+        ret = fi_getinfo(fi_primary_version, NULL, NULL, 0ULL, hints_dup, &providers);
+        if (FI_ENOSYS == -ret) {
+            /* libfabric is not new enough, fallback to use older version of API */
+           ret = fi_getinfo(fi_alternate_version, NULL, NULL, 0ULL, hints_dup, &providers);
+	}
 
         opal_output_verbose(1, opal_common_ofi.output,
                             "%s:%d: EFA specific fi_getinfo(): %s\n",
@@ -720,19 +750,38 @@ ompi_mtl_ofi_component_init(bool enable_progress_threads,
      * remote node or service.  this does not necessarily allocate resources.
      * Pass NULL for name/service because we want a list of providers supported.
      */
-    ret = fi_getinfo(fi_version,    /* OFI version requested                    */
+    ret = fi_getinfo(fi_primary_version,    /* OFI version requested            */
                      NULL,          /* Optional name or fabric to resolve       */
                      NULL,          /* Optional service name or port to request */
                      0ULL,          /* Optional flag                            */
                      hints,         /* In: Hints to filter providers            */
                      &providers);   /* Out: List of matching providers          */
+    if (FI_ENOSYS == -ret) {
+        ret = fi_getinfo(fi_alternate_version, NULL, NULL, 0ULL, hints, &providers);
+    }
 
     opal_output_verbose(1, opal_common_ofi.output,
                         "%s:%d: fi_getinfo(): %s\n",
                         __FILE__, __LINE__, fi_strerror(-ret));
 
-    if (FI_ENODATA == -ret) {
-        // It is not an error if no information is returned.
+    if ((FI_ENODATA == -ret)
+        || (0 == ret && include_list
+            && 0 == opal_common_ofi_count_providers_in_list(providers, include_list))
+        || (0 == ret && !include_list && exclude_list
+            && opal_common_ofi_providers_subset_of_list(providers, exclude_list))) {
+#if defined(FI_HMEM)
+        /* Attempt selecting a provider without FI_HMEM hints */
+        if (hints->caps & FI_HMEM) {
+            hints->caps &= ~FI_HMEM;
+            hints->domain_attr->mr_mode &= ~FI_MR_HMEM;
+            if (providers) {
+                (void) fi_freeinfo(providers);
+                providers = NULL;
+            }
+            goto no_hmem;
+        }
+#endif
+        /* It is not an error if no information is returned. */
         goto error;
     } else if (0 != ret) {
         opal_show_help("help-mtl-ofi.txt", "OFI call fail", true,
@@ -759,6 +808,41 @@ select_prov:
     opal_argv_free(exclude_list);
     exclude_list = NULL;
 
+    *accelerator_support = false;
+#if defined(FI_HMEM)
+    if (!(prov->caps & FI_HMEM) || (true == ompi_mtl_ofi.disable_hmem)) {
+        if (!(prov->caps & FI_HMEM) && (false == ompi_mtl_ofi.disable_hmem)) {
+            opal_output_verbose(50, opal_common_ofi.output,
+                                "%s:%d: Libfabric provider does not support device buffers. Continuing with device to host copies.\n",
+                               __FILE__, __LINE__);
+        }
+        if (true == ompi_mtl_ofi.disable_hmem) {
+            opal_output_verbose(50, opal_common_ofi.output,
+                                "%s:%d: Support for device buffers disabled by MCA parameter. Continuing with device to host copies.\n",
+                               __FILE__, __LINE__);
+        }
+    } else {
+        *accelerator_support = true;
+        ompi_mtl_ofi.hmem_needs_reg = true;
+        /*
+         * Workaround for the fact that the CXI provider actually doesn't need for accelerator memory to be registered
+         * for local buffers, but if one does do so using fi_mr_regattr, one actually needs to manage the
+         * requested_key field in the fi_mr_attr attr argument, and the OFI MTL doesn't track which requested_keys
+         * have already been registered. So just set a flag to disable local registration.  Note the OFI BTL doesn't
+         * have a problem here since it uses fi_mr_regattr only within the context of an rcache, and manages the
+         * requested_key field in this way.
+         */
+         if (!strncasecmp(prov->fabric_attr->prov_name, "cxi", 3)) {
+             ompi_mtl_ofi.hmem_needs_reg = false;
+         }
+
+    }
+#else
+    opal_output_verbose(50, opal_common_ofi.output,
+                        "%s:%d: Libfabric provider does not support device buffers. Continuing with device to host copies.\n",
+                        __FILE__, __LINE__);
+#endif
+
     /**
      * Select the format of the OFI tag
      */
@@ -871,7 +955,7 @@ select_prov:
      * that before domain open.  Silently ignore not-supported errors,
      * as they are not critical to program correctness, but only
      * indicate that LIbfabric will have to pick a different, possibly
-     * less optimial, monitor. */
+     * less optimal, monitor. */
     ret = opal_common_ofi_export_memory_monitor();
     if (0 != ret && -FI_ENOSYS != ret) {
         opal_output_verbose(1, opal_common_ofi.output,
@@ -908,19 +992,6 @@ select_prov:
          prov->domain_attr->mr_mode = FI_MR_BASIC;
     }
 
-#if OPAL_CUDA_SUPPORT
-    /**
-     * Some providers do not require the use of the CUDA convertor
-     * in OMPI and its use will cause performance degradation. The
-     * following providers will disable it when selected.
-     */
-    if (!strncmp(prov->fabric_attr->prov_name, "psm3", 4)
-        || !strncmp(prov->fabric_attr->prov_name, "psm2", 4))
-    {
-        ompi_mtl_ofi.base.mtl_flags |= MCA_MTL_BASE_FLAG_CUDA_INIT_DISABLE;
-    }
-#endif /* OPAL_CUDA_SUPPORT */
-
     /**
      * Create the access domain, which is the physical or virtual network or
      * hardware port/collection of ports.  Returns a domain object that can be
@@ -928,7 +999,7 @@ select_prov:
      */
     ret = fi_domain(ompi_mtl_ofi.fabric,  /* In:  Fabric object                 */
                     prov,                 /* In:  Provider                      */
-                    &ompi_mtl_ofi.domain, /* Out: Domain oject                  */
+                    &ompi_mtl_ofi.domain, /* Out: Domain object                 */
                     NULL);                /* Optional context for domain events */
     if (0 != ret) {
         opal_show_help("help-mtl-ofi.txt", "OFI call fail", true,
@@ -1031,15 +1102,11 @@ select_prov:
     fi_freeinfo(providers);
     providers = NULL;
 
-    /**
-     * Get our address and publish it with modex.
-     */
-    namelen = sizeof(ep_name);
-    ret = fi_getname((fid_t)ompi_mtl_ofi.sep,
-                     &ep_name[0],
-                     &namelen);
-    if (ret) {
-        MTL_OFI_LOG_FI_ERR(ret, "fi_getname failed");
+    ret = opal_common_ofi_fi_getname((fid_t)ompi_mtl_ofi.sep,
+                                     &ep_name,
+                                     &namelen);
+    if (OMPI_SUCCESS != ret) {
+        MTL_OFI_LOG_FI_ERR(ret, "opal_common_ofi_fi_getname failed");
         goto error;
     }
 
@@ -1055,11 +1122,16 @@ select_prov:
     }
 
     ompi_mtl_ofi.epnamelen = namelen;
+    free(ep_name);
 
     /**
      * Set the ANY_SRC address.
      */
     ompi_mtl_ofi.any_addr = FI_ADDR_UNSPEC;
+    ompi_mtl_ofi.is_initialized = false;
+    ompi_mtl_ofi.has_posted_initial_buffer = false;
+    
+    ompi_mtl_ofi.base.mtl_flags |= MCA_MTL_BASE_FLAG_SUPPORTS_EXT_CID;
 
     return &ompi_mtl_ofi.base;
 
@@ -1106,6 +1178,9 @@ error:
     if (ompi_mtl_ofi.ofi_ctxt) {
         free(ompi_mtl_ofi.ofi_ctxt);
     }
+    if (ep_name) {
+        free(ep_name);
+    }
 
     return NULL;
 }
