@@ -20,98 +20,86 @@ OBJ_CLASS_INSTANCE(ompi_osc_ucx_lock_t, opal_object_t, NULL, NULL);
 
 static inline int start_shared(ompi_osc_ucx_module_t *module, int target) {
     uint64_t result_value = -1;
-    ucp_ep_h ep = OSC_UCX_GET_EP(module->comm, target);
-    ucp_rkey_h rkey = (module->state_info_array)[target].rkey;
-    uint64_t remote_addr = (module->state_info_array)[target].addr + OSC_UCX_STATE_LOCK_OFFSET;
-    ucs_status_t status;
-    int ret;
+    uint64_t remote_addr = (module->state_addrs)[target] + OSC_UCX_STATE_LOCK_OFFSET;
+    ucp_ep_h *ep;
+    OSC_UCX_GET_DEFAULT_EP(ep, module, target);
+    int ret = OMPI_SUCCESS;
 
     while (true) {
-        ret = opal_common_ucx_atomic_fetch(ep, UCP_ATOMIC_FETCH_OP_FADD, 1,
-                                           &result_value, sizeof(result_value),
-                                           remote_addr, rkey, mca_osc_ucx_component.ucp_worker);
+        ret = opal_common_ucx_wpmem_fetch(module->state_mem, UCP_ATOMIC_FETCH_OP_FADD, 1,
+                                        target, &result_value, sizeof(result_value),
+                                        remote_addr, ep);
         if (OMPI_SUCCESS != ret) {
             return ret;
         }
+
         assert((int64_t)result_value >= 0);
         if (result_value >= TARGET_LOCK_EXCLUSIVE) {
-            status = ucp_atomic_post(ep, UCP_ATOMIC_POST_OP_ADD, (-1), sizeof(uint64_t),
-                                     remote_addr, rkey);
-            if (status != UCS_OK) {
-                OSC_UCX_VERBOSE(1, "ucp_atomic_add64 failed: %d", status);
-                return OMPI_ERROR;
+            ret = opal_common_ucx_wpmem_post(module->state_mem,
+                                           UCP_ATOMIC_POST_OP_ADD, (-1), target,
+                                           sizeof(uint64_t), remote_addr, ep);
+            if (OMPI_SUCCESS != ret) {
+                return ret;
             }
         } else {
             break;
         }
-        ucp_worker_progress(mca_osc_ucx_component.ucp_worker);
+        opal_common_ucx_wpool_progress(mca_osc_ucx_component.wpool);
     }
 
-    return OMPI_SUCCESS;
+    return ret;
 }
 
 static inline int end_shared(ompi_osc_ucx_module_t *module, int target) {
-    ucp_ep_h ep = OSC_UCX_GET_EP(module->comm, target);
-    ucp_rkey_h rkey = (module->state_info_array)[target].rkey;
-    uint64_t remote_addr = (module->state_info_array)[target].addr + OSC_UCX_STATE_LOCK_OFFSET;
-    ucs_status_t status;
-
-    status = ucp_atomic_post(ep, UCP_ATOMIC_POST_OP_ADD, (-1), sizeof(uint64_t),
-                             remote_addr, rkey);
-    if (status != UCS_OK) {
-        OSC_UCX_VERBOSE(1, "ucp_atomic_post(OP_ADD) failed: %d", status);
-        return OMPI_ERROR;
-    }
-
-    return OMPI_SUCCESS;
+    uint64_t remote_addr = (module->state_addrs)[target] + OSC_UCX_STATE_LOCK_OFFSET;
+    ucp_ep_h *ep;
+    OSC_UCX_GET_DEFAULT_EP(ep, module, target);
+    return opal_common_ucx_wpmem_post(module->state_mem, UCP_ATOMIC_POST_OP_ADD,
+                                    (-1), target, sizeof(uint64_t), remote_addr, ep);
 }
 
 static inline int start_exclusive(ompi_osc_ucx_module_t *module, int target) {
     uint64_t result_value = -1;
-    ucp_ep_h ep = OSC_UCX_GET_EP(module->comm, target);
-    ucp_rkey_h rkey = (module->state_info_array)[target].rkey;
-    uint64_t remote_addr = (module->state_info_array)[target].addr + OSC_UCX_STATE_LOCK_OFFSET;
-    ucs_status_t status;
+    uint64_t remote_addr = (module->state_addrs)[target] + OSC_UCX_STATE_LOCK_OFFSET;
+    ucp_ep_h *ep;
+    OSC_UCX_GET_DEFAULT_EP(ep, module, target);
+    int ret = OMPI_SUCCESS;
 
     for (;;) {
-        status = opal_common_ucx_atomic_cswap(ep, TARGET_LOCK_UNLOCKED, TARGET_LOCK_EXCLUSIVE,
-                                              &result_value, sizeof(result_value),
-                                              remote_addr, rkey,
-                                              mca_osc_ucx_component.ucp_worker);
-        if (status != UCS_OK) {
-            return OMPI_ERROR;
+        ret = opal_common_ucx_wpmem_cmpswp(module->state_mem,
+                                         TARGET_LOCK_UNLOCKED, TARGET_LOCK_EXCLUSIVE,
+                                         target, &result_value, sizeof(result_value),
+                                         remote_addr, ep);
+        if (OMPI_SUCCESS != ret) {
+            return ret;
         }
         if (result_value == TARGET_LOCK_UNLOCKED) {
             return OMPI_SUCCESS;
         }
-
-        ucp_worker_progress(mca_osc_ucx_component.ucp_worker);
+        opal_common_ucx_wpool_progress(mca_osc_ucx_component.wpool);
     }
 }
 
 static inline int end_exclusive(ompi_osc_ucx_module_t *module, int target) {
-    ucp_ep_h ep = OSC_UCX_GET_EP(module->comm, target);
-    ucp_rkey_h rkey = (module->state_info_array)[target].rkey;
-    uint64_t remote_addr = (module->state_info_array)[target].addr + OSC_UCX_STATE_LOCK_OFFSET;
-    ucs_status_t status;
-
-    status = ucp_atomic_post(ep, UCP_ATOMIC_POST_OP_ADD,
-                             -((int64_t)TARGET_LOCK_EXCLUSIVE), sizeof(uint64_t),
-                             remote_addr, rkey);
-    if (UCS_OK != status) {
-        OSC_UCX_VERBOSE(1, "ucp_atomic_post(OP_ADD) failed: %d", status);
-        return OMPI_ERROR;
-    }
-
-    return OMPI_SUCCESS;
+    uint64_t remote_addr = (module->state_addrs)[target] + OSC_UCX_STATE_LOCK_OFFSET;
+    ucp_ep_h *ep;
+    OSC_UCX_GET_DEFAULT_EP(ep, module, target);
+    return opal_common_ucx_wpmem_post(module->state_mem, UCP_ATOMIC_POST_OP_ADD,
+                                      -((int64_t)TARGET_LOCK_EXCLUSIVE), target,
+                                      sizeof(uint64_t), remote_addr, ep);
 }
 
-int ompi_osc_ucx_lock(int lock_type, int target, int assert, struct ompi_win_t *win) {
+int ompi_osc_ucx_lock(int lock_type, int target, int mpi_assert, struct ompi_win_t *win) {
     ompi_osc_ucx_module_t *module = (ompi_osc_ucx_module_t *)win->w_osc_module;
     ompi_osc_ucx_lock_t *lock = NULL;
     ompi_osc_ucx_epoch_t original_epoch = module->epoch_type.access;
     int ret = OMPI_SUCCESS;
 
+    if (module->no_locks) {
+        OSC_UCX_VERBOSE(1, "attempted to lock with no_locks set");
+        return OMPI_ERR_RMA_SYNC;
+    }
+
     if (module->lock_count == 0) {
         if (module->epoch_type.access != NONE_EPOCH &&
             module->epoch_type.access != FENCE_EPOCH) {
@@ -133,7 +121,7 @@ int ompi_osc_ucx_lock(int lock_type, int target, int assert, struct ompi_win_t *
     lock = OBJ_NEW(ompi_osc_ucx_lock_t);
     lock->target_rank = target;
 
-    if ((assert & MPI_MODE_NOCHECK) == 0) {
+    if ((mpi_assert & MPI_MODE_NOCHECK) == 0) {
         lock->is_nocheck = false;
         if (lock_type == MPI_LOCK_EXCLUSIVE) {
             ret = start_exclusive(module, target);
@@ -144,6 +132,11 @@ int ompi_osc_ucx_lock(int lock_type, int target, int assert, struct ompi_win_t *
         }
     } else {
         lock->is_nocheck = true;
+        if (lock_type == MPI_LOCK_EXCLUSIVE) {
+            lock->type = LOCK_EXCLUSIVE;
+        } else {
+            lock->type = LOCK_SHARED;
+        }
     }
 
     if (ret == OMPI_SUCCESS) {
@@ -160,7 +153,6 @@ int ompi_osc_ucx_unlock(int target, struct ompi_win_t *win) {
     ompi_osc_ucx_module_t *module = (ompi_osc_ucx_module_t *)win->w_osc_module;
     ompi_osc_ucx_lock_t *lock = NULL;
     int ret = OMPI_SUCCESS;
-    ucp_ep_h ep;
 
     if (module->epoch_type.access != PASSIVE_EPOCH) {
         return OMPI_ERR_RMA_SYNC;
@@ -171,17 +163,12 @@ int ompi_osc_ucx_unlock(int target, struct ompi_win_t *win) {
         return OMPI_ERR_RMA_SYNC;
     }
 
-    opal_hash_table_remove_value_uint32(&module->outstanding_locks,
-                                        (uint32_t)target);
-
-    ep = OSC_UCX_GET_EP(module->comm, target);
-    ret = opal_common_ucx_ep_flush(ep, mca_osc_ucx_component.ucp_worker);
+    ret = opal_common_ucx_ctx_flush(module->ctx, OPAL_COMMON_UCX_SCOPE_WORKER, 0);
     if (ret != OMPI_SUCCESS) {
         return ret;
     }
-
-    module->global_ops_num -= module->per_target_ops_nums[target];
-    module->per_target_ops_nums[target] = 0;
+    opal_hash_table_remove_value_uint32(&module->outstanding_locks,
+                                        (uint32_t)target);
 
     if (lock->is_nocheck == false) {
         if (lock->type == LOCK_EXCLUSIVE) {
@@ -197,16 +184,20 @@ int ompi_osc_ucx_unlock(int target, struct ompi_win_t *win) {
     assert(module->lock_count >= 0);
     if (module->lock_count == 0) {
         module->epoch_type.access = NONE_EPOCH;
-        assert(module->global_ops_num == 0);
     }
 
     return ret;
 }
 
-int ompi_osc_ucx_lock_all(int assert, struct ompi_win_t *win) {
+int ompi_osc_ucx_lock_all(int mpi_assert, struct ompi_win_t *win) {
     ompi_osc_ucx_module_t *module = (ompi_osc_ucx_module_t*) win->w_osc_module;
     int ret = OMPI_SUCCESS;
 
+    if (module->no_locks) {
+        OSC_UCX_VERBOSE(1, "attempted to lock with no_locks set");
+        return OMPI_ERR_RMA_SYNC;
+    }
+
     if (module->epoch_type.access != NONE_EPOCH &&
         module->epoch_type.access != FENCE_EPOCH) {
         return OMPI_ERR_RMA_SYNC;
@@ -214,7 +205,7 @@ int ompi_osc_ucx_lock_all(int assert, struct ompi_win_t *win) {
 
     module->epoch_type.access = PASSIVE_ALL_EPOCH;
 
-    if (0 == (assert & MPI_MODE_NOCHECK)) {
+    if (0 == (mpi_assert & MPI_MODE_NOCHECK)) {
         int i, comm_size;
         module->lock_all_is_nocheck = false;
         comm_size = ompi_comm_size(module->comm);
@@ -237,23 +228,19 @@ int ompi_osc_ucx_lock_all(int assert, struct ompi_win_t *win) {
 
 int ompi_osc_ucx_unlock_all(struct ompi_win_t *win) {
     ompi_osc_ucx_module_t *module = (ompi_osc_ucx_module_t*)win->w_osc_module;
-    int comm_size = ompi_comm_size(module->comm);
-    int ret = OMPI_SUCCESS;
+    int comm_size = ompi_comm_size(module->comm), ret = OMPI_SUCCESS;
 
     if (module->epoch_type.access != PASSIVE_ALL_EPOCH) {
         return OMPI_ERR_RMA_SYNC;
     }
 
     assert(module->lock_count == 0);
-
-    ret = opal_common_ucx_worker_flush(mca_osc_ucx_component.ucp_worker);
+ 
+    ret = opal_common_ucx_ctx_flush(module->ctx, OPAL_COMMON_UCX_SCOPE_WORKER, 0);
     if (ret != OMPI_SUCCESS) {
         return ret;
     }
 
-    module->global_ops_num = 0;
-    memset(module->per_target_ops_nums, 0, sizeof(int) * comm_size);
-
     if (!module->lock_all_is_nocheck) {
         int i;
         for (i = 0; i < comm_size; i++) {
@@ -268,7 +255,7 @@ int ompi_osc_ucx_unlock_all(struct ompi_win_t *win) {
 
 int ompi_osc_ucx_sync(struct ompi_win_t *win) {
     ompi_osc_ucx_module_t *module = (ompi_osc_ucx_module_t *)win->w_osc_module;
-    ucs_status_t status;
+    int ret = OMPI_SUCCESS;
 
     if (module->epoch_type.access != PASSIVE_EPOCH &&
         module->epoch_type.access != PASSIVE_ALL_EPOCH) {
@@ -277,55 +264,45 @@ int ompi_osc_ucx_sync(struct ompi_win_t *win) {
 
     opal_atomic_mb();
 
-    status = ucp_worker_fence(mca_osc_ucx_component.ucp_worker);
-    if (status != UCS_OK) {
-        OSC_UCX_VERBOSE(1, "ucp_worker_fence failed: %d", status);
-        return OMPI_ERROR;
+    ret = opal_common_ucx_wpmem_fence(module->mem);
+    if (ret != OMPI_SUCCESS) {
+        OSC_UCX_VERBOSE(1, "opal_common_ucx_mem_fence failed: %d", ret);
     }
 
-    return OMPI_SUCCESS;
+    return ret;
 }
 
 int ompi_osc_ucx_flush(int target, struct ompi_win_t *win) {
     ompi_osc_ucx_module_t *module = (ompi_osc_ucx_module_t*) win->w_osc_module;
-    ucp_ep_h ep;
-    int ret;
+    int ret = OMPI_SUCCESS;
 
     if (module->epoch_type.access != PASSIVE_EPOCH &&
         module->epoch_type.access != PASSIVE_ALL_EPOCH) {
         return OMPI_ERR_RMA_SYNC;
     }
 
-    ep = OSC_UCX_GET_EP(module->comm, target);
-    ret = opal_common_ucx_ep_flush(ep, mca_osc_ucx_component.ucp_worker);
+    ret = opal_common_ucx_ctx_flush(module->ctx, OPAL_COMMON_UCX_SCOPE_EP, target);
     if (ret != OMPI_SUCCESS) {
         return ret;
     }
 
-    module->global_ops_num -= module->per_target_ops_nums[target];
-    module->per_target_ops_nums[target] = 0;
-
     return OMPI_SUCCESS;
 }
 
 int ompi_osc_ucx_flush_all(struct ompi_win_t *win) {
     ompi_osc_ucx_module_t *module = (ompi_osc_ucx_module_t *)win->w_osc_module;
-    int ret;
+    int ret = OMPI_SUCCESS;
 
     if (module->epoch_type.access != PASSIVE_EPOCH &&
         module->epoch_type.access != PASSIVE_ALL_EPOCH) {
         return OMPI_ERR_RMA_SYNC;
     }
 
-    ret = opal_common_ucx_worker_flush(mca_osc_ucx_component.ucp_worker);
+    ret = opal_common_ucx_ctx_flush(module->ctx, OPAL_COMMON_UCX_SCOPE_WORKER, 0);
     if (ret != OMPI_SUCCESS) {
         return ret;
     }
 
-    module->global_ops_num = 0;
-    memset(module->per_target_ops_nums, 0,
-           sizeof(int) * ompi_comm_size(module->comm));
-
     return OMPI_SUCCESS;
 }
 
