@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2016 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -12,7 +12,7 @@
  *                         All rights reserved.
  * Copyright (c) 2009      Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2011-2012 NVIDIA Corporation.  All rights reserved.
- * Copyright (c) 2011-2016 Los Alamos National Security, LLC. All rights
+ * Copyright (c) 2011-2018 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2018      FUJITSU LIMITED.  All rights reserved.
  * $COPYRIGHT$
@@ -33,6 +33,7 @@
 #include "pml_ob1_rdma.h"
 #include "pml_ob1_rdmafrag.h"
 #include "ompi/mca/bml/bml.h"
+#include "ompi/memchecker.h"
 
 BEGIN_C_DECLS
 
@@ -45,12 +46,13 @@ typedef enum {
 struct mca_pml_ob1_send_request_t {
     mca_pml_base_send_request_t req_send;
     mca_bml_base_endpoint_t* req_endpoint;
+    mca_pml_ob1_comm_proc_t *ob1_proc;
     opal_ptr_t req_recv;
-    int32_t  req_state;
-    int32_t  req_lock;
+    opal_atomic_int32_t  req_state;
+    opal_atomic_int32_t  req_lock;
     bool     req_throttle_sends;
-    int32_t  req_pipeline_depth;
-    size_t   req_bytes_delivered;
+    opal_atomic_int32_t  req_pipeline_depth;
+    opal_atomic_size_t   req_bytes_delivered;
     uint32_t req_rdma_cnt;
     mca_pml_ob1_send_pending_t req_pending;
     opal_mutex_t req_send_range_lock;
@@ -142,7 +144,8 @@ get_request_from_send_pending(mca_pml_ob1_send_pending_t *type)
                                        tag,                             \
                                        comm,                            \
                                        sendmode,                        \
-                                       persistent)                      \
+                                       persistent,                      \
+                                       ob1_proc)                        \
     {                                                                   \
         MCA_PML_BASE_SEND_REQUEST_INIT(&(sendreq)->req_send,            \
                                        buf,                             \
@@ -155,11 +158,14 @@ get_request_from_send_pending(mca_pml_ob1_send_pending_t *type)
                                        persistent,                      \
                                        0); /* convertor_flags */        \
         (sendreq)->req_recv.pval = NULL;                                \
+        (sendreq)->ob1_proc = ob1_proc;                                 \
     }
 
 #define MCA_PML_OB1_SEND_REQUEST_RESET(sendreq)                         \
     MCA_PML_BASE_SEND_REQUEST_RESET(&(sendreq)->req_send)
 
+#define MCA_PML_OB1_SEND_REQUEST_REQUIRES_EXT_MATCH(sendreq) (-1 == sendreq->ob1_proc->comm_index)
+
 static inline void mca_pml_ob1_free_rdma_resources (mca_pml_ob1_send_request_t* sendreq)
 {
     size_t r;
@@ -203,7 +209,6 @@ do {
        (sendreq)->req_send.req_base.req_comm->c_my_rank;                             \
    (sendreq)->req_send.req_base.req_ompi.req_status.MPI_TAG =                        \
         (sendreq)->req_send.req_base.req_tag;                                        \
-   (sendreq)->req_send.req_base.req_ompi.req_status.MPI_ERROR = OMPI_SUCCESS;        \
    (sendreq)->req_send.req_base.req_ompi.req_status._ucount =                        \
         (sendreq)->req_send.req_bytes_packed;                                        \
    PERUSE_TRACE_COMM_EVENT( PERUSE_COMM_REQ_COMPLETE,                                \
@@ -214,6 +219,16 @@ do {
 
 static inline void mca_pml_ob1_send_request_fini (mca_pml_ob1_send_request_t *sendreq)
 {
+
+  /* make buffer defined when the request is completed,
+     and before releasing the objects. */
+     MEMCHECKER(
+            memchecker_call(&opal_memchecker_base_mem_defined,
+                            sendreq->req_send.req_base.req_addr,
+                            sendreq->req_send.req_base.req_count,
+                            sendreq->req_send.req_base.req_datatype);
+     );
+
     /*  Let the base handle the reference counts */
     MCA_PML_BASE_SEND_REQUEST_FINI((&(sendreq)->req_send));
     assert( NULL == sendreq->rdma_frag );
@@ -265,7 +280,10 @@ send_request_pml_complete(mca_pml_ob1_send_request_t *sendreq)
                 MCA_PML_OB1_SEND_REQUEST_MPI_COMPLETE(sendreq, true);
             } else {
                 if( MPI_SUCCESS != sendreq->req_send.req_base.req_ompi.req_status.MPI_ERROR ) {
-                    ompi_mpi_abort(&ompi_mpi_comm_world.comm, MPI_ERR_REQUEST);
+                    /* An error after freeing the request MUST be fatal
+                     * MPI3 ch3.7: MPI_REQUEST_FREE */
+                    int err = MPI_ERR_REQUEST;
+                    ompi_mpi_errors_are_fatal_comm_handler(NULL, &err, "Send error after request freed");
                 }
             }
         } else {
@@ -278,9 +296,8 @@ send_request_pml_complete(mca_pml_ob1_send_request_t *sendreq)
 static inline bool
 send_request_pml_complete_check(mca_pml_ob1_send_request_t *sendreq)
 {
-#if OPAL_ENABLE_MULTI_THREADS
     opal_atomic_rmb();
-#endif
+
     /* if no more events are expected for the request and the whole message is
      * already sent and send fragment scheduling isn't running in another
      * thread then complete the request on PML level. From now on, if user
@@ -334,12 +351,10 @@ mca_pml_ob1_send_request_schedule(mca_pml_ob1_send_request_t* sendreq)
     mca_pml_ob1_send_request_schedule_exclusive(sendreq);
 }
 
-#if OPAL_CUDA_SUPPORT
-int mca_pml_ob1_send_request_start_cuda(
+int mca_pml_ob1_send_request_start_accelerator(
     mca_pml_ob1_send_request_t* sendreq,
     mca_bml_base_btl_t* bml_btl,
     size_t size);
-#endif /* OPAL_CUDA_SUPPORT */
 
 /**
  *  Start the specified request
@@ -381,8 +396,8 @@ mca_pml_ob1_send_request_start_btl( mca_pml_ob1_send_request_t* sendreq,
     int rc;
 
 #if OPAL_CUDA_GDR_SUPPORT
-    if (btl->btl_cuda_eager_limit && (sendreq->req_send.req_base.req_convertor.flags & CONVERTOR_CUDA)) {
-        eager_limit = btl->btl_cuda_eager_limit - sizeof(mca_pml_ob1_hdr_t);
+    if (btl->btl_accelerator_eager_limit && (sendreq->req_send.req_base.req_convertor.flags & CONVERTOR_ACCELERATOR)) {
+        eager_limit = btl->btl_accelerator_eager_limit - sizeof(mca_pml_ob1_hdr_t);
     }
 #endif /* OPAL_CUDA_GDR_SUPPORT */
 
@@ -412,7 +427,9 @@ mca_pml_ob1_send_request_start_btl( mca_pml_ob1_send_request_t* sendreq,
         if(sendreq->req_send.req_send_mode == MCA_PML_BASE_SEND_BUFFERED) {
             rc = mca_pml_ob1_send_request_start_buffered(sendreq, bml_btl, size);
         } else if
-                (opal_convertor_need_buffers(&sendreq->req_send.req_base.req_convertor) == false) {
+                (opal_convertor_need_buffers(&sendreq->req_send.req_base.req_convertor) == false &&
+                !(sendreq->req_send.req_base.req_convertor.flags & CONVERTOR_ACCELERATOR) &&
+                !(sendreq->req_send.req_base.req_convertor.flags & CONVERTOR_ACCELERATOR_UNIFIED)) {
             unsigned char *base;
             opal_convertor_get_current_pointer( &sendreq->req_send.req_base.req_convertor, (void**)&base );
 
@@ -431,11 +448,9 @@ mca_pml_ob1_send_request_start_btl( mca_pml_ob1_send_request_t* sendreq,
                                                          MCA_PML_OB1_HDR_FLAGS_CONTIG);
             }
         } else {
-#if OPAL_CUDA_SUPPORT
-            if (sendreq->req_send.req_base.req_convertor.flags & CONVERTOR_CUDA) {
-                return mca_pml_ob1_send_request_start_cuda(sendreq, bml_btl, size);
+            if (sendreq->req_send.req_base.req_convertor.flags & CONVERTOR_ACCELERATOR) {
+                return mca_pml_ob1_send_request_start_accelerator(sendreq, bml_btl, size);
             }
-#endif /* OPAL_CUDA_SUPPORT */
             rc = mca_pml_ob1_send_request_start_rndv(sendreq, bml_btl, size, 0);
         }
     }
@@ -463,6 +478,18 @@ mca_pml_ob1_send_request_start_seq (mca_pml_ob1_send_request_t* sendreq, mca_bml
         /* select a btl */
         bml_btl = mca_bml_base_btl_array_get_next(&endpoint->btl_eager);
         rc = mca_pml_ob1_send_request_start_btl(sendreq, bml_btl);
+#if OPAL_ENABLE_FT_MPI
+        /* this first condition to keep the optimized path with as 
+         * little tests as possible */
+        if( OPAL_LIKELY(MPI_SUCCESS == rc) ) {
+            return rc;
+        }
+        if( OPAL_UNLIKELY(OMPI_ERR_UNREACH == rc) ) {
+            sendreq->req_send.req_base.req_ompi.req_status.MPI_ERROR = MPI_ERR_PROC_FAILED;
+            MCA_PML_OB1_SEND_REQUEST_MPI_COMPLETE(sendreq, false);
+            return MPI_SUCCESS;
+        }
+#endif /* OPAL_ENABLE_FT_MPI */
         if( OPAL_LIKELY(OMPI_ERR_OUT_OF_RESOURCE != rc) )
             return rc;
     }
@@ -490,6 +517,11 @@ mca_pml_ob1_send_request_start( mca_pml_ob1_send_request_t* sendreq )
     int32_t seqn;
 
     if (OPAL_UNLIKELY(NULL == endpoint)) {
+#if OPAL_ENABLE_FT_MPI
+        if (!sendreq->req_send.req_base.req_proc->proc_active) {
+            return MPI_ERR_PROC_FAILED;
+        }
+#endif /* OPAL_ENABLE_FT_MPI */
         return OMPI_ERR_UNREACH;
     }
 
@@ -502,9 +534,9 @@ mca_pml_ob1_send_request_start( mca_pml_ob1_send_request_t* sendreq )
  *  Initiate a put scheduled by the receiver.
  */
 
-void mca_pml_ob1_send_request_put( mca_pml_ob1_send_request_t* sendreq,
-                                   mca_btl_base_module_t* btl,
-                                   mca_pml_ob1_rdma_hdr_t* hdr );
+void mca_pml_ob1_send_request_put (mca_pml_ob1_send_request_t *sendreq,
+                                   mca_btl_base_module_t *btl,
+                                   const mca_pml_ob1_rdma_hdr_t *hdr);
 
 int mca_pml_ob1_send_request_put_frag(mca_pml_ob1_rdma_frag_t* frag);
 
