@@ -35,6 +35,8 @@
 #include "sharedfp_sm.h"
 
 #include "mpi.h"
+#include "opal/util/printf.h"
+#include "opal/util/output.h"
 #include "ompi/constants.h"
 #include "ompi/group/group.h"
 #include "ompi/proc/proc.h"
@@ -62,7 +64,6 @@ int mca_sharedfp_sm_file_open (struct ompi_communicator_t *comm,
     struct mca_sharedfp_sm_offset * sm_offset_ptr;
     struct mca_sharedfp_sm_offset sm_offset;
     int sm_fd;
-    uint32_t comm_cid;
     int int_pid;
     pid_t my_pid;
 
@@ -103,9 +104,8 @@ int mca_sharedfp_sm_file_open (struct ompi_communicator_t *comm,
     ** For sharedfp we also want to put the file backed shared memory into the tmp directory
     */
     filename_basename = opal_basename((char*)filename);
-    /* format is "%s/%s_cid-%d-%d.sm", see below */
+    /* format is "%s/%s_cid-%s-%d.sm", see below */
 
-    comm_cid = ompi_comm_get_cid(comm);
     if ( 0 == fh->f_rank ) {
         my_pid = getpid();
         int_pid = (int) my_pid;
@@ -119,8 +119,8 @@ int mca_sharedfp_sm_file_open (struct ompi_communicator_t *comm,
         return err;
     }
 
-    asprintf(&sm_filename, "%s/%s_cid-%d-%d.sm", ompi_process_info.job_session_dir,
-             filename_basename, comm_cid, int_pid);
+    opal_asprintf(&sm_filename, "%s/%s_cid-%s-%d.sm", ompi_process_info.job_session_dir,
+                  filename_basename, ompi_comm_print_cid(comm), int_pid);
     /* open shared memory file, initialize to 0, map into memory */
     sm_fd = open(sm_filename, O_RDWR | O_CREAT,
                  S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
@@ -139,7 +139,15 @@ int mca_sharedfp_sm_file_open (struct ompi_communicator_t *comm,
     /* TODO: is it necessary to write to the file first? */
     if( 0 == fh->f_rank ){
         memset ( &sm_offset, 0, sizeof (struct mca_sharedfp_sm_offset ));
-        write ( sm_fd, &sm_offset, sizeof(struct mca_sharedfp_sm_offset));
+        err = opal_best_effort_write ( sm_fd, &sm_offset, sizeof(struct mca_sharedfp_sm_offset));
+        if (OPAL_SUCCESS != err) {
+            free(filename_basename);
+            free(sm_filename);
+            free(sm_data);
+            free(sh);
+            close (sm_fd);
+            return err;
+        }
     }
     err = comm->c_coll->coll_barrier (comm, comm->c_coll->coll_barrier_module );
     if ( OMPI_SUCCESS != err ) {
@@ -253,7 +261,7 @@ int mca_sharedfp_sm_file_close (ompio_file_t *fh)
      */
     fh->f_comm->c_coll->coll_barrier (fh->f_comm, fh->f_comm->c_coll->coll_barrier_module );
 
-    file_data = (sm_data*)(sh->selected_module_data);
+    file_data = (sm_data_global*)(sh->selected_module_data);
     if (file_data)  {
         /*Close sm handle*/
         if (file_data->sm_offset_ptr) {
