@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2008      The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -11,6 +12,8 @@
  * Copyright (c) 2014-2015 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2017      IBM Corp.  All rights reserved.
+ * Copyright (c) 2018      Triad National Security, LLC. All rights
+ *                         reserved.
  */
 
 #include "ompi_config.h"
@@ -20,40 +23,23 @@
 #include "ompi/mca/topo/base/base.h"
 
 
-int mca_topo_base_dist_graph_create_adjacent(mca_topo_base_module_t* module,
-                                             ompi_communicator_t *comm_old,
-                                             int indegree, const int sources[],
-                                             const int sourceweights[],
-                                             int outdegree,
-                                             const int destinations[],
-                                             const int destweights[],
-                                             opal_info_t *info, int reorder,
-                                             ompi_communicator_t **newcomm)
+static int _mca_topo_base_dist_graph_create_adjacent (mca_topo_base_module_t* module, int indegree,
+                                                      const int sources[], const int sourceweights[],
+                                                      int outdegree, const int destinations[],
+                                                      const int destweights[], int reorder,
+                                                      ompi_communicator_t **newcomm)
 {
     mca_topo_base_comm_dist_graph_2_2_0_t *topo = NULL;
     int err;
 
-    if( OMPI_SUCCESS != (err = ompi_comm_create(comm_old,
-                                                comm_old->c_local_group,
-                                                newcomm)) ) {
-        return err;
-    }
-    // But if there is an info object, the above call didn't make use
-    // of it, so we'll do a dup-with-info to get the final comm and
-    // free the above intermediate newcomm:
-    if (info && info != &(MPI_INFO_NULL->super)) {
-        ompi_communicator_t *intermediate_comm = *newcomm;
-        ompi_comm_dup_with_info (intermediate_comm, info, newcomm);
-        ompi_comm_free(&intermediate_comm);
-    }
-
     err = OMPI_ERR_OUT_OF_RESOURCE;  /* suppose by default something bad will happens */
 
     assert( NULL == (*newcomm)->c_topo );
 
     topo = OBJ_NEW(mca_topo_base_comm_dist_graph_2_2_0_t);
-    if( NULL == topo ) {
-        goto bail_out;
+    if (NULL == topo) {
+        ompi_comm_free (newcomm);
+        return OMPI_ERR_OUT_OF_RESOURCE;
     }
     topo->in = topo->inw = NULL;
     topo->out = topo->outw = NULL;
@@ -103,16 +89,29 @@ int mca_topo_base_dist_graph_create_adjacent(mca_topo_base_module_t* module,
 
  bail_out:
     if (NULL != topo) {
-        if( NULL != topo->in ) free(topo->in);
-        if( MPI_UNWEIGHTED != sourceweights ) {
-            if( NULL != topo->inw ) free(topo->inw);
-        }
-        if( NULL != topo->out ) free(topo->out);
-        if( MPI_UNWEIGHTED != destweights ) {
-            if( NULL != topo->outw ) free(topo->outw);
-        }
         OBJ_RELEASE(topo);
     }
+
     ompi_comm_free(newcomm);
     return err;
 }
+
+int mca_topo_base_dist_graph_create_adjacent(mca_topo_base_module_t* module,
+                                             ompi_communicator_t *comm_old,
+                                             int indegree, const int sources[],
+                                             const int sourceweights[],
+                                             int outdegree,
+                                             const int destinations[],
+                                             const int destweights[],
+                                             opal_info_t *info, int reorder,
+                                             ompi_communicator_t **newcomm)
+{
+    int err;
+
+    if (OMPI_SUCCESS != (err = ompi_comm_dup_with_info (comm_old, info, newcomm))) {
+        return err;
+    }
+
+    return _mca_topo_base_dist_graph_create_adjacent (module, indegree, sources, sourceweights, outdegree,
+                                                      destinations, destweights, reorder, newcomm);
+}
