@@ -2,14 +2,15 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2018 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2010-2012 Oak Ridge National Labs.  All rights reserved.
+ * Copyright (c) 2015-2018 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
  *
@@ -52,9 +53,10 @@ int MPI_Bcast(void *buffer, int count, MPI_Datatype datatype,
             if (ompi_comm_rank(comm) == root) {
                 /* check whether root's send buffer is defined. */
                 memchecker_call(&opal_memchecker_base_isdefined, buffer, count, datatype);
+            } else {
+                /* check whether receive buffer is addressable. */
+                memchecker_call(&opal_memchecker_base_isaddressable, buffer, count, datatype);
             }
-            /* check whether receive buffer is addressable. */
-            memchecker_call(&opal_memchecker_base_isaddressable, buffer, count, datatype);
         } else {
             if (MPI_ROOT == root) {
                 /* check whether root's send buffer is defined. */
@@ -70,7 +72,7 @@ int MPI_Bcast(void *buffer, int count, MPI_Datatype datatype,
       err = MPI_SUCCESS;
       OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
       if (ompi_comm_invalid(comm)) {
-          return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM,
+          return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_COMM,
                                      FUNC_NAME);
       }
 
@@ -98,7 +100,18 @@ int MPI_Bcast(void *buffer, int count, MPI_Datatype datatype,
             return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_ROOT, FUNC_NAME);
         }
       }
+    } 
+
+#if OPAL_ENABLE_FT_MPI
+    /*
+     * An early check, so as to return early if we are using a broken
+     * communicator. This is not absolutely necessary since we will
+     * check for this, and other, error conditions during the operation.
+     */
+    if( OPAL_UNLIKELY(!ompi_comm_iface_coll_check(comm, &err)) ) {
+        OMPI_ERRHANDLER_RETURN(err, comm, err, FUNC_NAME);
     }
+#endif
 
     /* If there's only one node, or if the count is 0, we're done */
 
@@ -107,8 +120,6 @@ int MPI_Bcast(void *buffer, int count, MPI_Datatype datatype,
         return MPI_SUCCESS;
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
     /* Invoke the coll component to perform the back-end operation */
 
     err = comm->c_coll->coll_bcast(buffer, count, datatype, root, comm,
