@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2013 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -59,7 +59,7 @@ int MPI_Cart_create(MPI_Comm old_comm, int ndims, const int dims[],
             return OMPI_ERRHANDLER_INVOKE (MPI_COMM_WORLD, MPI_ERR_COMM,
                                           FUNC_NAME);
         } else if (OMPI_COMM_IS_INTER(old_comm)) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM,
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_COMM,
                                           FUNC_NAME);
         }
         if (ndims < 0) {
@@ -100,11 +100,21 @@ int MPI_Cart_create(MPI_Comm old_comm, int ndims, const int dims[],
         return err;
     }
 
+#if OPAL_ENABLE_FT_MPI
+    /*
+     * An early check, so as to return early if we are using a broken
+     * communicator. This is not absolutely necessary since we will
+     * check for this, and other, error conditions during the operation.
+     */
+    if( OPAL_UNLIKELY(!ompi_comm_iface_create_check(old_comm, &err)) ) {
+       OMPI_ERRHANDLER_RETURN(err, old_comm, err, FUNC_NAME);
+    }
+#endif
+
     /* Now let that topology module rearrange procs/ranks if it wants to */
     err = topo->topo.cart.cart_create(topo, old_comm,
                                       ndims, dims, periods,
                                       (0 == reorder) ? false : true, comm_cart);
-    OPAL_CR_EXIT_LIBRARY();
 
     if (MPI_SUCCESS != err) {
         OBJ_RELEASE(topo);
