@@ -2,14 +2,14 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2007-2020 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2015      Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * $COPYRIGHT$
@@ -41,21 +41,17 @@ static const char FUNC_NAME[] = "MPI_Free_mem";
 
 int MPI_Free_mem(void *baseptr)
 {
-    OPAL_CR_ENTER_LIBRARY();
-
     /* Per these threads:
 
-         http://www.open-mpi.org/community/lists/devel/2007/07/1977.php
-         http://www.open-mpi.org/community/lists/devel/2007/07/1979.php
+         https://www.open-mpi.org/community/lists/devel/2007/07/1977.php
+         https://www.open-mpi.org/community/lists/devel/2007/07/1979.php
 
        If you call MPI_ALLOC_MEM with a size of 0, you get NULL
        back.  So don't consider a NULL==baseptr an error. */
     if (NULL != baseptr && OMPI_SUCCESS != mca_mpool_base_free(baseptr)) {
-        OPAL_CR_EXIT_LIBRARY();
-        return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_NO_MEM, FUNC_NAME);
+        return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_NO_MEM, FUNC_NAME);
     }
 
-    OPAL_CR_EXIT_LIBRARY();
     return MPI_SUCCESS;
 }
 
