@@ -78,7 +78,12 @@ int MPI_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status
         return MPI_SUCCESS;
     }
 
-    OPAL_CR_ENTER_LIBRARY();
+#if OPAL_ENABLE_FT_MPI
+    /*
+     * The call will check for process failure errors during the
+     * operation. So no need to check here.
+     */
+#endif
 
     rc = MCA_PML_CALL(iprobe(source, tag, comm, flag, status));
 
