@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2017 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2008 High Performance Computing Center Stuttgart,
@@ -13,8 +13,8 @@
  * Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2015      Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2015-2019 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -29,7 +29,9 @@
 #include "ompi/communicator/communicator.h"
 #include "ompi/errhandler/errhandler.h"
 #include "ompi/datatype/ompi_datatype.h"
+#include "ompi/mca/coll/base/coll_base_util.h"
 #include "ompi/memchecker.h"
+#include "ompi/runtime/ompi_spc.h"
 
 #if OMPI_BUILD_MPI_PROFILING
 #if OPAL_HAVE_WEAK_SYMBOLS
@@ -47,6 +49,8 @@ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[
 {
     int i, size, err;
 
+    SPC_RECORD(OMPI_SPC_ISCATTERV, 1);
+
     MEMCHECKER(
         ptrdiff_t ext;
 
@@ -93,7 +97,7 @@ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[
         err = MPI_SUCCESS;
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
         if (ompi_comm_invalid(comm)) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM,
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_COMM,
                                           FUNC_NAME);
         } else if ((ompi_comm_rank(comm) != root && MPI_IN_PLACE == recvbuf) ||
                    (ompi_comm_rank(comm) == root && MPI_IN_PLACE == sendbuf)) {
@@ -187,11 +191,28 @@ int MPI_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[
         }
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
     /* Invoke the coll component to perform the back-end operation */
     err = comm->c_coll->coll_iscatterv(sendbuf, sendcounts, displs,
                                       sendtype, recvbuf, recvcount, recvtype, root, comm,
                                       request, comm->c_coll->coll_iscatterv_module);
+    if (OPAL_LIKELY(OMPI_SUCCESS == err)) {
+        if (OMPI_COMM_IS_INTRA(comm)) {
+            if (MPI_IN_PLACE == recvbuf) {
+                recvtype = NULL;
+            } else if (ompi_comm_rank(comm) != root) {
+                sendtype = NULL;
+            }
+        } else {
+            if (MPI_ROOT == root) {
+                recvtype = NULL;
+            } else if (MPI_PROC_NULL == root) {
+                sendtype = NULL;
+                recvtype = NULL;
+            } else {
+                sendtype = NULL;
+            }
+        }
+        ompi_coll_base_retain_datatypes(*request, sendtype, recvtype);
+    }
     OMPI_ERRHANDLER_RETURN(err, comm, err, FUNC_NAME);
 }
