@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2007 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -45,16 +45,14 @@ int MPI_Type_free_keyval(int *type_keyval)
     if (MPI_PARAM_CHECK) {
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
         if (NULL == type_keyval) {
-            return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD,
+            return OMPI_ERRHANDLER_NOHANDLE_INVOKE(
                                           MPI_ERR_ARG,
                                           FUNC_NAME);
         }
     }
 
-    OPAL_CR_ENTER_LIBRARY();
-
     ret = ompi_attr_free_keyval(TYPE_ATTR, type_keyval, 0);
 
-    OMPI_ERRHANDLER_RETURN(ret, MPI_COMM_WORLD,
+    OMPI_ERRHANDLER_NOHANDLE_RETURN(ret, 
 			  MPI_ERR_OTHER, FUNC_NAME);
 }
