@@ -1,8 +1,9 @@
 #!/usr/bin/env perl
 #
 # Copyright (c) 2014-2015 Cisco Systems, Inc.  All rights reserved.
-# Copyright (c) 2015-2018 Research Organization for Information Science
+# Copyright (c) 2015-2021 Research Organization for Information Science
 #                         and Technology (RIST).  All rights reserved.
+# Copyright (c) 2022      IBM Corporation.  All rights reserved.
 # $COPYRIGHT$
 #
 # Script to generate the overloaded MPI_SIZEOF interfaces and
@@ -32,13 +33,19 @@ my $ierror_arg;
 my $maxrank_arg;
 my $generate_arg;
 my $mpi_arg;
+my $mpi_real2;
+my $mpi_iso_real16;
 my $mpi_real16;
+my $mpi_complex4;
 my $mpi_complex32;
 my $pmpi_arg;
 my $help_arg = 0;
+my $request_deprecate = 0;
+my $mpi_version = 0;
 
 &Getopt::Long::Configure("bundling");
 my $ok = Getopt::Long::GetOptions("complex32=i" => \$mpi_complex32,
+                                  "complex4=i" => \$mpi_complex4,
                                   "header=s" => \$header_arg,
                                   "impl=s" => \$impl_arg,
                                   "ierror=s" => \$ierror_arg,
@@ -47,6 +54,10 @@ my $ok = Getopt::Long::GetOptions("complex32=i" => \$mpi_complex32,
                                   "mpi" => \$mpi_arg,
                                   "pmpi" => \$pmpi_arg,
                                   "real16=i" => \$mpi_real16,
+                                  "real2=i" => \$mpi_real2,
+                                  "iso_real16=i" => \$mpi_iso_real16,
+                                  "request_deprecate=i" => \$request_deprecate,
+                                  "mpi_version=i" => \$mpi_version,
                                   "help|h" => \$help_arg);
 
 die "Must specify header and/or impl filenames to output"
@@ -60,8 +71,9 @@ die "max array rank must be >= 4 and <=15"
 die "Must specify --pmpi and/or --mpi if --impl is specified"
     if (defined($generate_arg) && $generate_arg &&
         (defined($impl_arg) && !defined($mpi_arg) && !defined($pmpi_arg)));
-die "Must specify real16 and complex32"
-    if (!defined($mpi_real16) || !defined($mpi_complex32));
+die "Must specify real2, iso_real16, real16, complex4, and complex32"
+    if (!defined($mpi_real2) || !defined($mpi_iso_real16) || !defined($mpi_real16) ||
+        !defined($mpi_complex4) || !defined($mpi_complex32));
 
 #############################################################################
 
@@ -99,7 +111,13 @@ ${indent}  INTEGER$optional_ierror_param, INTENT(OUT) :: ierror";
     $subr->{start} = $start;
     $subr->{middle} = "${indent}  size = storage_size(x) / 8
 ${indent}  ${optional_ierror_statement}ierror = 0";
-    $subr->{end} = "${indent}END SUBROUTINE ^PREFIX^$sub_name^RANK^";
+    if (($mpi_version >= 4) && ($request_deprecate == 1)) {
+        $subr->{end} = "!GCC\$ ATTRIBUTES DEPRECATED :: ^PREFIX^$sub_name^RANK^\n";
+    }
+    else {
+        $subr->{end} = "";
+    }
+    $subr->{end} .= "${indent}END SUBROUTINE ^PREFIX^$sub_name^RANK^";
 
     # Save it in the overall hash
     $subs->{$sub_name} = $subr;
@@ -148,14 +166,20 @@ sub generate {
 for my $size (qw/8 16 32 64/) {
     queue_sub("integer(int${size})", "int${size}", "int${size}");
 }
-for my $size (qw/32 64 128/) {
-    if ($size != 128 || $mpi_real16 == 1) {
+for my $size (qw/16 32 64 128/) {
+    if (!($size == 16 && $mpi_real2 == 0 && $mpi_iso_real16 == 0) &&
+        !($size == 128 && $mpi_real16 == 0)) {
         queue_sub("real(real${size})", "real${size}", "real${size}");
     }
-    if ($size != 128 || $mpi_complex32 == 1) {
+    if (!($size == 16 && $mpi_complex4 == 0 && $mpi_iso_real16 == 0) &&
+        !($size == 128 && $mpi_complex32 == 0)) {
         queue_sub("complex(real${size})", "complex${size}", "real${size}");
     }
 }
+if ($mpi_real2 == 1 && $mpi_iso_real16 == 0) {
+    queue_sub("real*2", "real16");
+    queue_sub("complex*4", "complex16");
+}
 queue_sub("character", "character");
 queue_sub("logical", "logical");
 
@@ -220,7 +244,7 @@ sub output_file {
 ! Specifically: we need support for the INTERFACE keyword,
 ! ISO_FORTRAN_ENV, and the STORAGE_SIZE() intrinsic on all types.
 ! Apparently, this compiler does not support both of those things, so
-! this file will be (effecitvely) blank (i.e., we didn't bother
+! this file will be (effectively) blank (i.e., we didn't bother
 ! generating the necessary stuff for MPI_SIZEOF because the compiler
 ! doesn't support
 ! it).
