@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -82,7 +82,7 @@ void ompi_pack_external_size_f(char *datarep, MPI_Fint *incount,
 
     if (OMPI_SUCCESS != (ret = ompi_fortran_string_f2c(datarep, datarep_len,
                                                        &c_datarep))) {
-        c_ierr = OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, ret,
+        c_ierr = OMPI_ERRHANDLER_NOHANDLE_INVOKE(ret,
                                         "MPI_PACK_EXTERNAL");
         if (NULL != ierr) *ierr = OMPI_INT_2_FINT(c_ierr);
         return;
