@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -34,13 +34,12 @@
 #pragma weak PMPI_Win_create_keyval_f = ompi_win_create_keyval_f
 #pragma weak PMPI_Win_create_keyval_f08 = ompi_win_create_keyval_f
 #else
-OMPI_GENERATE_F77_BINDINGS (PMPI_WIN_CREATE_KEYVAL,
-                           pmpi_win_create_keyval,
-                           pmpi_win_create_keyval_,
-                           pmpi_win_create_keyval__,
-                           pompi_win_create_keyval_f,
-                           (ompi_aint_copy_attr_function* win_copy_attr_fn, ompi_aint_delete_attr_function* win_delete_attr_fn, MPI_Fint *win_keyval, MPI_Aint *extra_state, MPI_Fint *ierr),
-                           (win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state, ierr) )
+OMPI_GENERATE_F77_BINDINGS(PMPI_WIN_CREATE_KEYVAL, pmpi_win_create_keyval, pmpi_win_create_keyval_,
+                           pmpi_win_create_keyval__, pompi_win_create_keyval_f,
+                           (ompi_aint_copy_attr_function win_copy_attr_fn,
+                            ompi_aint_delete_attr_function win_delete_attr_fn, MPI_Fint *win_keyval,
+                            MPI_Aint *extra_state, MPI_Fint *ierr),
+                           (win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state, ierr))
 #endif
 #endif
 
@@ -54,13 +53,12 @@ OMPI_GENERATE_F77_BINDINGS (PMPI_WIN_CREATE_KEYVAL,
 #pragma weak MPI_Win_create_keyval_f08 = ompi_win_create_keyval_f
 #else
 #if ! OMPI_BUILD_MPI_PROFILING
-OMPI_GENERATE_F77_BINDINGS (MPI_WIN_CREATE_KEYVAL,
-                           mpi_win_create_keyval,
-                           mpi_win_create_keyval_,
-                           mpi_win_create_keyval__,
-                           ompi_win_create_keyval_f,
-                           (ompi_aint_copy_attr_function* win_copy_attr_fn, ompi_aint_delete_attr_function* win_delete_attr_fn, MPI_Fint *win_keyval, MPI_Aint *extra_state, MPI_Fint *ierr),
-                           (win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state, ierr) )
+OMPI_GENERATE_F77_BINDINGS(MPI_WIN_CREATE_KEYVAL, mpi_win_create_keyval, mpi_win_create_keyval_,
+                           mpi_win_create_keyval__, ompi_win_create_keyval_f,
+                           (ompi_aint_copy_attr_function win_copy_attr_fn,
+                            ompi_aint_delete_attr_function win_delete_attr_fn, MPI_Fint *win_keyval,
+                            MPI_Aint *extra_state, MPI_Fint *ierr),
+                           (win_copy_attr_fn, win_delete_attr_fn, win_keyval, extra_state, ierr))
 #else
 #define ompi_win_create_keyval_f pompi_win_create_keyval_f
 #endif
@@ -68,9 +66,9 @@ OMPI_GENERATE_F77_BINDINGS (MPI_WIN_CREATE_KEYVAL,
 
 static char FUNC_NAME[] = "MPI_Win_create_keyval";
 
-void ompi_win_create_keyval_f(ompi_aint_copy_attr_function* win_copy_attr_fn,
-                             ompi_aint_delete_attr_function* win_delete_attr_fn,
-                             MPI_Fint *win_keyval, MPI_Aint *extra_state, MPI_Fint *ierr)
+void ompi_win_create_keyval_f(ompi_aint_copy_attr_function win_copy_attr_fn,
+                              ompi_aint_delete_attr_function win_delete_attr_fn,
+                              MPI_Fint *win_keyval, MPI_Aint *extra_state, MPI_Fint *ierr)
 {
     int ret, c_ierr;
     OMPI_SINGLE_NAME_DECL(win_keyval);
@@ -90,7 +88,7 @@ void ompi_win_create_keyval_f(ompi_aint_copy_attr_function* win_copy_attr_fn,
                                        NULL);
 
     if (MPI_SUCCESS != ret) {
-        c_ierr = OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_OTHER,
+        c_ierr = OMPI_ERRHANDLER_NOHANDLE_INVOKE(MPI_ERR_OTHER,
                                         FUNC_NAME);
         if (NULL != ierr) *ierr = OMPI_INT_2_FINT(c_ierr);
     } else {
