@@ -4,6 +4,11 @@
  *                         reserved.
  * Copyright (c) 2014 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2017      IBM Corporation. All rights reserved.
+ * Copyright (c) 2020      The University of Tennessee and The University
+ *                         of Tennessee Research Foundation.  All rights
+ *                         reserved.
+ * Copyright (c) 2021      Amazon.com, Inc. or its affiliates.  All Rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -11,17 +16,17 @@
  * $HEADER$
  */
 
+#include "ompi_config.h"
+
 #include "ompi/mpi/tool/mpit-internal.h"
 
-#if OPAL_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
+#if OMPI_BUILD_MPI_PROFILING
+#if OPAL_HAVE_WEAK_SYMBOLS
 #pragma weak MPI_T_enum_get_item = PMPI_T_enum_get_item
 #endif
-
-#if OMPI_PROFILING_DEFINES
-#include "ompi/mpi/tool/profile/defines.h"
+#define MPI_T_enum_get_item PMPI_T_enum_get_item
 #endif
 
-
 int MPI_T_enum_get_item(MPI_T_enum enumtype, int index, int *value, char *name,
                         int *name_len)
 {
@@ -37,7 +42,7 @@ int MPI_T_enum_get_item(MPI_T_enum enumtype, int index, int *value, char *name,
     do {
         rc = enumtype->get_count (enumtype, &count);
         if (OPAL_SUCCESS != rc) {
-            rc = MPI_ERR_OTHER;
+            rc = MPI_T_ERR_INVALID;
             break;
         }
 
@@ -48,7 +53,7 @@ int MPI_T_enum_get_item(MPI_T_enum enumtype, int index, int *value, char *name,
 
         rc = enumtype->get_value(enumtype, index, value, &tmp);
         if (OPAL_SUCCESS != rc) {
-            rc = MPI_ERR_OTHER;
+            rc = MPI_T_ERR_INVALID;
             break;
         }
 
