@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2006 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2011 The University of Tennessee and The University
+ * Copyright (c) 2004-2016 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
@@ -11,13 +11,15 @@
  * Copyright (c) 2004-2006 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006-2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2010-2012 Oak Ridge National Labs.  All rights reserved.
  * Copyright (c) 2012-2015 Los Alamos National Security, LLC.  All rights
  *                         reserved.
- * Copyright (c) 2013-2015 Intel, Inc. All rights reserved
- * Copyright (c) 2014-2016 Research Organization for Information Science
+ * Copyright (c) 2013-2020 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2014-2017 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2015-2017 Mellanox Technologies. All rights reserved.
  *
+ * Copyright (c) 2021      Nanook Consulting.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -32,18 +34,18 @@
 
 #include "ompi/constants.h"
 #include "opal/datatype/opal_convertor.h"
-#include "opal/threads/mutex.h"
-#include "opal/dss/dss.h"
+#include "opal/mca/threads/mutex.h"
 #include "opal/util/arch.h"
 #include "opal/util/show_help.h"
 #include "opal/mca/hwloc/base/base.h"
-#include "opal/mca/pmix/pmix.h"
+#include "opal/mca/pmix/pmix-internal.h"
 #include "opal/util/argv.h"
 
 #include "ompi/proc/proc.h"
 #include "ompi/datatype/ompi_datatype.h"
 #include "ompi/runtime/mpiruntime.h"
 #include "ompi/runtime/params.h"
+#include "ompi/mca/pml/pml.h"
 
 opal_list_t  ompi_proc_list = {{0}};
 static opal_mutex_t ompi_proc_lock;
@@ -65,6 +67,9 @@ OBJ_CLASS_INSTANCE(
 
 void ompi_proc_construct(ompi_proc_t* proc)
 {
+#if OPAL_ENABLE_FT_MPI
+    proc->proc_active = true;
+#endif
     bzero(proc->proc_endpoints, sizeof(proc->proc_endpoints));
 
     /* By default all processors are supposedly having the same architecture as me. Thus,
@@ -85,9 +90,6 @@ void ompi_proc_destruct(ompi_proc_t* proc)
      * destroyed here. It will be destroyed later when the ompi_datatype_finalize is called.
      */
     OBJ_RELEASE( proc->super.proc_convertor );
-    if (NULL != proc->super.proc_hostname) {
-        free(proc->super.proc_hostname);
-    }
     opal_mutex_lock (&ompi_proc_lock);
     opal_list_remove_item(&ompi_proc_list, (opal_list_item_t*)proc);
     opal_hash_table_remove_value_ptr (&ompi_proc_hash, &proc->super.proc_name, sizeof (proc->super.proc_name));
@@ -134,40 +136,36 @@ static int ompi_proc_allocate (ompi_jobid_t jobid, ompi_vpid_t vpid, ompi_proc_t
  */
 int ompi_proc_complete_init_single (ompi_proc_t *proc)
 {
-    int ret;
-
     if ((OMPI_CAST_RTE_NAME(&proc->super.proc_name)->jobid == OMPI_PROC_MY_NAME->jobid) &&
         (OMPI_CAST_RTE_NAME(&proc->super.proc_name)->vpid  == OMPI_PROC_MY_NAME->vpid)) {
         /* nothing else to do */
         return OMPI_SUCCESS;
     }
 
-    /* we can retrieve the hostname at no cost because it
-     * was provided at startup - but make it optional so
-     * we don't chase after it if some system doesn't
-     * provide it */
-    proc->super.proc_hostname = NULL;
-    OPAL_MODEX_RECV_VALUE_OPTIONAL(ret, OPAL_PMIX_HOSTNAME, &proc->super.proc_name,
-                                   (char**)&(proc->super.proc_hostname), OPAL_STRING);
-
 #if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
     /* get the remote architecture - this might force a modex except
      * for those environments where the RM provides it */
     {
         uint32_t *ui32ptr;
-        ui32ptr = &(proc->super.proc_arch);
-        OPAL_MODEX_RECV_VALUE(ret, OPAL_PMIX_ARCH, &proc->super.proc_name,
-                              (void**)&ui32ptr, OPAL_UINT32);
-        if (OPAL_SUCCESS == ret) {
-            /* if arch is different than mine, create a new convertor for this proc */
-            if (proc->super.proc_arch != opal_local_arch) {
-                OBJ_RELEASE(proc->super.proc_convertor);
-                proc->super.proc_convertor = opal_convertor_create(proc->super.proc_arch, 0);
-            }
-        } else if (OMPI_ERR_NOT_IMPLEMENTED == ret) {
+        int ret;
+        /* if the proc is local, then no need to fetch it */
+        if (OPAL_PROC_ON_LOCAL_NODE(proc->super.proc_flags)) {
             proc->super.proc_arch = opal_local_arch;
         } else {
-            return ret;
+            ui32ptr = &(proc->super.proc_arch);
+            OPAL_MODEX_RECV_VALUE_OPTIONAL(ret, "OMPI_ARCH", &proc->super.proc_name,
+                                           (void**)&ui32ptr, PMIX_UINT32);
+            if (OPAL_SUCCESS == ret) {
+                /* if arch is different than mine, create a new convertor for this proc */
+                if (proc->super.proc_arch != opal_local_arch) {
+                    OBJ_RELEASE(proc->super.proc_convertor);
+                    proc->super.proc_convertor = opal_convertor_create(proc->super.proc_arch, 0);
+                }
+            } else if (OMPI_ERR_NOT_IMPLEMENTED == ret) {
+                proc->super.proc_arch = opal_local_arch;
+            } else {
+                return ret;
+            }
         }
     }
 #else
@@ -263,14 +261,13 @@ int ompi_proc_init(void)
     /* set local process data */
     ompi_proc_local_proc = proc;
     proc->super.proc_flags = OPAL_PROC_ALL_LOCAL;
-    proc->super.proc_hostname = strdup(ompi_process_info.nodename);
     proc->super.proc_arch = opal_local_arch;
     /* Register the local proc with OPAL */
     opal_proc_local_set(&proc->super);
 #if OPAL_ENABLE_HETEROGENEOUS_SUPPORT
     /* add our arch to the modex */
-    OPAL_MODEX_SEND_VALUE(ret, OPAL_PMIX_GLOBAL,
-                          OPAL_PMIX_ARCH, &opal_local_arch, OPAL_UINT32);
+    OPAL_MODEX_SEND_VALUE(ret, PMIX_GLOBAL,
+                          "OMPI_ARCH", &opal_local_arch, PMIX_UINT32);
     if (OPAL_SUCCESS != ret) {
         return ret;
     }
@@ -304,15 +301,19 @@ static int ompi_proc_compare_vid (opal_list_item_t **a, opal_list_item_t **b)
  */
 int ompi_proc_complete_init(void)
 {
+    opal_process_name_t wildcard_rank;
     ompi_proc_t *proc;
     int ret, errcode = OMPI_SUCCESS;
-    char *val;
+    char *val = NULL;
 
     opal_mutex_lock (&ompi_proc_lock);
 
     /* Add all local peers first */
-    OPAL_MODEX_RECV_VALUE(ret, OPAL_PMIX_LOCAL_PEERS,
-                          ORTE_PROC_MY_NAME, &val, OPAL_STRING);
+    wildcard_rank.jobid = OMPI_PROC_MY_NAME->jobid;
+    wildcard_rank.vpid = OMPI_NAME_WILDCARD->vpid;
+    /* retrieve the local peers */
+    OPAL_MODEX_RECV_VALUE(ret, PMIX_LOCAL_PEERS,
+                          &wildcard_rank, &val, PMIX_STRING);
     if (OPAL_SUCCESS == ret && NULL != val) {
         char **peers = opal_argv_split(val, ',');
         int i;
@@ -329,7 +330,7 @@ int ompi_proc_complete_init(void)
             }
             /* get the locality information - all RTEs are required
              * to provide this information at startup */
-            OPAL_MODEX_RECV_VALUE_OPTIONAL(ret, OPAL_PMIX_LOCALITY, &proc->super.proc_name, &u16ptr, OPAL_UINT16);
+            OPAL_MODEX_RECV_VALUE_OPTIONAL(ret, PMIX_LOCALITY, &proc->super.proc_name, &u16ptr, PMIX_UINT16);
             if (OPAL_SUCCESS == ret) {
                 proc->super.proc_flags = u16;
             }
@@ -351,7 +352,7 @@ int ompi_proc_complete_init(void)
      * are already in the hash table
      */
     if (ompi_process_info.num_procs < ompi_add_procs_cutoff) {
-        /* sinse ompi_proc_for_name is locking internally -
+        /* since ompi_proc_for_name is locking internally -
          * we need to release lock here
          */
         opal_mutex_unlock (&ompi_proc_lock);
@@ -368,6 +369,7 @@ int ompi_proc_complete_init(void)
     }
 
     opal_list_sort (&ompi_proc_list, ompi_proc_compare_vid);
+
     opal_mutex_unlock (&ompi_proc_lock);
 
     return errcode;
@@ -375,7 +377,7 @@ int ompi_proc_complete_init(void)
 
 int ompi_proc_finalize (void)
 {
-    opal_list_item_t *item;
+    ompi_proc_t *proc;
 
     /* Unregister the local proc from OPAL */
     opal_proc_local_set(NULL);
@@ -399,8 +401,8 @@ int ompi_proc_finalize (void)
      * it is thread safe to do so...though it may not -appear- to be so
      * without walking through the entire list/destructor sequence.
      */
-    while (opal_list_get_end(&ompi_proc_list) != (item = opal_list_get_first(&ompi_proc_list))) {
-        OBJ_RELEASE(item);
+    while ((ompi_proc_t *)opal_list_get_end(&ompi_proc_list) != (proc = (ompi_proc_t *)opal_list_get_first(&ompi_proc_list))) {
+        OBJ_RELEASE(proc);
     }
     /* now destruct the list and thread lock */
     OBJ_DESTRUCT(&ompi_proc_list);
@@ -603,7 +605,6 @@ int ompi_proc_refresh(void)
         if (i == OMPI_PROC_MY_NAME->vpid) {
             ompi_proc_local_proc = proc;
             proc->super.proc_flags = OPAL_PROC_ALL_LOCAL;
-            proc->super.proc_hostname = ompi_process_info.nodename;
             proc->super.proc_arch = opal_local_arch;
             opal_proc_local_set(&proc->super);
         } else {
@@ -621,7 +622,7 @@ int ompi_proc_refresh(void)
 
 int
 ompi_proc_pack(ompi_proc_t **proclist, int proclistsize,
-               opal_buffer_t* buf)
+               pmix_data_buffer_t* buf)
 {
     int rc;
     char *nspace;
@@ -642,40 +643,35 @@ ompi_proc_pack(ompi_proc_t **proclist, int proclistsize,
      */
     for (int i = 0 ; i < proclistsize ; ++i) {
         ompi_proc_t *proc = proclist[i];
+        pmix_proc_t prc;
 
         if (ompi_proc_is_sentinel (proc)) {
             proc = ompi_proc_for_name_nolock (ompi_proc_sentinel_to_name ((uintptr_t) proc));
         }
 
         /* send proc name */
-        rc = opal_dss.pack(buf, &(proc->super.proc_name), 1, OMPI_NAME);
-        if(rc != OPAL_SUCCESS) {
-            OMPI_ERROR_LOG(rc);
+        OPAL_PMIX_CONVERT_NAME(&prc, &(proc->super.proc_name));
+        rc = PMIx_Data_pack(NULL, buf, &prc, 1, PMIX_PROC);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
             opal_mutex_unlock (&ompi_proc_lock);
-            return rc;
+            return opal_pmix_convert_status(rc);
         }
         /* retrieve and send the corresponding nspace for this job
          * as the remote side may not know the translation */
-        nspace = (char*)opal_pmix.get_nspace(proc->super.proc_name.jobid);
-        rc = opal_dss.pack(buf, &nspace, 1, OPAL_STRING);
-        if(rc != OPAL_SUCCESS) {
-            OMPI_ERROR_LOG(rc);
+        nspace = opal_jobid_print(proc->super.proc_name.jobid);
+        rc = PMIx_Data_pack(NULL, buf, &nspace, 1, PMIX_STRING);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
             opal_mutex_unlock (&ompi_proc_lock);
-            return rc;
+            return opal_pmix_convert_status(rc);
         }
         /* pack architecture flag */
-        rc = opal_dss.pack(buf, &(proc->super.proc_arch), 1, OPAL_UINT32);
-        if(rc != OPAL_SUCCESS) {
-            OMPI_ERROR_LOG(rc);
+        rc = PMIx_Data_pack(NULL, buf, &(proc->super.proc_arch), 1, PMIX_UINT32);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
             opal_mutex_unlock (&ompi_proc_lock);
-            return rc;
-        }
-        /* pass the name of the host this proc is on */
-        rc = opal_dss.pack(buf, &(proc->super.proc_hostname), 1, OPAL_STRING);
-        if(rc != OPAL_SUCCESS) {
-            OMPI_ERROR_LOG(rc);
-            opal_mutex_unlock (&ompi_proc_lock);
-            return rc;
+            return opal_pmix_convert_status(rc);
         }
     }
     opal_mutex_unlock (&ompi_proc_lock);
@@ -704,7 +700,6 @@ ompi_proc_find_and_add(const ompi_process_name_t * name, bool* isnew)
      */
     if (NULL == rproc) {
         *isnew = true;
-        rproc = OBJ_NEW(ompi_proc_t);
         ompi_proc_allocate (name->jobid, name->vpid, &rproc);
     }
 
@@ -715,7 +710,7 @@ ompi_proc_find_and_add(const ompi_process_name_t * name, bool* isnew)
 
 
 int
-ompi_proc_unpack(opal_buffer_t* buf,
+ompi_proc_unpack(pmix_data_buffer_t* buf,
                  int proclistsize, ompi_proc_t ***proclist,
                  int *newproclistsize, ompi_proc_t ***newproclist)
 {
@@ -741,41 +736,35 @@ ompi_proc_unpack(opal_buffer_t* buf,
     for (int i = 0; i < proclistsize ; ++i){
         int32_t count=1;
         ompi_process_name_t new_name;
+        pmix_proc_t prc;
         uint32_t new_arch;
-        char *new_hostname;
         bool isnew = false;
         int rc;
         char *nspace;
+        uint16_t u16, *u16ptr;
 
-        rc = opal_dss.unpack(buf, &new_name, &count, OMPI_NAME);
-        if (rc != OPAL_SUCCESS) {
-            OMPI_ERROR_LOG(rc);
+        rc = PMIx_Data_unpack(NULL, buf, &prc, &count, PMIX_PROC);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
             free(plist);
             free(newprocs);
-            return rc;
+            return opal_pmix_convert_status(rc);
         }
-        rc = opal_dss.unpack(buf, &nspace, &count, OPAL_STRING);
-        if (rc != OPAL_SUCCESS) {
-            OMPI_ERROR_LOG(rc);
+        OPAL_PMIX_CONVERT_PROCT(rc, &new_name, &prc);
+        rc = PMIx_Data_unpack(NULL, buf, &nspace, &count, PMIX_STRING);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
             free(plist);
             free(newprocs);
-            return rc;
+            return opal_pmix_convert_status(rc);
         }
-        opal_pmix.register_jobid(new_name.jobid, nspace);
         free(nspace);
-        rc = opal_dss.unpack(buf, &new_arch, &count, OPAL_UINT32);
-        if (rc != OPAL_SUCCESS) {
-            OMPI_ERROR_LOG(rc);
+        rc = PMIx_Data_unpack(NULL, buf, &new_arch, &count, PMIX_UINT32);
+        if (PMIX_SUCCESS != rc) {
+            PMIX_ERROR_LOG(rc);
             free(plist);
             free(newprocs);
-            return rc;
-        }
-        rc = opal_dss.unpack(buf, &new_hostname, &count, OPAL_STRING);
-        if (rc != OPAL_SUCCESS) {
-            OMPI_ERROR_LOG(rc);
-            free(plist);
-            free(newprocs);
-            return rc;
+            return opal_pmix_convert_status(rc);
         }
         /* see if this proc is already on our ompi_proc_list */
         plist[i] = ompi_proc_find_and_add(&new_name, &isnew);
@@ -794,24 +783,24 @@ ompi_proc_unpack(opal_buffer_t* buf,
                 OBJ_RELEASE(plist[i]->super.proc_convertor);
                 plist[i]->super.proc_convertor = opal_convertor_create(plist[i]->super.proc_arch, 0);
 #else
+                char *errhost = opal_get_proc_hostname(&plist[i]->super);
                 opal_show_help("help-mpi-runtime.txt",
                                "heterogeneous-support-unavailable",
                                true, ompi_process_info.nodename,
-                               new_hostname == NULL ? "<hostname unavailable>" :
-                               new_hostname);
+                               errhost);
                 free(plist);
                 free(newprocs);
+                free(errhost);
                 return OMPI_ERR_NOT_SUPPORTED;
 #endif
             }
 
-            if (NULL != new_hostname) {
-                if (0 == strcmp(ompi_proc_local_proc->super.proc_hostname, new_hostname)) {
-                    plist[i]->super.proc_flags |= (OPAL_PROC_ON_NODE | OPAL_PROC_ON_CU | OPAL_PROC_ON_CLUSTER);
-                }
-
-                /* Save the hostname */
-                plist[i]->super.proc_hostname = new_hostname;
+            /* get the locality information - all RTEs are required
+             * to provide this information at startup */
+            u16ptr = &u16;
+            OPAL_MODEX_RECV_VALUE_OPTIONAL(rc, PMIX_LOCALITY, &plist[i]->super.proc_name, &u16ptr, PMIX_UINT16);
+            if (OPAL_SUCCESS == rc) {
+                plist[i]->super.proc_flags = u16;
             }
         }
     }
