@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2016 The University of Tennessee and The University
+ * Copyright (c) 2004-2020 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2007 High Performance Computing Center Stuttgart,
@@ -13,7 +13,7 @@
  * Copyright (c) 2011-2012 University of Houston. All rights reserved.
  * Copyright (c) 2010-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2014      Intel, Inc. All rights reserved.
+ * Copyright (c) 2014-2019 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -43,18 +43,11 @@
 #include "opal/class/opal_object.h"
 #include "opal/class/opal_pointer_array.h"
 #include "opal/runtime/opal.h"
-#if OPAL_ENABLE_FT_CR == 1
-#include "opal/runtime/opal_cr.h"
-#endif
 #include "opal/mca/base/base.h"
 #include "opal/runtime/opal_info_support.h"
 #include "opal/util/argv.h"
 #include "opal/util/show_help.h"
 
-#if OMPI_RTE_ORTE
-#include "orte/runtime/orte_info_support.h"
-#endif
-
 #include "ompi/communicator/communicator.h"
 #include "ompi/tools/ompi_info/ompi_info.h"
 #include "ompi/runtime/ompi_info_support.h"
@@ -103,8 +96,8 @@ int main(int argc, char *argv[])
     }
 
     if (opal_cmd_line_is_taken(ompi_info_cmd_line, "version")) {
-        fprintf(stdout, "Open MPI v%s\n\n%s\n",
-                OPAL_VERSION, PACKAGE_BUGREPORT);
+        fprintf(stdout, "%s v%s\n\n%s\n",
+                PACKAGE_NAME, OPAL_VERSION, PACKAGE_BUGREPORT);
         exit(0);
     }
 
@@ -115,11 +108,6 @@ int main(int argc, char *argv[])
     /* add in the opal frameworks */
     opal_info_register_types(&mca_types);
 
-#if OMPI_RTE_ORTE
-    /* add in the orte frameworks */
-    orte_info_register_types(&mca_types);
-#endif
-
     ompi_info_register_types(&mca_types);
 
     /* init the component map */
@@ -176,7 +164,6 @@ int main(int argc, char *argv[])
         ompi_info_show_ompi_version(opal_info_ver_full);
         opal_info_show_path(opal_info_path_prefix, opal_install_dirs.prefix);
         opal_info_do_arch();
-        opal_info_do_hostname();
         ompi_info_do_config(false);
         opal_info_show_component_version(&mca_types, &component_map, opal_info_type_all,
                                          opal_info_component_all, opal_info_ver_full,
