@@ -23,9 +23,9 @@
 
 #include "opal_config.h"
 
+#include "opal/class/opal_graph.h"
 #include "opal/class/opal_list.h"
 #include "opal/constants.h"
-#include "opal/class/opal_graph.h"
 #include "opal/util/output.h"
 
 static int compare_vertex_distance(const void *item1, const void *item2);
@@ -34,49 +34,28 @@ static int compare_vertex_distance(const void *item1, const void *item2);
  *  Graph classes
  */
 
-
 static void opal_graph_vertex_construct(opal_graph_vertex_t *vertex);
 static void opal_graph_vertex_destruct(opal_graph_vertex_t *vertex);
 
-OBJ_CLASS_INSTANCE(
-    opal_graph_vertex_t,
-    opal_list_item_t,
-    opal_graph_vertex_construct,
-    opal_graph_vertex_destruct
-);
-
+OBJ_CLASS_INSTANCE(opal_graph_vertex_t, opal_list_item_t, opal_graph_vertex_construct,
+                   opal_graph_vertex_destruct);
 
 static void opal_graph_edge_construct(opal_graph_edge_t *edge);
 static void opal_graph_edge_destruct(opal_graph_edge_t *edge);
 
-OBJ_CLASS_INSTANCE(
-    opal_graph_edge_t,
-    opal_list_item_t,
-    opal_graph_edge_construct,
-    opal_graph_edge_destruct
-);
+OBJ_CLASS_INSTANCE(opal_graph_edge_t, opal_list_item_t, opal_graph_edge_construct,
+                   opal_graph_edge_destruct);
 
 static void opal_graph_construct(opal_graph_t *graph);
 static void opal_graph_destruct(opal_graph_t *graph);
 
-OBJ_CLASS_INSTANCE(
-    opal_graph_t,
-    opal_object_t,
-    opal_graph_construct,
-    opal_graph_destruct
-);
+OBJ_CLASS_INSTANCE(opal_graph_t, opal_object_t, opal_graph_construct, opal_graph_destruct);
 
 static void opal_adjacency_list_construct(opal_adjacency_list_t *aj_list);
 static void opal_adjacency_list_destruct(opal_adjacency_list_t *aj_list);
 
-OBJ_CLASS_INSTANCE(
-    opal_adjacency_list_t,
-    opal_list_item_t,
-    opal_adjacency_list_construct,
-    opal_adjacency_list_destruct
-);
-
-
+OBJ_CLASS_INSTANCE(opal_adjacency_list_t, opal_list_item_t, opal_adjacency_list_construct,
+                   opal_adjacency_list_destruct);
 
 /*
  *
@@ -112,7 +91,6 @@ static void opal_graph_vertex_destruct(opal_graph_vertex_t *vertex)
     vertex->print_vertex = NULL;
 }
 
-
 /*
  *
  *      opal_graph_edge_t interface
@@ -127,7 +105,6 @@ static void opal_graph_edge_construct(opal_graph_edge_t *edge)
     edge->in_adj_list = NULL;
 }
 
-
 static void opal_graph_edge_destruct(opal_graph_edge_t *edge)
 {
     edge->end = NULL;
@@ -136,14 +113,12 @@ static void opal_graph_edge_destruct(opal_graph_edge_t *edge)
     edge->in_adj_list = NULL;
 }
 
-
 /*
  *
  *     opal_graph_t  interface
  *
  */
 
-
 static void opal_graph_construct(opal_graph_t *graph)
 {
     graph->adjacency_list = OBJ_NEW(opal_list_t);
@@ -172,8 +147,8 @@ static void opal_adjacency_list_construct(opal_adjacency_list_t *aj_list)
 
 static void opal_adjacency_list_destruct(opal_adjacency_list_t *aj_list)
 {
-   aj_list->vertex = NULL;
-   OPAL_LIST_RELEASE(aj_list->edges);
+    aj_list->vertex = NULL;
+    OPAL_LIST_RELEASE(aj_list->edges);
 }
 
 /**
@@ -191,17 +166,17 @@ static void delete_all_edges_conceded_to_vertex(opal_graph_t *graph, opal_graph_
     /**
      * for all the adjacency list in the graph
      */
-    OPAL_LIST_FOREACH(aj_list, graph->adjacency_list, opal_adjacency_list_t) {
+    OPAL_LIST_FOREACH (aj_list, graph->adjacency_list, opal_adjacency_list_t) {
         /**
          * for all the edges in the adjacency list
          */
-        OPAL_LIST_FOREACH_SAFE(edge, next, aj_list->edges, opal_graph_edge_t) {
+        OPAL_LIST_FOREACH_SAFE (edge, next, aj_list->edges, opal_graph_edge_t) {
             /**
              * if the edge is ended in the vertex
              */
             if (edge->end == vertex) {
                 /* Delete this edge  */
-                opal_list_remove_item(edge->in_adj_list->edges, (opal_list_item_t*)edge);
+                opal_list_remove_item(edge->in_adj_list->edges, (opal_list_item_t *) edge);
                 /* distract this edge */
                 OBJ_RELEASE(edge);
             }
@@ -218,31 +193,30 @@ static void delete_all_edges_conceded_to_vertex(opal_graph_t *graph, opal_graph_
  */
 void opal_graph_add_vertex(opal_graph_t *graph, opal_graph_vertex_t *vertex)
 {
-   opal_adjacency_list_t *aj_list;
+    opal_adjacency_list_t *aj_list;
 
-   /**
-    * Find if this vertex already exists in the graph.
-    */
-   OPAL_LIST_FOREACH(aj_list, graph->adjacency_list, opal_adjacency_list_t) {
-       if (aj_list->vertex == vertex) {
-           /* If this vertex exists, dont do anything. */
-           return;
-       }
-   }
-   /* Construct a new adjacency list */
-   aj_list = OBJ_NEW(opal_adjacency_list_t);
-   aj_list->vertex = vertex;
-   /* point the vertex to the adjacency list of the vertex (for easy searching) */
-   vertex->in_adj_list = aj_list;
-   /* Append the new creates adjacency list to the graph */
-   opal_list_append(graph->adjacency_list, (opal_list_item_t*)aj_list);
-   /* point the vertex to the graph it belongs to (mostly for debug uses)*/
-   vertex->in_graph = graph;
-   /* increase the number of vertices in the graph */
-   graph->number_of_vertices++;
+    /**
+     * Find if this vertex already exists in the graph.
+     */
+    OPAL_LIST_FOREACH (aj_list, graph->adjacency_list, opal_adjacency_list_t) {
+        if (aj_list->vertex == vertex) {
+            /* If this vertex exists, dont do anything. */
+            return;
+        }
+    }
+    /* Construct a new adjacency list */
+    aj_list = OBJ_NEW(opal_adjacency_list_t);
+    aj_list->vertex = vertex;
+    /* point the vertex to the adjacency list of the vertex (for easy searching) */
+    vertex->in_adj_list = aj_list;
+    /* Append the new creates adjacency list to the graph */
+    opal_list_append(graph->adjacency_list, (opal_list_item_t *) aj_list);
+    /* point the vertex to the graph it belongs to (mostly for debug uses)*/
+    vertex->in_graph = graph;
+    /* increase the number of vertices in the graph */
+    graph->number_of_vertices++;
 }
 
-
 /**
  * This graph API adds an edge (connection between two
  * vertices) to a graph. The most common use
@@ -256,14 +230,13 @@ void opal_graph_add_vertex(opal_graph_t *graph, opal_graph_vertex_t *vertex)
  */
 int opal_graph_add_edge(opal_graph_t *graph, opal_graph_edge_t *edge)
 {
-    opal_adjacency_list_t *aj_list, *start_aj_list= NULL;
+    opal_adjacency_list_t *aj_list, *start_aj_list = NULL;
     bool end_found = false;
 
-
     /**
      * find the vertices that this edge should connect.
      */
-    OPAL_LIST_FOREACH(aj_list, graph->adjacency_list, opal_adjacency_list_t) {
+    OPAL_LIST_FOREACH (aj_list, graph->adjacency_list, opal_adjacency_list_t) {
         if (aj_list->vertex == edge->start) {
             start_aj_list = aj_list;
         }
@@ -279,15 +252,14 @@ int opal_graph_add_edge(opal_graph_t *graph, opal_graph_edge_t *edge)
         return OPAL_ERROR;
     }
     /* point the edge to the adjacency list of the start vertex (for easy search) */
-    edge->in_adj_list=start_aj_list;
+    edge->in_adj_list = start_aj_list;
     /* append the edge to the adjacency list of the start vertex */
-    opal_list_append(start_aj_list->edges, (opal_list_item_t*)edge);
+    opal_list_append(start_aj_list->edges, (opal_list_item_t *) edge);
     /* increase the graph size */
     graph->number_of_edges++;
     return OPAL_SUCCESS;
 }
 
-
 /**
  * This graph API removes an edge (a connection between two
  * vertices) from the graph. The most common use of this API is
@@ -299,10 +271,10 @@ int opal_graph_add_edge(opal_graph_t *graph, opal_graph_edge_t *edge)
  * @param graph The graph that this edge will be remove from.
  * @param edge the edge that we want to remove.
  */
-void opal_graph_remove_edge (opal_graph_t *graph, opal_graph_edge_t *edge)
+void opal_graph_remove_edge(opal_graph_t *graph, opal_graph_edge_t *edge)
 {
     /* remove the edge from the list it belongs to */
-    opal_list_remove_item(edge->in_adj_list->edges, (opal_list_item_t*)edge);
+    opal_list_remove_item(edge->in_adj_list->edges, (opal_list_item_t *) edge);
     /* decrees the number of edges in the graph */
     graph->number_of_edges--;
     /* Note that the edge is not destructed - the caller should destruct this edge. */
@@ -328,7 +300,7 @@ void opal_graph_remove_vertex(opal_graph_t *graph, opal_graph_vertex_t *vertex)
      * remove the adjscency list of this vertex from the graph and
      * destruct it.
      */
-    opal_list_remove_item(graph->adjacency_list, (opal_list_item_t*)adj_list);
+    opal_list_remove_item(graph->adjacency_list, (opal_list_item_t *) adj_list);
     OBJ_RELEASE(adj_list);
     /**
      * delete all the edges that connected *to* the vertex.
@@ -351,7 +323,8 @@ void opal_graph_remove_vertex(opal_graph_t *graph, opal_graph_vertex_t *vertex)
  *         vertices or infinity if the vertices are not
  *         connected.
  */
-uint32_t opal_graph_adjacent(opal_graph_t *graph, opal_graph_vertex_t *vertex1, opal_graph_vertex_t *vertex2)
+uint32_t opal_graph_adjacent(opal_graph_t *graph, opal_graph_vertex_t *vertex1,
+                             opal_graph_vertex_t *vertex2)
 {
     opal_adjacency_list_t *adj_list;
     opal_graph_edge_t *edge;
@@ -360,14 +333,16 @@ uint32_t opal_graph_adjacent(opal_graph_t *graph, opal_graph_vertex_t *vertex1,
      * Verify that the first vertex belongs to the graph.
      */
     if (graph != vertex1->in_graph) {
-        OPAL_OUTPUT((0,"opal_graph_adjacent 1 Vertex1 %p not in the graph %p\n",(void *)vertex1,(void *)graph));
+        OPAL_OUTPUT((0, "opal_graph_adjacent 1 Vertex1 %p not in the graph %p\n", (void *) vertex1,
+                     (void *) graph));
         return DISTANCE_INFINITY;
     }
     /**
      * Verify that the second vertex belongs to the graph.
      */
     if (graph != vertex2->in_graph) {
-        OPAL_OUTPUT((0,"opal_graph_adjacent 2 Vertex2 %p not in the graph %p\n",(void *)vertex2,(void *)graph));
+        OPAL_OUTPUT((0, "opal_graph_adjacent 2 Vertex2 %p not in the graph %p\n", (void *) vertex2,
+                     (void *) graph));
         return DISTANCE_INFINITY;
     }
     /**
@@ -382,9 +357,10 @@ uint32_t opal_graph_adjacent(opal_graph_t *graph, opal_graph_vertex_t *vertex1,
      * vertex.
      */
     adj_list = (opal_adjacency_list_t *) vertex1->in_adj_list;
-    OPAL_LIST_FOREACH(edge, adj_list->edges, opal_graph_edge_t) {
+    OPAL_LIST_FOREACH (edge, adj_list->edges, opal_graph_edge_t) {
         if (edge->end == vertex2) {
-            /* if the second vertex was found in the adjacency list of the first one, return the weight */
+            /* if the second vertex was found in the adjacency list of the first one, return the
+             * weight */
             return edge->weight;
         }
     }
@@ -434,7 +410,7 @@ opal_graph_vertex_t *opal_graph_find_vertex(opal_graph_t *graph, void *vertex_da
     /**
      * Run on all the vertices of the graph
      */
-    OPAL_LIST_FOREACH(aj_list, graph->adjacency_list, opal_adjacency_list_t) {
+    OPAL_LIST_FOREACH (aj_list, graph->adjacency_list, opal_adjacency_list_t) {
         if (NULL != aj_list->vertex->compare_vertex) {
             /* if the vertex data of a vertex is equal to the vertex data */
             if (0 == aj_list->vertex->compare_vertex(aj_list->vertex->vertex_data, vertex_data)) {
@@ -447,7 +423,6 @@ opal_graph_vertex_t *opal_graph_find_vertex(opal_graph_t *graph, void *vertex_da
     return NULL;
 }
 
-
 /**
  * This graph API returns an array of pointers of all the
  * vertices in the graph.
@@ -471,9 +446,9 @@ int opal_graph_get_graph_vertices(opal_graph_t *graph, opal_pointer_array_t *ver
         return 0;
     }
     /* Run on all the vertices of the graph */
-    OPAL_LIST_FOREACH(aj_list, graph->adjacency_list, opal_adjacency_list_t) {
+    OPAL_LIST_FOREACH (aj_list, graph->adjacency_list, opal_adjacency_list_t) {
         /* Add the vertex to the vertices array */
-        opal_pointer_array_add(vertices_list,(void *)aj_list->vertex);
+        opal_pointer_array_add(vertices_list, (void *) aj_list->vertex);
     }
     /* return the vertices list */
     return graph->number_of_vertices;
@@ -492,7 +467,8 @@ int opal_graph_get_graph_vertices(opal_graph_t *graph, opal_pointer_array_t *ver
  *
  * @return int the number of adjacents in the list.
  */
-int opal_graph_get_adjacent_vertices(opal_graph_t *graph, opal_graph_vertex_t *vertex, opal_value_array_t *adjacents)
+int opal_graph_get_adjacent_vertices(opal_graph_t *graph, opal_graph_vertex_t *vertex,
+                                     opal_value_array_t *adjacents)
 {
     opal_adjacency_list_t *adj_list;
     opal_graph_edge_t *edge;
@@ -503,7 +479,7 @@ int opal_graph_get_adjacent_vertices(opal_graph_t *graph, opal_graph_vertex_t *v
      * Verify that the vertex belongs to the graph.
      */
     if (graph != vertex->in_graph) {
-        OPAL_OUTPUT((0,"Vertex %p not in the graph %p\n", (void *)vertex, (void *)graph));
+        OPAL_OUTPUT((0, "Vertex %p not in the graph %p\n", (void *) vertex, (void *) graph));
         return 0;
     }
     /**
@@ -513,7 +489,7 @@ int opal_graph_get_adjacent_vertices(opal_graph_t *graph, opal_graph_vertex_t *v
     /* find the number of adjcents of this vertex */
     adjacents_number = opal_list_get_size(adj_list->edges);
     /* Run on all the edges from this vertex */
-    OPAL_LIST_FOREACH(edge, adj_list->edges, opal_graph_edge_t) {
+    OPAL_LIST_FOREACH (edge, adj_list->edges, opal_graph_edge_t) {
         /* assign vertices and their weight in the adjcents list */
         distance_from.vertex = edge->end;
         distance_from.weight = edge->weight;
@@ -533,7 +509,8 @@ int opal_graph_get_adjacent_vertices(opal_graph_t *graph, opal_graph_vertex_t *v
  * @return uint32_t the distance between the two vertices.
  */
 
-uint32_t opal_graph_spf(opal_graph_t *graph, opal_graph_vertex_t *vertex1, opal_graph_vertex_t *vertex2)
+uint32_t opal_graph_spf(opal_graph_t *graph, opal_graph_vertex_t *vertex1,
+                        opal_graph_vertex_t *vertex2)
 {
     opal_value_array_t *distance_array;
     uint32_t items_in_distance_array, spf = DISTANCE_INFINITY;
@@ -544,14 +521,16 @@ uint32_t opal_graph_spf(opal_graph_t *graph, opal_graph_vertex_t *vertex1, opal_
      * Verify that the first vertex belongs to the graph.
      */
     if (graph != vertex1->in_graph) {
-        OPAL_OUTPUT((0,"opal_graph_spf 1 Vertex1 %p not in the graph %p\n",(void *)vertex1,(void *)graph));
+        OPAL_OUTPUT((0, "opal_graph_spf 1 Vertex1 %p not in the graph %p\n", (void *) vertex1,
+                     (void *) graph));
         return DISTANCE_INFINITY;
     }
     /**
      * Verify that the second vertex belongs to the graph.
      */
     if (graph != vertex2->in_graph) {
-        OPAL_OUTPUT((0,"opal_graph_spf 2 Vertex2 %p not in the graph %p\n",(void *)vertex2,(void *)graph));
+        OPAL_OUTPUT((0, "opal_graph_spf 2 Vertex2 %p not in the graph %p\n", (void *) vertex2,
+                     (void *) graph));
         return DISTANCE_INFINITY;
     }
     /**
@@ -559,7 +538,7 @@ uint32_t opal_graph_spf(opal_graph_t *graph, opal_graph_vertex_t *vertex1, opal_
      */
     distance_array = OBJ_NEW(opal_value_array_t);
     opal_value_array_init(distance_array, sizeof(vertex_distance_from_t));
-    opal_value_array_reserve(distance_array,50);
+    opal_value_array_reserve(distance_array, 50);
     items_in_distance_array = opal_graph_dijkstra(graph, vertex1, distance_array);
     /**
      * find the end vertex in the distance array that Dijkstra
@@ -595,8 +574,8 @@ static int compare_vertex_distance(const void *item1, const void *item2)
     vertex_distance_from_t *vertex_dist1, *vertex_dist2;
 
     /* convert the void pointers to vertex distance pointers. */
-    vertex_dist1 = (vertex_distance_from_t *)item1;
-    vertex_dist2 = (vertex_distance_from_t *)item2;
+    vertex_dist1 = (vertex_distance_from_t *) item1;
+    vertex_dist2 = (vertex_distance_from_t *) item2;
 
     /* If the first item weight is higher then the second item weight return 1*/
     if (vertex_dist1->weight > vertex_dist2->weight) {
@@ -610,7 +589,6 @@ static int compare_vertex_distance(const void *item1, const void *item2)
     return -1;
 }
 
-
 /**
  * This graph API returns the distance (weight) from a reference
  * vertex to all other vertices in the graph using the Dijkstra
@@ -623,7 +601,8 @@ static int compare_vertex_distance(const void *item1, const void *item2)
  *
  * @return uint32_t the size of the distance array
  */
-uint32_t opal_graph_dijkstra(opal_graph_t *graph, opal_graph_vertex_t *vertex, opal_value_array_t *distance_array)
+uint32_t opal_graph_dijkstra(opal_graph_t *graph, opal_graph_vertex_t *vertex,
+                             opal_value_array_t *distance_array)
 {
     int graph_order;
     vertex_distance_from_t *Q, *q_start, *current_vertex;
@@ -632,23 +611,23 @@ uint32_t opal_graph_dijkstra(opal_graph_t *graph, opal_graph_vertex_t *vertex, o
     int i;
     uint32_t weight;
 
-
     /**
      * Verify that the reference vertex belongs to the graph.
      */
     if (graph != vertex->in_graph) {
-        OPAL_OUTPUT((0,"opal:graph:dijkstra: vertex %p not in the graph %p\n",(void *)vertex,(void *)graph));
+        OPAL_OUTPUT((0, "opal:graph:dijkstra: vertex %p not in the graph %p\n", (void *) vertex,
+                     (void *) graph));
         return 0;
     }
     /* get the order of the graph and allocate a working queue accordingly */
     graph_order = opal_graph_get_order(graph);
-    Q = (vertex_distance_from_t *)malloc(graph_order * sizeof(vertex_distance_from_t));
+    Q = (vertex_distance_from_t *) malloc(graph_order * sizeof(vertex_distance_from_t));
     /* assign a pointer to the start of the queue */
     q_start = Q;
     /* run on all the vertices of the graph */
     i = 0;
-    OPAL_LIST_FOREACH(adj_list, graph->adjacency_list, opal_adjacency_list_t) {
-        /* insert the vertices pointes to the working queue */
+    OPAL_LIST_FOREACH (adj_list, graph->adjacency_list, opal_adjacency_list_t) {
+        /* insert the vertices points to the working queue */
         Q[i].vertex = adj_list->vertex;
         /**
          * assign an infinity distance to all the vertices in the queue
@@ -667,7 +646,8 @@ uint32_t opal_graph_dijkstra(opal_graph_t *graph, opal_graph_vertex_t *vertex, o
         q_start++;
         /* decrees the number of vertices in the queue */
         number_of_items_in_q--;
-        /* find the distance of all other vertices in the queue from the first vertex in the queue */
+        /* find the distance of all other vertices in the queue from the first vertex in the queue
+         */
         for (i = 0; i < number_of_items_in_q; i++) {
             weight = opal_graph_adjacent(graph, current_vertex->vertex, q_start[i].vertex);
             /**
@@ -682,11 +662,12 @@ uint32_t opal_graph_dijkstra(opal_graph_t *graph, opal_graph_vertex_t *vertex, o
             }
         }
         /* sort again the working queue */
-        qsort(q_start, number_of_items_in_q, sizeof(vertex_distance_from_t), compare_vertex_distance);
+        qsort(q_start, number_of_items_in_q, sizeof(vertex_distance_from_t),
+              compare_vertex_distance);
     }
     /* copy the working queue the the returned distance array */
-    for (i = 0; i < graph_order-1; i++) {
-        opal_value_array_append_item(distance_array, (void *)&(Q[i+1]));
+    for (i = 0; i < graph_order - 1; i++) {
+        opal_value_array_append_item(distance_array, (void *) &(Q[i + 1]));
     }
     /* free the working queue */
     free(Q);
@@ -694,10 +675,9 @@ uint32_t opal_graph_dijkstra(opal_graph_t *graph, opal_graph_vertex_t *vertex, o
     return graph_order - 1;
 }
 
-
 /**
  * This graph API duplicates a graph. Note that this API does
- * not copy the graph but builds a new graph while coping just
+ * not copy the graph but builds a new graph while copying just
  * the vertex data.
  *
  * @param dest The new created graph.
@@ -712,7 +692,7 @@ void opal_graph_duplicate(opal_graph_t **dest, opal_graph_t *src)
     /* construct a new graph */
     *dest = OBJ_NEW(opal_graph_t);
     /* Run on all the vertices of the src graph */
-    OPAL_LIST_FOREACH(aj_list, src->adjacency_list, opal_adjacency_list_t) {
+    OPAL_LIST_FOREACH (aj_list, src->adjacency_list, opal_adjacency_list_t) {
         /* for each vertex in the src graph, construct a new vertex */
         vertex = OBJ_NEW(opal_graph_vertex_t);
         /* associate the new vertex to a vertex from the original graph */
@@ -741,14 +721,15 @@ void opal_graph_duplicate(opal_graph_t **dest, opal_graph_t *src)
      * Now, copy all the edges from the source graph
      */
     /* Run on all the adjscency lists in the graph */
-    OPAL_LIST_FOREACH(aj_list, src->adjacency_list, opal_adjacency_list_t) {
+    OPAL_LIST_FOREACH (aj_list, src->adjacency_list, opal_adjacency_list_t) {
         /* for all the edges in the adjscency list */
-        OPAL_LIST_FOREACH(edge, aj_list->edges, opal_graph_edge_t) {
+        OPAL_LIST_FOREACH (edge, aj_list->edges, opal_graph_edge_t) {
             /* construct new edge for the new graph */
             new_edge = OBJ_NEW(opal_graph_edge_t);
             /* copy the edge weight from the original edge */
             new_edge->weight = edge->weight;
-            /* connect the new edge according to start and end associations to the vertices of the src graph */
+            /* connect the new edge according to start and end associations to the vertices of the
+             * src graph */
             new_edge->start = edge->start->sibling;
             new_edge->end = edge->end->sibling;
             /* add the new edge to the new graph */
@@ -772,31 +753,29 @@ void opal_graph_print(opal_graph_t *graph)
     opal_output(0, "      Graph         ");
     opal_output(0, "====================");
     /* run on all the vertices of the graph */
-    OPAL_LIST_FOREACH(aj_list, graph->adjacency_list, opal_adjacency_list_t) {
+    OPAL_LIST_FOREACH (aj_list, graph->adjacency_list, opal_adjacency_list_t) {
         /* print vertex data to temporary string*/
         if (NULL != aj_list->vertex->print_vertex) {
             need_free1 = true;
             tmp_str1 = aj_list->vertex->print_vertex(aj_list->vertex->vertex_data);
-        }
-        else {
+        } else {
             need_free1 = false;
             tmp_str1 = "";
         }
         /* print vertex */
-        opal_output(0, "V(%s) Connections:",tmp_str1);
+        opal_output(0, "V(%s) Connections:", tmp_str1);
         /* run on all the edges of the vertex */
-        OPAL_LIST_FOREACH(edge, aj_list->edges, opal_graph_edge_t) {
+        OPAL_LIST_FOREACH (edge, aj_list->edges, opal_graph_edge_t) {
             /* print the vertex data of the vertex in the end of the edge to a temporary string */
             if (NULL != edge->end->print_vertex) {
                 need_free2 = true;
                 tmp_str2 = edge->end->print_vertex(edge->end->vertex_data);
-            }
-            else {
+            } else {
                 need_free2 = false;
                 tmp_str2 = "";
             }
             /* print the edge */
-            opal_output(0, "    E(%s -> %d -> %s)",tmp_str1, edge->weight, tmp_str2);
+            opal_output(0, "    E(%s -> %d -> %s)", tmp_str1, edge->weight, tmp_str2);
             if (need_free2) {
                 free(tmp_str2);
             }
@@ -806,4 +785,3 @@ void opal_graph_print(opal_graph_t *graph)
         }
     }
 }
-
