@@ -12,7 +12,7 @@
  * Copyright (c) 2006      Sandia National Laboratories. All rights
  *                         reserved.
  * Copyright (c) 2013-2016 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2013-2014 Intel, Inc. All rights reserved
+ * Copyright (c) 2013-2020 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -23,33 +23,32 @@
 
 #include "opal_config.h"
 
-#include "opal_stdint.h"
-#include "opal/util/arch.h"
-#include "opal/util/show_help.h"
 #include "opal/constants.h"
+#include "opal/util/arch.h"
 #include "opal/util/bipartite_graph.h"
+#include "opal/util/show_help.h"
+#include "opal/util/string_copy.h"
+#include "opal_stdint.h"
 
-#include "btl_usnic_compat.h"
 #include "btl_usnic.h"
-#include "btl_usnic_proc.h"
+#include "btl_usnic_compat.h"
 #include "btl_usnic_endpoint.h"
 #include "btl_usnic_module.h"
+#include "btl_usnic_proc.h"
 #include "btl_usnic_util.h"
 
 /* larger weight values are more desirable (i.e., worth, not cost) */
-enum {
-    WEIGHT_UNREACHABLE = -1
-};
+enum { WEIGHT_UNREACHABLE = -1 };
 
 /* Helper macros for "match_modex" and friends for translating between array
  * indices and vertex IDs.  Module vertices always come first in the graph,
  * followed by proc (endpoint) vertices. */
-#define PROC_VERTEX(modex_idx) (mca_btl_usnic_component.num_modules + modex_idx)
-#define MODULE_VERTEX(module_idx) (module_idx)
-#define PROC_INDEX(proc_vertex) ((proc_vertex) - mca_btl_usnic_component.num_modules)
+#define PROC_VERTEX(modex_idx)      (mca_btl_usnic_component.num_modules + modex_idx)
+#define MODULE_VERTEX(module_idx)   (module_idx)
+#define PROC_INDEX(proc_vertex)     ((proc_vertex) -mca_btl_usnic_component.num_modules)
 #define MODULE_INDEX(module_vertex) (module_vertex)
 
-static void proc_construct(opal_btl_usnic_proc_t* proc)
+static void proc_construct(opal_btl_usnic_proc_t *proc)
 {
     proc->proc_opal = 0;
     proc->proc_modex = NULL;
@@ -64,8 +63,7 @@ static void proc_construct(opal_btl_usnic_proc_t* proc)
     opal_list_append(&mca_btl_usnic_component.usnic_procs, &proc->super);
 }
 
-
-static void proc_destruct(opal_btl_usnic_proc_t* proc)
+static void proc_destruct(opal_btl_usnic_proc_t *proc)
 {
     /* remove from list of all proc instances */
     opal_list_remove_item(&mca_btl_usnic_component.usnic_procs, &proc->super);
@@ -93,27 +91,21 @@ static void proc_destruct(opal_btl_usnic_proc_t* proc)
     }
 }
 
-
-OBJ_CLASS_INSTANCE(opal_btl_usnic_proc_t,
-                   opal_list_item_t,
-                   proc_construct,
-                   proc_destruct);
+OBJ_CLASS_INSTANCE(opal_btl_usnic_proc_t, opal_list_item_t, proc_construct, proc_destruct);
 
 /*
  * Look for an existing usnic process instance based on the
  * associated opal_proc_t instance.
  */
-opal_btl_usnic_proc_t *
-opal_btl_usnic_proc_lookup_ompi(opal_proc_t* opal_proc)
+opal_btl_usnic_proc_t *opal_btl_usnic_proc_lookup_ompi(opal_proc_t *opal_proc)
 {
-    opal_btl_usnic_proc_t* usnic_proc;
+    opal_btl_usnic_proc_t *usnic_proc;
 
-    for (usnic_proc = (opal_btl_usnic_proc_t*)
-             opal_list_get_first(&mca_btl_usnic_component.usnic_procs);
-         usnic_proc != (opal_btl_usnic_proc_t*)
-             opal_list_get_end(&mca_btl_usnic_component.usnic_procs);
-         usnic_proc  = (opal_btl_usnic_proc_t*)
-             opal_list_get_next(usnic_proc)) {
+    for (usnic_proc = (opal_btl_usnic_proc_t *) opal_list_get_first(
+             &mca_btl_usnic_component.usnic_procs);
+         usnic_proc
+         != (opal_btl_usnic_proc_t *) opal_list_get_end(&mca_btl_usnic_component.usnic_procs);
+         usnic_proc = (opal_btl_usnic_proc_t *) opal_list_get_next(usnic_proc)) {
         if (usnic_proc->proc_opal == opal_proc) {
             return usnic_proc;
         }
@@ -122,38 +114,32 @@ opal_btl_usnic_proc_lookup_ompi(opal_proc_t* opal_proc)
     return NULL;
 }
 
-
 /*
  * Look for an existing usnic proc based on a hashed RTE process
  * name.
  */
-opal_btl_usnic_endpoint_t *
-opal_btl_usnic_proc_lookup_endpoint(opal_btl_usnic_module_t *receiver,
-                                    uint64_t sender_proc_name)
+opal_btl_usnic_endpoint_t *opal_btl_usnic_proc_lookup_endpoint(opal_btl_usnic_module_t *receiver,
+                                                               uint64_t sender_proc_name)
 {
     opal_btl_usnic_proc_t *proc;
     opal_btl_usnic_endpoint_t *endpoint;
     opal_list_item_t *item;
 
-    MSGDEBUG1_OUT("lookup_endpoint: recvmodule=%p sendhash=0x%" PRIx64,
-                  (void *)receiver, sender_proc_name);
+    MSGDEBUG1_OUT("lookup_endpoint: recvmodule=%p sendhash=0x%" PRIx64, (void *) receiver,
+                  sender_proc_name);
 
     opal_mutex_lock(&receiver->all_endpoints_lock);
     for (item = opal_list_get_first(&receiver->all_endpoints);
-         item != opal_list_get_end(&receiver->all_endpoints);
-         item = opal_list_get_next(item)) {
-        endpoint = container_of(item, opal_btl_usnic_endpoint_t,
-                                endpoint_endpoint_li);
+         item != opal_list_get_end(&receiver->all_endpoints); item = opal_list_get_next(item)) {
+        endpoint = container_of(item, opal_btl_usnic_endpoint_t, endpoint_endpoint_li);
         proc = endpoint->endpoint_proc;
         /* Note that this works today because opal_proc_t->proc_name
            is unique across the universe.  George is potentially
            working to give handles instead of proc names, and then
            have a function pointer to perform comparisons.  This would
            be bad here in the critical path, though... */
-        if (usnic_compat_rte_hash_name(&(proc->proc_opal->proc_name)) ==
-            sender_proc_name) {
-            MSGDEBUG1_OUT("lookup_endpoint: matched endpoint=%p",
-                          (void *)endpoint);
+        if (usnic_compat_rte_hash_name(&(proc->proc_opal->proc_name)) == sender_proc_name) {
+            MSGDEBUG1_OUT("lookup_endpoint: matched endpoint=%p", (void *) endpoint);
             opal_mutex_unlock(&receiver->all_endpoints_lock);
             return endpoint;
         }
@@ -171,8 +157,7 @@ opal_btl_usnic_proc_lookup_endpoint(opal_btl_usnic_module_t *receiver,
  * Returns OPAL_ERR_UNREACH if we can't reach the peer (i.e., we can't
  * find their modex data).
  */
-static int create_proc(opal_proc_t *opal_proc,
-                       opal_btl_usnic_proc_t **usnic_proc)
+static int create_proc(opal_proc_t *opal_proc, opal_btl_usnic_proc_t **usnic_proc)
 {
     opal_btl_usnic_proc_t *proc = NULL;
     size_t size;
@@ -191,8 +176,8 @@ static int create_proc(opal_proc_t *opal_proc,
     proc->proc_opal = opal_proc;
 
     /* query for the peer address info */
-    usnic_compat_modex_recv(&rc, &mca_btl_usnic_component.super.btl_version,
-                            opal_proc, &proc->proc_modex, &size);
+    usnic_compat_modex_recv(&rc, &mca_btl_usnic_component.super.btl_version, opal_proc,
+                            &proc->proc_modex, &size);
 
     /* If this proc simply doesn't have this key, then they're not
        running the usnic BTL -- just ignore them.  Otherwise, show an
@@ -201,13 +186,9 @@ static int create_proc(opal_proc_t *opal_proc,
         OBJ_RELEASE(proc);
         return OPAL_ERR_UNREACH;
     } else if (OPAL_SUCCESS != rc) {
-        opal_show_help("help-mpi-btl-usnic.txt",
-                       "internal error during init",
-                       true,
-                       opal_process_info.nodename,
-                       "<none>", "<none>",
-                       "opal_modex_recv() failed", __FILE__, __LINE__,
-                       opal_strerror(rc));
+        opal_show_help("help-mpi-btl-usnic.txt", "internal error during init", true,
+                       opal_process_info.nodename, "<none>", "<none>", "opal_modex_recv() failed",
+                       __FILE__, __LINE__, opal_strerror(rc));
         OBJ_RELEASE(proc);
         return OPAL_ERROR;
     }
@@ -215,16 +196,12 @@ static int create_proc(opal_proc_t *opal_proc,
     if ((size % sizeof(opal_btl_usnic_modex_t)) != 0) {
         char msg[1024];
 
-        snprintf(msg, sizeof(msg),
-                 "sizeof(modex for peer %s data) == %d, expected multiple of %d",
-                 usnic_compat_proc_name_print(&opal_proc->proc_name),
-                 (int) size, (int) sizeof(opal_btl_usnic_modex_t));
-        opal_show_help("help-mpi-btl-usnic.txt", "internal error during init",
-                       true,
-                       opal_process_info.nodename,
-                       "<none>", 0,
-                       "invalid modex data", __FILE__, __LINE__,
-                       msg);
+        snprintf(msg, sizeof(msg), "sizeof(modex for peer %s data) == %d, expected multiple of %d",
+                 usnic_compat_proc_name_print(&opal_proc->proc_name), (int) size,
+                 (int) sizeof(opal_btl_usnic_modex_t));
+        opal_show_help("help-mpi-btl-usnic.txt", "internal error during init", true,
+                       opal_process_info.nodename, "<none>", 0, "invalid modex data", __FILE__,
+                       __LINE__, msg);
 
         OBJ_RELEASE(proc);
         return OPAL_ERR_VALUE_OUT_OF_BOUNDS;
@@ -232,22 +209,16 @@ static int create_proc(opal_proc_t *opal_proc,
 
     /* See if the peer has the same underlying wire protocol as me.
        If not, then print an error and ignore this peer. */
-// RFXXX - things are weird when i force this to fail
-    if (mca_btl_usnic_component.transport_protocol !=
-        proc->proc_modex->protocol) {
+    // RFXXX - things are weird when i force this to fail
+    if (mca_btl_usnic_component.transport_protocol != proc->proc_modex->protocol) {
         uint64_t proto;
         char protostr[32];
         proto = mca_btl_usnic_component.transport_protocol;
         memset(protostr, 0, sizeof(protostr));
-        strncpy(protostr, fi_tostr(&proto, FI_TYPE_PROTOCOL),
-                sizeof(protostr) - 1);
+        opal_string_copy(protostr, fi_tostr(&proto, FI_TYPE_PROTOCOL), sizeof(protostr));
         proto = proc->proc_modex->protocol;
-        opal_show_help("help-mpi-btl-usnic.txt",
-                       "transport mismatch",
-                       true,
-                       opal_process_info.nodename,
-                       protostr,
-                       "peer",
+        opal_show_help("help-mpi-btl-usnic.txt", "transport mismatch", true,
+                       opal_process_info.nodename, protostr, "peer",
                        fi_tostr(&proto, FI_TYPE_PROTOCOL));
 
         OBJ_RELEASE(proc);
@@ -261,16 +232,15 @@ static int create_proc(opal_proc_t *opal_proc,
         return OPAL_ERR_UNREACH;
     }
 
-    proc->proc_modex_claimed = (bool*)
-        calloc(proc->proc_modex_count, sizeof(bool));
+    proc->proc_modex_claimed = (bool *) calloc(proc->proc_modex_count, sizeof(bool));
     if (NULL == proc->proc_modex_claimed) {
         OPAL_ERROR_LOG(OPAL_ERR_OUT_OF_RESOURCE);
         OBJ_RELEASE(proc);
         return OPAL_ERR_OUT_OF_RESOURCE;
     }
 
-    proc->proc_endpoints = (mca_btl_base_endpoint_t**)
-        calloc(proc->proc_modex_count, sizeof(mca_btl_base_endpoint_t*));
+    proc->proc_endpoints = (mca_btl_base_endpoint_t **) calloc(proc->proc_modex_count,
+                                                               sizeof(mca_btl_base_endpoint_t *));
     if (NULL == proc->proc_endpoints) {
         OPAL_ERROR_LOG(OPAL_ERR_OUT_OF_RESOURCE);
         OBJ_RELEASE(proc);
@@ -284,9 +254,8 @@ static int create_proc(opal_proc_t *opal_proc,
 /* Compare the addresses of the local interface corresponding to module and the
  * remote interface corresponding to proc_modex_addr.  Returns a weight value
  * (higher values indicate more desirable connections). */
-static uint64_t compute_weight(
-    opal_btl_usnic_module_t *module,
-    opal_btl_usnic_modex_t *proc_modex_addr)
+static uint64_t compute_weight(opal_btl_usnic_module_t *module,
+                               opal_btl_usnic_modex_t *proc_modex_addr)
 {
     char my_ip_string[INET_ADDRSTRLEN], peer_ip_string[INET_ADDRSTRLEN];
     struct sockaddr_in sin;
@@ -299,10 +268,8 @@ static uint64_t compute_weight(
 
     uip = &module->usnic_info;
     sinp = module->fabric_info->src_addr;
-    inet_ntop(AF_INET, &sinp->sin_addr,
-              my_ip_string, sizeof(my_ip_string));
-    inet_ntop(AF_INET, &proc_modex_addr->ipv4_addr,
-              peer_ip_string, sizeof(peer_ip_string));
+    inet_ntop(AF_INET, &sinp->sin_addr, my_ip_string, sizeof(my_ip_string));
+    inet_ntop(AF_INET, &proc_modex_addr->ipv4_addr, peer_ip_string, sizeof(peer_ip_string));
 
     /* Just compare the CIDR-masked IP address to see if they're on
        the same network.  If so, we're good. */
@@ -310,14 +277,11 @@ static uint64_t compute_weight(
     peernet = proc_modex_addr->ipv4_addr & proc_modex_addr->netmask;
     opal_output_verbose(5, USNIC_OUT,
                         "btl:usnic:%s: checking my IP address/subnet (%s/%d) vs. peer (%s/%d): %s",
-                        __func__, my_ip_string,
-                        usnic_netmask_to_cidrlen(uip->ui.v1.ui_netmask_be),
-                        peer_ip_string,
-                        usnic_netmask_to_cidrlen(proc_modex_addr->netmask),
+                        __func__, my_ip_string, usnic_netmask_to_cidrlen(uip->ui.v1.ui_netmask_be),
+                        peer_ip_string, usnic_netmask_to_cidrlen(proc_modex_addr->netmask),
                         (mynet == peernet ? "match" : "DO NOT match"));
 
-    min_link_speed_gbps = MIN(module->super.btl_bandwidth,
-                              proc_modex_addr->link_speed_mbps) / 1000;
+    min_link_speed_gbps = MIN(module->super.btl_bandwidth, proc_modex_addr->link_speed_mbps) / 1000;
 
     /* Returned metric is:
      *    0 - same VLAN
@@ -331,8 +295,7 @@ static uint64_t compute_weight(
     err = module->usnic_av_ops->get_distance(module->av, &sin, &metric);
     if (0 != err || (0 == err && -1 == metric)) {
         return 0; /* no connectivity */
-    }
-    else {
+    } else {
         /* Format in binary    MSB                             LSB
          * most sig. 32-bits:  00000000 0000000A BBBBBBBB 00000001
          * least sig. 32-bits: CCCCCCCC CCCCCCCC CCCCCCCC CCCCCCCC
@@ -358,10 +321,8 @@ static uint64_t compute_weight(
                                 min_link_speed_gbps);
             min_link_speed_gbps = 0xff;
         }
-        return ((uint64_t)(mynet == peernet) << 48) |
-               ((uint64_t)(min_link_speed_gbps & 0xff) << 40) |
-               ((uint64_t)0x1 << 32) |
-               (/*metric=*/0);
+        return ((uint64_t)(mynet == peernet) << 48) | ((uint64_t)(min_link_speed_gbps & 0xff) << 40)
+               | ((uint64_t) 0x1 << 32) | (/*metric=*/0);
     }
 }
 
@@ -370,23 +331,20 @@ static uint64_t compute_weight(
  * (DJG: this unfortunately knows a bit too much about the internals of
  * "match_modex")
  */
-static void edge_pairs_to_match_table(
-    opal_btl_usnic_proc_t *proc,
-    bool proc_is_left,
-    int nme,
-    int *me)
+static void edge_pairs_to_match_table(opal_btl_usnic_proc_t *proc, bool proc_is_left, int nme,
+                                      int *me)
 {
     int i;
     int left, right;
     int module_idx, proc_idx;
     int num_modules;
 
-    num_modules = (int)mca_btl_usnic_component.num_modules;
+    num_modules = (int) mca_btl_usnic_component.num_modules;
 
     assert(nme >= 0);
     for (i = 0; i < nme; ++i) {
-        left  = me[2*i+0];
-        right = me[2*i+1];
+        left = me[2 * i + 0];
+        right = me[2 * i + 1];
 
         if (proc_is_left) {
             proc_idx = PROC_INDEX(left);
@@ -396,7 +354,7 @@ static void edge_pairs_to_match_table(
             proc_idx = PROC_INDEX(right);
         }
         assert(module_idx >= 0 && module_idx < num_modules);
-        assert(proc_idx >= 0 && proc_idx < (int)proc->proc_modex_count);
+        assert(proc_idx >= 0 && proc_idx < (int) proc->proc_modex_count);
         proc->proc_ep_match_table[module_idx] = proc_idx;
         proc->proc_match_exists = true;
     }
@@ -404,17 +362,15 @@ static void edge_pairs_to_match_table(
     /* emit match summary for debugging purposes */
     for (i = 0; i < num_modules; ++i) {
         if (-1 != proc->proc_ep_match_table[i]) {
-            opal_output_verbose(5, USNIC_OUT,
-                                "btl:usnic:%s: module[%d] (%p) should claim endpoint[%d] on proc %p",
-                                __func__, i,
-                                (void *)mca_btl_usnic_component.usnic_active_modules[i],
-                                proc->proc_ep_match_table[i], (void *)proc);
+            opal_output_verbose(
+                5, USNIC_OUT, "btl:usnic:%s: module[%d] (%p) should claim endpoint[%d] on proc %p",
+                __func__, i, (void *) mca_btl_usnic_component.usnic_active_modules[i],
+                proc->proc_ep_match_table[i], (void *) proc);
         } else {
-            opal_output_verbose(5, USNIC_OUT,
-                                "btl:usnic:%s: module[%d] (%p) will NOT claim an endpoint on proc %p",
-                                __func__, i,
-                                (void *)mca_btl_usnic_component.usnic_active_modules[i],
-                                (void *)proc);
+            opal_output_verbose(
+                5, USNIC_OUT, "btl:usnic:%s: module[%d] (%p) will NOT claim an endpoint on proc %p",
+                __func__, i, (void *) mca_btl_usnic_component.usnic_active_modules[i],
+                (void *) proc);
         }
     }
 }
@@ -424,10 +380,8 @@ static void edge_pairs_to_match_table(
  * remote interfaces.  The resulting vertices will always have the module
  * vertices appear before the proc vertices.
  */
-static int create_proc_module_graph(
-    opal_btl_usnic_proc_t *proc,
-    bool proc_is_left,
-    opal_bp_graph_t **g_out)
+static int create_proc_module_graph(opal_btl_usnic_proc_t *proc, bool proc_is_left,
+                                    opal_bp_graph_t **g_out)
 {
     int err;
     int i, j;
@@ -440,7 +394,7 @@ static int create_proc_module_graph(
     }
     *g_out = NULL;
 
-    num_modules = (int)mca_btl_usnic_component.num_modules;
+    num_modules = (int) mca_btl_usnic_component.num_modules;
 
     /* Construct a bipartite graph with remote interfaces on the one side and
      * local interfaces (modules) on the other. */
@@ -453,28 +407,26 @@ static int create_proc_module_graph(
     /* create vertices for each interface (local and remote) */
     for (i = 0; i < num_modules; ++i) {
         int idx = -1;
-        err = opal_bp_graph_add_vertex(g,
-				       mca_btl_usnic_component.usnic_active_modules[i],
-				       &idx);
+        err = opal_bp_graph_add_vertex(g, mca_btl_usnic_component.usnic_active_modules[i], &idx);
         if (OPAL_SUCCESS != err) {
             OPAL_ERROR_LOG(err);
             goto out_free_graph;
         }
         assert(idx == MODULE_VERTEX(i));
     }
-    for (i = 0; i < (int)proc->proc_modex_count; ++i) {
+    for (i = 0; i < (int) proc->proc_modex_count; ++i) {
         int idx = -1;
         err = opal_bp_graph_add_vertex(g, &proc->proc_modex[i], &idx);
         if (OPAL_SUCCESS != err) {
             OPAL_ERROR_LOG(err);
             goto out_free_graph;
         }
-        assert(idx == (int)PROC_VERTEX(i));
+        assert(idx == (int) PROC_VERTEX(i));
     }
 
     /* now add edges between interfaces that can communicate */
     for (i = 0; i < num_modules; ++i) {
-        for (j = 0; j < (int)proc->proc_modex_count; ++j) {
+        for (j = 0; j < (int) proc->proc_modex_count; ++j) {
             int64_t weight, cost;
 
             /* assumption: compute_weight returns the same weight on the
@@ -483,11 +435,11 @@ static int create_proc_module_graph(
                                     &proc->proc_modex[j]);
 
             opal_output_verbose(20, USNIC_OUT,
-                                "btl:usnic:%s: weight=0x%016" PRIx64 " for edge module[%d] (%p) <--> endpoint[%d] on proc %p",
-                                __func__,
-                                weight, i,
-                                (void *)mca_btl_usnic_component.usnic_active_modules[i],
-                                j, (void *)proc);
+                                "btl:usnic:%s: weight=0x%016" PRIx64
+                                " for edge module[%d] (%p) <--> endpoint[%d] on proc %p",
+                                __func__, weight, i,
+                                (void *) mca_btl_usnic_component.usnic_active_modules[i], j,
+                                (void *) proc);
 
             if (WEIGHT_UNREACHABLE == weight) {
                 continue;
@@ -507,11 +459,12 @@ static int create_proc_module_graph(
                 v = PROC_VERTEX(j);
             }
             opal_output_verbose(20, USNIC_OUT,
-                                "btl:usnic:%s: adding edge (%d,%d) with cost=%" PRIi64 " for edge module[%d] <--> endpoint[%d]",
+                                "btl:usnic:%s: adding edge (%d,%d) with cost=%" PRIi64
+                                " for edge module[%d] <--> endpoint[%d]",
                                 __func__, u, v, cost, i, j);
             err = opal_bp_graph_add_edge(g, u, v, cost,
-					 /*capacity=*/1,
-					 /*e_data=*/NULL);
+                                         /*capacity=*/1,
+                                         /*e_data=*/NULL);
             if (OPAL_SUCCESS != err) {
                 OPAL_ERROR_LOG(err);
                 goto out_free_graph;
@@ -540,9 +493,7 @@ out:
  * canonicalize this match ordering somehow, probably by (jobid,vpid) pair or
  * by the interface MAC or IP address.
  */
-static int match_modex(opal_btl_usnic_module_t *module,
-                       opal_btl_usnic_proc_t *proc,
-                       int *index_out)
+static int match_modex(opal_btl_usnic_module_t *module, opal_btl_usnic_proc_t *proc, int *index_out)
 {
     int err = OPAL_SUCCESS;
     size_t i;
@@ -558,15 +509,14 @@ static int match_modex(opal_btl_usnic_module_t *module,
     num_modules = mca_btl_usnic_component.num_modules;
 
     opal_output_verbose(20, USNIC_OUT, "btl:usnic:%s: module=%p proc=%p with dimensions %d x %d",
-                        __func__, (void *)module, (void *)proc,
-                        num_modules, (int)proc->proc_modex_count);
+                        __func__, (void *) module, (void *) proc, num_modules,
+                        (int) proc->proc_modex_count);
 
     /* We compute an interface match-up table once for each (module,proc) pair
      * and cache it in the proc.  Store per-proc instead of per-module, since
      * MPI dynamic process routines can add procs but not new modules. */
     if (NULL == proc->proc_ep_match_table) {
-        proc->proc_ep_match_table = malloc(num_modules *
-                                       sizeof(*proc->proc_ep_match_table));
+        proc->proc_ep_match_table = malloc(num_modules * sizeof(*proc->proc_ep_match_table));
         if (NULL == proc->proc_ep_match_table) {
             OPAL_ERROR_LOG(OPAL_ERR_OUT_OF_RESOURCE);
             return OPAL_ERR_OUT_OF_RESOURCE;
@@ -587,9 +537,8 @@ static int match_modex(opal_btl_usnic_module_t *module,
         proc_is_left = (proc->proc_opal->proc_name <
                         opal_proc_local_get()->proc_name);
 #else
-        proc_is_left =
-            usnic_compat_proc_name_compare(proc->proc_opal->proc_name,
-                                           opal_proc_local_get()->proc_name);
+        proc_is_left = usnic_compat_proc_name_compare(proc->proc_opal->proc_name,
+                                                      opal_proc_local_get()->proc_name);
 #endif
 
         err = create_proc_module_graph(proc, proc_is_left, &g);
@@ -615,11 +564,10 @@ static int match_modex(opal_btl_usnic_module_t *module,
         }
     }
 
-
     if (!proc->proc_match_exists) {
-        opal_output_verbose(5, USNIC_OUT, "btl:usnic:%s: unable to find any valid interface pairs for proc %s",
-                            __func__,
-                            usnic_compat_proc_name_print(&proc->proc_opal->proc_name));
+        opal_output_verbose(5, USNIC_OUT,
+                            "btl:usnic:%s: unable to find any valid interface pairs for proc %s",
+                            __func__, usnic_compat_proc_name_print(&proc->proc_opal->proc_name));
         return OPAL_ERR_NOT_FOUND;
     }
 
@@ -637,17 +585,14 @@ static int match_modex(opal_btl_usnic_module_t *module,
     /* TODO with UDP, do we still want to enforce this restriction or just take
      * the min of the two MTUs?  Another choice is to disqualify this pairing
      * before running the matching algorithm on it. */
-    if (*index_out >= 0 &&
-        proc->proc_modex[*index_out].max_msg_size !=
-        (uint16_t) module->fabric_info->ep_attr->max_msg_size) {
-        opal_show_help("help-mpi-btl-usnic.txt", "MTU mismatch",
-                       true,
-                       opal_process_info.nodename,
-                       module->linux_device_name,
-                       module->fabric_info->ep_attr->max_msg_size,
-                       (NULL == proc->proc_opal->proc_hostname) ?
-                       "unknown" : proc->proc_opal->proc_hostname,
-                       proc->proc_modex[*index_out].max_msg_size);
+    if (*index_out >= 0
+        && proc->proc_modex[*index_out].max_msg_size
+               != (uint16_t) module->fabric_info->ep_attr->max_msg_size) {
+        char *errhost = opal_get_proc_hostname(proc->proc_opal);
+        opal_show_help("help-mpi-btl-usnic.txt", "MTU mismatch", true, opal_process_info.nodename,
+                       module->linux_device_name, module->fabric_info->ep_attr->max_msg_size,
+                       errhost, proc->proc_modex[*index_out].max_msg_size);
+        free(errhost);
         *index_out = -1;
         return OPAL_ERR_UNREACH;
     }
@@ -667,9 +612,8 @@ out_free_table:
  * Initiate the process to create a USD dest.
  * It will be polled for completion later.
  */
-static int start_av_insert(opal_btl_usnic_module_t *module,
-                                  opal_btl_usnic_endpoint_t *endpoint,
-                                  int channel)
+static int start_av_insert(opal_btl_usnic_module_t *module, opal_btl_usnic_endpoint_t *endpoint,
+                           int channel)
 {
     int ret;
     opal_btl_usnic_modex_t *modex = &endpoint->endpoint_remote_modex;
@@ -681,11 +625,9 @@ static int start_av_insert(opal_btl_usnic_module_t *module,
     context->channel_id = channel;
 
     char str[IPV4STRADDRLEN];
-    opal_btl_usnic_snprintf_ipv4_addr(str, sizeof(str), modex->ipv4_addr,
-                                      modex->netmask);
-    opal_output_verbose(5, USNIC_OUT,
-                        "btl:usnic:start_av_insert: to channel %d at %s:%d",
-                        channel, str, modex->ports[channel]);
+    opal_btl_usnic_snprintf_ipv4_addr(str, sizeof(str), modex->ipv4_addr, modex->netmask);
+    opal_output_verbose(5, USNIC_OUT, "btl:usnic:start_av_insert: to channel %d at %s:%d", channel,
+                        str, modex->ports[channel]);
 
     /* build remote address */
     memset(&sin, 0, sizeof(sin));
@@ -693,17 +635,12 @@ static int start_av_insert(opal_btl_usnic_module_t *module,
     sin.sin_port = htons(modex->ports[channel]);
     sin.sin_addr.s_addr = modex->ipv4_addr;
 
-    ret = fi_av_insert(module->av, &sin, 1,
-            &endpoint->endpoint_remote_addrs[channel], 0, context);
+    ret = fi_av_insert(module->av, &sin, 1, &endpoint->endpoint_remote_addrs[channel], 0, context);
     /* Did an error occur? */
     if (0 != ret) {
-        opal_show_help("help-mpi-btl-usnic.txt", "libfabric API failed",
-                       true,
-                       opal_process_info.nodename,
-                       module->linux_device_name,
-                       "fi_av_insert()", __FILE__, __LINE__,
-                       ret,
-                       "Failed to initiate AV insert");
+        opal_show_help("help-mpi-btl-usnic.txt", "libfabric API failed", true,
+                       opal_process_info.nodename, module->linux_device_name, "fi_av_insert()",
+                       __FILE__, __LINE__, ret, "Failed to initiate AV insert");
         free(context);
         return OPAL_ERROR;
     }
@@ -714,10 +651,8 @@ static int start_av_insert(opal_btl_usnic_module_t *module,
 /*
  * Create an endpoint and claim the matched modex slot
  */
-int
-opal_btl_usnic_create_endpoint(opal_btl_usnic_module_t *module,
-                opal_btl_usnic_proc_t *proc,
-                opal_btl_usnic_endpoint_t **endpoint_o)
+int opal_btl_usnic_create_endpoint(opal_btl_usnic_module_t *module, opal_btl_usnic_proc_t *proc,
+                                   opal_btl_usnic_endpoint_t **endpoint_o)
 {
     int rc;
     int modex_index;
@@ -737,15 +672,15 @@ opal_btl_usnic_create_endpoint(opal_btl_usnic_module_t *module,
         return OPAL_ERR_OUT_OF_RESOURCE;
     }
 
-    /* Initalize the endpoint */
+    /* Initialize the endpoint */
     endpoint->endpoint_module = module;
-    assert(modex_index >= 0 && modex_index < (int)proc->proc_modex_count);
+    assert(modex_index >= 0 && modex_index < (int) proc->proc_modex_count);
     endpoint->endpoint_remote_modex = proc->proc_modex[modex_index];
     endpoint->endpoint_send_credits = module->sd_num;
 
     /* Start creating destinations; one for each channel.  These
        progress in the background.a */
-    for (int i = 0; i < USNIC_NUM_CHANNELS; ++i)  {
+    for (int i = 0; i < USNIC_NUM_CHANNELS; ++i) {
         rc = start_av_insert(module, endpoint, i);
         if (OPAL_SUCCESS != rc) {
             OBJ_RELEASE(endpoint);
@@ -756,17 +691,15 @@ opal_btl_usnic_create_endpoint(opal_btl_usnic_module_t *module,
     /* Initialize endpoint sequence number info */
     endpoint->endpoint_next_seq_to_send = module->local_modex.isn;
     endpoint->endpoint_ack_seq_rcvd = endpoint->endpoint_next_seq_to_send - 1;
-    endpoint->endpoint_next_contig_seq_to_recv =
-        endpoint->endpoint_remote_modex.isn;
-    endpoint->endpoint_highest_seq_rcvd =
-        endpoint->endpoint_next_contig_seq_to_recv - 1;
+    endpoint->endpoint_next_contig_seq_to_recv = endpoint->endpoint_remote_modex.isn;
+    endpoint->endpoint_highest_seq_rcvd = endpoint->endpoint_next_contig_seq_to_recv - 1;
     endpoint->endpoint_rfstart = WINDOW_SIZE_MOD(endpoint->endpoint_next_contig_seq_to_recv);
 
     /* Now claim that modex slot */
     proc->proc_modex_claimed[modex_index] = true;
-    MSGDEBUG1_OUT("create_endpoint: module=%p claimed endpoint=%p on proc=%p (hash=0x%" PRIx64 ")\n",
-                  (void *)module, (void *)endpoint, (void *)proc,
-                  proc->proc_opal->proc_name);
+    MSGDEBUG1_OUT("create_endpoint: module=%p claimed endpoint=%p on proc=%p (hash=0x%" PRIx64
+                  ")\n",
+                  (void *) module, (void *) endpoint, (void *) proc, proc->proc_opal->proc_name);
 
     /* Save the endpoint on this proc's array of endpoints */
     proc->proc_endpoints[proc->proc_endpoint_count] = endpoint;
@@ -779,8 +712,7 @@ opal_btl_usnic_create_endpoint(opal_btl_usnic_module_t *module,
        not in the endpoint constructor because we aren't able to pass
        the module as a constructor argument -- doh!). */
     opal_mutex_lock(&module->all_endpoints_lock);
-    opal_list_append(&(module->all_endpoints),
-            &(endpoint->endpoint_endpoint_li));
+    opal_list_append(&(module->all_endpoints), &(endpoint->endpoint_endpoint_li));
     endpoint->endpoint_on_all_endpoints = true;
     opal_mutex_unlock(&module->all_endpoints_lock);
 
@@ -805,8 +737,7 @@ opal_btl_usnic_create_endpoint(opal_btl_usnic_module_t *module,
  * opal_btl_usnic_endpoint_t instances, and published addresses/modex
  * info.
  */
-int opal_btl_usnic_proc_match(opal_proc_t *opal_proc,
-                              opal_btl_usnic_module_t *module,
+int opal_btl_usnic_proc_match(opal_proc_t *opal_proc, opal_btl_usnic_module_t *module,
                               opal_btl_usnic_proc_t **proc)
 {
     /* Check if we have already created a proc structure for this peer
