@@ -11,7 +11,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2008      Sun Microsystems, Inc.  All rights reserved.
- * Copyright (c) 2010-2015 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2010-2022 Cisco Systems, Inc.  All rights reserved
  * Copyright (c) 2014-2017 Los Alamos National Security, LLC. All rights
  *                         reserved.
  * Copyright (c) 2015-2016 Research Organization for Information Science
@@ -29,64 +29,65 @@
 
 #include <string.h>
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+#    include <unistd.h>
 #endif
 #include <errno.h>
 #ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
+#    include <sys/types.h>
 #endif
 #ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
+#    include <sys/socket.h>
 #endif
 #ifdef HAVE_SYS_SOCKIO_H
-#include <sys/sockio.h>
+#    include <sys/sockio.h>
 #endif
 #ifdef HAVE_SYS_IOCTL_H
-#include <sys/ioctl.h>
+#    include <sys/ioctl.h>
 #endif
 #ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
+#    include <netinet/in.h>
 #endif
 #ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
+#    include <arpa/inet.h>
 #endif
 #ifdef HAVE_NET_IF_H
-#include <net/if.h>
+#    include <net/if.h>
 #endif
 #ifdef HAVE_NETDB_H
-#include <netdb.h>
+#    include <netdb.h>
 #endif
 #ifdef HAVE_IFADDRS_H
-#include <ifaddrs.h>
+#    include <ifaddrs.h>
 #endif
 #include <ctype.h>
 
 #include "opal/class/opal_list.h"
+#include "opal/constants.h"
+#include "opal/util/argv.h"
 #include "opal/util/if.h"
 #include "opal/util/net.h"
 #include "opal/util/output.h"
-#include "opal/util/argv.h"
 #include "opal/util/show_help.h"
-#include "opal/constants.h"
+#include "opal/util/string_copy.h"
 
 #include "opal/mca/if/base/base.h"
 
 #ifdef HAVE_STRUCT_SOCKADDR_IN
 
-#ifndef MIN
-#  define MIN(a,b)                ((a) < (b) ? (a) : (b))
-#endif
+#    ifndef MIN
+#        define MIN(a, b) ((a) < (b) ? (a) : (b))
+#    endif
 
 /*
  *  Look for interface by name and returns its address
  *  as a dotted decimal formatted string.
  */
 
-int opal_ifnametoaddr(const char* if_name, struct sockaddr* addr, int length)
+int opal_ifnametoaddr(const char *if_name, struct sockaddr *addr, int length)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (strcmp(intf->if_name, if_name) == 0) {
             memcpy(addr, &intf->if_addr, length);
             return OPAL_SUCCESS;
@@ -95,17 +96,16 @@ int opal_ifnametoaddr(const char* if_name, struct sockaddr* addr, int length)
     return OPAL_ERROR;
 }
 
-
 /*
  *  Look for interface by name and returns its
  *  corresponding opal_list index.
  */
 
-int opal_ifnametoindex(const char* if_name)
+int opal_ifnametoindex(const char *if_name)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (strcmp(intf->if_name, if_name) == 0) {
             return intf->if_index;
         }
@@ -113,17 +113,16 @@ int opal_ifnametoindex(const char* if_name)
     return -1;
 }
 
-
 /*
  *  Look for interface by name and returns its
  *  corresponding kernel index.
  */
 
-int opal_ifnametokindex(const char* if_name)
+int opal_ifnametokindex(const char *if_name)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (strcmp(intf->if_name, if_name) == 0) {
             return intf->if_kernel_index;
         }
@@ -131,7 +130,6 @@ int opal_ifnametokindex(const char* if_name)
     return -1;
 }
 
-
 /*
  *  Look for interface by opal_list index and returns its
  *  corresponding kernel index.
@@ -139,9 +137,9 @@ int opal_ifnametokindex(const char* if_name)
 
 int opal_ifindextokindex(int if_index)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (if_index == intf->if_index) {
             return intf->if_kernel_index;
         }
@@ -149,15 +147,14 @@ int opal_ifindextokindex(int if_index)
     return -1;
 }
 
-
 /*
- *  Attempt to resolve the adddress (given as either IPv4/IPv6 string
+ *  Attempt to resolve the address (given as either IPv4/IPv6 string
  *  or hostname) and lookup corresponding interface.
  */
 
-int opal_ifaddrtoname(const char* if_addr, char* if_name, int length)
+int opal_ifaddrtoname(const char *if_addr, char *if_name, int length)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
     int error;
     struct addrinfo hints, *res = NULL, *r;
 
@@ -176,40 +173,40 @@ int opal_ifaddrtoname(const char* if_addr, char* if_name, int length)
 
     if (error) {
         if (NULL != res) {
-            freeaddrinfo (res);
+            freeaddrinfo(res);
         }
         return OPAL_ERR_NOT_FOUND;
     }
 
     for (r = res; r != NULL; r = r->ai_next) {
-        OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+        OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
             if (AF_INET == r->ai_family) {
                 struct sockaddr_in ipv4;
                 struct sockaddr_in *inaddr;
 
-                inaddr = (struct sockaddr_in*) &intf->if_addr;
-                memcpy (&ipv4, r->ai_addr, r->ai_addrlen);
+                inaddr = (struct sockaddr_in *) &intf->if_addr;
+                memcpy(&ipv4, r->ai_addr, r->ai_addrlen);
 
                 if (inaddr->sin_addr.s_addr == ipv4.sin_addr.s_addr) {
-                    strncpy(if_name, intf->if_name, length);
-                    freeaddrinfo (res);
+                    opal_string_copy(if_name, intf->if_name, length);
+                    freeaddrinfo(res);
                     return OPAL_SUCCESS;
                 }
             }
-#if OPAL_ENABLE_IPV6
+#    if OPAL_ENABLE_IPV6
             else {
-                if (IN6_ARE_ADDR_EQUAL(&((struct sockaddr_in6*) &intf->if_addr)->sin6_addr,
-                    &((struct sockaddr_in6*) r->ai_addr)->sin6_addr)) {
-                    strncpy(if_name, intf->if_name, length);
-                    freeaddrinfo (res);
+                if (IN6_ARE_ADDR_EQUAL(&((struct sockaddr_in6 *) &intf->if_addr)->sin6_addr,
+                                       &((struct sockaddr_in6 *) r->ai_addr)->sin6_addr)) {
+                    opal_string_copy(if_name, intf->if_name, length);
+                    freeaddrinfo(res);
                     return OPAL_SUCCESS;
                 }
             }
-#endif
+#    endif
         }
     }
     if (NULL != res) {
-        freeaddrinfo (res);
+        freeaddrinfo(res);
     }
 
     /* if we get here, it wasn't found */
@@ -221,9 +218,9 @@ int opal_ifaddrtoname(const char* if_addr, char* if_name, int length)
  *  or hostname) and return the kernel index of the interface
  *  on the same network as the specified address
  */
-int opal_ifaddrtokindex(const char* if_addr)
+int opal_ifaddrtokindex(const char *if_addr)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
     int error;
     struct addrinfo hints, *res = NULL, *r;
     int if_kernel_index;
@@ -236,40 +233,44 @@ int opal_ifaddrtokindex(const char* if_addr)
 
     if (error) {
         if (NULL != res) {
-            freeaddrinfo (res);
+            freeaddrinfo(res);
         }
         return OPAL_ERR_NOT_FOUND;
     }
 
     for (r = res; r != NULL; r = r->ai_next) {
-        OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+        OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
             if (AF_INET == r->ai_family && AF_INET == intf->af_family) {
                 struct sockaddr_in ipv4;
-                len = (r->ai_addrlen < sizeof(struct sockaddr_in)) ? r->ai_addrlen : sizeof(struct sockaddr_in);
+                len = (r->ai_addrlen < sizeof(struct sockaddr_in)) ? r->ai_addrlen
+                                                                   : sizeof(struct sockaddr_in);
                 memcpy(&ipv4, r->ai_addr, len);
-                if (opal_net_samenetwork((struct sockaddr*)&ipv4, (struct sockaddr*)&intf->if_addr, intf->if_mask)) {
+                if (opal_net_samenetwork((struct sockaddr *) &ipv4,
+                                         (struct sockaddr *) &intf->if_addr, intf->if_mask)) {
                     if_kernel_index = intf->if_kernel_index;
-                    freeaddrinfo (res);
+                    freeaddrinfo(res);
                     return if_kernel_index;
                 }
             }
-#if OPAL_ENABLE_IPV6
+#    if OPAL_ENABLE_IPV6
             else if (AF_INET6 == r->ai_family && AF_INET6 == intf->af_family) {
                 struct sockaddr_in6 ipv6;
-                len = (r->ai_addrlen < sizeof(struct sockaddr_in6)) ? r->ai_addrlen : sizeof(struct sockaddr_in6);
+                len = (r->ai_addrlen < sizeof(struct sockaddr_in6)) ? r->ai_addrlen
+                                                                    : sizeof(struct sockaddr_in6);
                 memcpy(&ipv6, r->ai_addr, len);
-                if (opal_net_samenetwork((struct sockaddr*)((struct sockaddr_in6*)&intf->if_addr),
-                                         (struct sockaddr*)&ipv6, intf->if_mask)) {
+                if (opal_net_samenetwork((struct sockaddr *) ((struct sockaddr_in6 *) &intf
+                                                                  ->if_addr),
+                                         (struct sockaddr *) &ipv6, intf->if_mask)) {
                     if_kernel_index = intf->if_kernel_index;
-                    freeaddrinfo (res);
+                    freeaddrinfo(res);
                     return if_kernel_index;
                 }
             }
-#endif
+#    endif
         }
     }
     if (NULL != res) {
-        freeaddrinfo (res);
+        freeaddrinfo(res);
     }
     return OPAL_ERR_NOT_FOUND;
 }
@@ -283,7 +284,6 @@ int opal_ifcount(void)
     return opal_list_get_size(&opal_if_list);
 }
 
-
 /*
  *  Return the opal_list interface index for the first
  *  interface in our list.
@@ -293,13 +293,13 @@ int opal_ifbegin(void)
 {
     opal_if_t *intf;
 
-    intf = (opal_if_t*)opal_list_get_first(&opal_if_list);
-    if (NULL != intf)
+    intf = (opal_if_t *) opal_list_get_first(&opal_if_list);
+    if (NULL != intf) {
         return intf->if_index;
+    }
     return (-1);
 }
 
-
 /*
  *  Located the current position in the list by if_index and
  *  return the interface index of the next element in our list
@@ -310,70 +310,67 @@ int opal_ifnext(int if_index)
 {
     opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (intf->if_index == if_index) {
             do {
-                opal_if_t* if_next = (opal_if_t*)opal_list_get_next(intf);
-                opal_if_t* if_end =  (opal_if_t*)opal_list_get_end(&opal_if_list);
+                opal_if_t *if_next = (opal_if_t *) opal_list_get_next(intf);
+                opal_if_t *if_end = (opal_if_t *) opal_list_get_end(&opal_if_list);
                 if (if_next == if_end) {
                     return -1;
                 }
                 intf = if_next;
-            } while(intf->if_index == if_index);
+            } while (intf->if_index == if_index);
             return intf->if_index;
         }
     }
     return (-1);
 }
 
-
 /*
  *  Lookup the interface by opal_list index and return the
  *  primary address assigned to the interface.
  */
 
-int opal_ifindextoaddr(int if_index, struct sockaddr* if_addr, unsigned int length)
+int opal_ifindextoaddr(int if_index, struct sockaddr *if_addr, unsigned int length)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (intf->if_index == if_index) {
-            memcpy(if_addr, &intf->if_addr, MIN(length, sizeof (intf->if_addr)));
+            memcpy(if_addr, &intf->if_addr, MIN(length, sizeof(intf->if_addr)));
             return OPAL_SUCCESS;
         }
     }
     return OPAL_ERROR;
 }
 
-
 /*
  *  Lookup the interface by opal_list kindex and return the
  *  primary address assigned to the interface.
  */
-int opal_ifkindextoaddr(int if_kindex, struct sockaddr* if_addr, unsigned int length)
+int opal_ifkindextoaddr(int if_kindex, struct sockaddr *if_addr, unsigned int length)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (intf->if_kernel_index == if_kindex) {
-            memcpy(if_addr, &intf->if_addr, MIN(length, sizeof (intf->if_addr)));
+            memcpy(if_addr, &intf->if_addr, MIN(length, sizeof(intf->if_addr)));
             return OPAL_SUCCESS;
         }
     }
     return OPAL_ERROR;
 }
 
-
 /*
  *  Lookup the interface by opal_list index and return the
  *  network mask assigned to the interface.
  */
 
-int opal_ifindextomask(int if_index, uint32_t* if_mask, int length)
+int opal_ifindextomask(int if_index, uint32_t *if_mask, int length)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (intf->if_index == if_index) {
             memcpy(if_mask, &intf->if_mask, length);
             return OPAL_SUCCESS;
@@ -389,9 +386,9 @@ int opal_ifindextomask(int if_index, uint32_t* if_mask, int length)
 
 int opal_ifindextomac(int if_index, uint8_t mac[6])
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (intf->if_index == if_index) {
             memcpy(mac, &intf->if_mac, 6);
             return OPAL_SUCCESS;
@@ -407,9 +404,9 @@ int opal_ifindextomac(int if_index, uint8_t mac[6])
 
 int opal_ifindextomtu(int if_index, int *mtu)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (intf->if_index == if_index) {
             *mtu = intf->ifmtu;
             return OPAL_SUCCESS;
@@ -423,11 +420,11 @@ int opal_ifindextomtu(int if_index, int *mtu)
  *  flags assigned to the interface.
  */
 
-int opal_ifindextoflags(int if_index, uint32_t* if_flags)
+int opal_ifindextoflags(int if_index, uint32_t *if_flags)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (intf->if_index == if_index) {
             memcpy(if_flags, &intf->if_flags, sizeof(uint32_t));
             return OPAL_SUCCESS;
@@ -436,56 +433,51 @@ int opal_ifindextoflags(int if_index, uint32_t* if_flags)
     return OPAL_ERROR;
 }
 
-
-
 /*
  *  Lookup the interface by opal_list index and return
  *  the associated name.
  */
 
-int opal_ifindextoname(int if_index, char* if_name, int length)
+int opal_ifindextoname(int if_index, char *if_name, int length)
 {
     opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (intf->if_index == if_index) {
-            strncpy(if_name, intf->if_name, length);
+            opal_string_copy(if_name, intf->if_name, length);
             return OPAL_SUCCESS;
         }
     }
     return OPAL_ERROR;
 }
 
-
 /*
  *  Lookup the interface by kernel index and return
  *  the associated name.
  */
 
-int opal_ifkindextoname(int if_kindex, char* if_name, int length)
+int opal_ifkindextoname(int if_kindex, char *if_name, int length)
 {
     opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (intf->if_kernel_index == if_kindex) {
-            strncpy(if_name, intf->if_name, length);
+            opal_string_copy(if_name, intf->if_name, length);
             return OPAL_SUCCESS;
         }
     }
     return OPAL_ERROR;
 }
 
-
-#define ADDRLEN 100
-bool
-opal_ifislocal(const char *hostname)
+#    define ADDRLEN 100
+bool opal_ifislocal(const char *hostname)
 {
-#if OPAL_ENABLE_IPV6
+#    if OPAL_ENABLE_IPV6
     char addrname[NI_MAXHOST]; /* should be larger than ADDRLEN, but I think
                                   they really mean IFNAMESIZE */
-#else
+#    else
     char addrname[ADDRLEN + 1];
-#endif
+#    endif
 
     if (OPAL_SUCCESS == opal_ifaddrtoname(hostname, addrname, ADDRLEN)) {
         return true;
@@ -494,16 +486,16 @@ opal_ifislocal(const char *hostname)
     return false;
 }
 
-static int parse_ipv4_dots(const char *addr, uint32_t* net, int* dots)
+static int parse_ipv4_dots(const char *addr, uint32_t *net, int *dots)
 {
     const char *start = addr, *end;
-    uint32_t n[]={0,0,0,0};
+    uint32_t n[] = {0, 0, 0, 0};
     int i;
 
     /* now assemble the address */
-    for( i = 0; i < 4; i++ ) {
-        n[i] = strtoul(start, (char**)&end, 10);
-        if( end == start ) {
+    for (i = 0; i < 4; i++) {
+        n[i] = strtoul(start, (char **) &end, 10);
+        if (end == start) {
             /* this is not an error, but indicates that
              * we were given a partial address - e.g.,
              * 192.168 - usually indicating an IP range
@@ -512,20 +504,22 @@ static int parse_ipv4_dots(const char *addr, uint32_t* net, int* dots)
             break;
         }
         /* did we read something sensible? */
-        if( n[i] > 255 ) {
+        if (n[i] > 255) {
             return OPAL_ERR_NETWORK_NOT_PARSEABLE;
         }
         /* skip all the . */
-        for( start = end; '\0' != *start; start++ )
-            if( '.' != *start ) break;
+        for (start = end; '\0' != *start; start++) {
+            if ('.' != *start) {
+                break;
+            }
+        }
     }
     *dots = i;
     *net = OPAL_IF_ASSEMBLE_NETWORK(n[0], n[1], n[2], n[3]);
     return OPAL_SUCCESS;
 }
 
-int
-opal_iftupletoaddr(const char *inaddr, uint32_t *net, uint32_t *mask)
+int opal_iftupletoaddr(const char *inaddr, uint32_t *net, uint32_t *mask)
 {
     int pval, dots, rc = OPAL_SUCCESS;
     const char *ptr;
@@ -537,14 +531,14 @@ opal_iftupletoaddr(const char *inaddr, uint32_t *net, uint32_t *mask)
 
         /* if entry includes mask, split that off */
         if (NULL != (ptr = strchr(inaddr, '/'))) {
-            ptr = ptr + 1;  /* skip the / */
+            ptr = ptr + 1; /* skip the / */
             /* is the mask a tuple? */
             if (NULL != strchr(ptr, '.')) {
                 /* yes - extract mask from it */
                 rc = parse_ipv4_dots(ptr, mask, &dots);
             } else {
                 /* no - must be an int telling us how much of the addr to use: e.g., /16
-                 * For more information please read http://en.wikipedia.org/wiki/Subnetwork.
+                 * For more information please read https://en.wikipedia.org/wiki/Subnetwork.
                  */
                 pval = strtol(ptr, NULL, 10);
                 if ((pval > 31) || (pval < 1)) {
@@ -555,7 +549,7 @@ opal_iftupletoaddr(const char *inaddr, uint32_t *net, uint32_t *mask)
             }
         } else {
             /* use the number of dots to determine it */
-            for (ptr = inaddr, pval = 0; '\0'!= *ptr; ptr++) {
+            for (ptr = inaddr, pval = 0; '\0' != *ptr; ptr++) {
                 if ('.' == *ptr) {
                     pval++;
                 }
@@ -566,11 +560,11 @@ opal_iftupletoaddr(const char *inaddr, uint32_t *net, uint32_t *mask)
              */
             if (3 == pval) {
                 *mask = 0xFFFFFFFF;
-            } else if (2 == pval) {         /* 2 dots */
+            } else if (2 == pval) { /* 2 dots */
                 *mask = 0xFFFFFF00;
-            } else if (1 == pval) {  /* 1 dot */
+            } else if (1 == pval) { /* 1 dot */
                 *mask = 0xFFFF0000;
-            } else if (0 == pval) {  /* no dots */
+            } else if (0 == pval) { /* no dots */
                 *mask = 0xFF000000;
             } else {
                 opal_output(0, "opal_iftupletoaddr: unknown mask");
@@ -594,9 +588,9 @@ opal_iftupletoaddr(const char *inaddr, uint32_t *net, uint32_t *mask)
 
 bool opal_ifisloopback(int if_index)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
         if (intf->if_index == if_index) {
             if ((intf->if_flags & IFF_LOOPBACK) != 0) {
                 return true;
@@ -620,17 +614,18 @@ int opal_ifmatches(int kidx, char **nets)
     uint32_t addr, netaddr, netmask;
 
     /* get the address info for the given network in case we need it */
-    if (OPAL_SUCCESS != (rc = opal_ifkindextoaddr(kidx, (struct sockaddr*)&inaddr, sizeof(inaddr)))) {
+    if (OPAL_SUCCESS
+        != (rc = opal_ifkindextoaddr(kidx, (struct sockaddr *) &inaddr, sizeof(inaddr)))) {
         return rc;
     }
     addr = ntohl(inaddr.sin_addr.s_addr);
 
-    for (i=0; NULL != nets[i]; i++) {
+    for (i = 0; NULL != nets[i]; i++) {
         /* if the specified interface contains letters in it, then it
          * was given as an interface name and not an IP tuple
          */
         named_if = false;
-        for (j=0; j < strlen(nets[i]); j++) {
+        for (j = 0; j < strlen(nets[i]); j++) {
             if (isalpha(nets[i][j]) && '.' != nets[i][j]) {
                 named_if = true;
                 break;
@@ -659,19 +654,19 @@ int opal_ifmatches(int kidx, char **nets)
 
 void opal_ifgetaliases(char ***aliases)
 {
-    opal_if_t* intf;
+    opal_if_t *intf;
     char ipv4[INET_ADDRSTRLEN];
     struct sockaddr_in *addr;
-#if OPAL_ENABLE_IPV6
+#    if OPAL_ENABLE_IPV6
     char ipv6[INET6_ADDRSTRLEN];
     struct sockaddr_in6 *addr6;
-#endif
+#    endif
 
     /* set default answer */
     *aliases = NULL;
 
-    OPAL_LIST_FOREACH(intf, &opal_if_list, opal_if_t) {
-        addr = (struct sockaddr_in*) &intf->if_addr;
+    OPAL_LIST_FOREACH (intf, &opal_if_list, opal_if_t) {
+        addr = (struct sockaddr_in *) &intf->if_addr;
         /* ignore purely loopback interfaces */
         if ((intf->if_flags & IFF_LOOPBACK) != 0) {
             continue;
@@ -680,13 +675,13 @@ void opal_ifgetaliases(char ***aliases)
             inet_ntop(AF_INET, &(addr->sin_addr.s_addr), ipv4, INET_ADDRSTRLEN);
             opal_argv_append_nosize(aliases, ipv4);
         }
-#if OPAL_ENABLE_IPV6
+#    if OPAL_ENABLE_IPV6
         else {
-            addr6 = (struct sockaddr_in6*) &intf->if_addr;
+            addr6 = (struct sockaddr_in6 *) &intf->if_addr;
             inet_ntop(AF_INET6, &(addr6->sin6_addr), ipv6, INET6_ADDRSTRLEN);
             opal_argv_append_nosize(aliases, ipv6);
         }
-#endif
+#    endif
     }
 }
 
@@ -695,88 +690,72 @@ void opal_ifgetaliases(char ***aliases)
 /* if we don't have struct sockaddr_in, we don't have traditional
    ethernet devices.  Just make everything a no-op error call */
 
-int
-opal_ifnametoaddr(const char* if_name,
-                  struct sockaddr* if_addr, int size)
+int opal_ifnametoaddr(const char *if_name, struct sockaddr *if_addr, int size)
 {
     return OPAL_ERR_NOT_SUPPORTED;
 }
 
-int
-opal_ifaddrtoname(const char* if_addr,
-                  char* if_name, int size)
+int opal_ifaddrtoname(const char *if_addr, char *if_name, int size)
 {
     return OPAL_ERR_NOT_SUPPORTED;
 }
 
-int
-opal_ifnametoindex(const char* if_name)
+int opal_ifnametoindex(const char *if_name)
 {
     return OPAL_ERR_NOT_SUPPORTED;
 }
 
-int
-opal_ifnametokindex(const char* if_name)
+int opal_ifnametokindex(const char *if_name)
 {
     return OPAL_ERR_NOT_SUPPORTED;
 }
 
-int
-opal_ifindextokindex(int if_index)
+int opal_ifindextokindex(int if_index)
 {
     return OPAL_ERR_NOT_SUPPORTED;
 }
 
-int
-opal_ifcount(void)
+int opal_ifcount(void)
 {
     return OPAL_ERR_NOT_SUPPORTED;
 }
 
-int
-opal_ifbegin(void)
+int opal_ifbegin(void)
 {
     return OPAL_ERR_NOT_SUPPORTED;
 }
 
-int
-opal_ifnext(int if_index)
+int opal_ifnext(int if_index)
 {
     return OPAL_ERR_NOT_SUPPORTED;
 }
 
-int
-opal_ifindextoname(int if_index, char* if_name, int length)
+int opal_ifindextoname(int if_index, char *if_name, int length)
 {
     return OPAL_ERR_NOT_SUPPORTED;
 }
 
-int
-opal_ifkindextoname(int kif_index, char* if_name, int length)
+int opal_ifkindextoname(int kif_index, char *if_name, int length)
 {
     return OPAL_ERR_NOT_SUPPORTED;
 }
 
-int
-opal_ifindextoaddr(int if_index, struct sockaddr* if_addr, unsigned int length)
+int opal_ifindextoaddr(int if_index, struct sockaddr *if_addr, unsigned int length)
 {
     return OPAL_ERR_NOT_SUPPORTED;
 }
 
-int
-opal_ifindextomask(int if_index, uint32_t* if_addr, int length)
+int opal_ifindextomask(int if_index, uint32_t *if_addr, int length)
 {
     return OPAL_ERR_NOT_SUPPORTED;
 }
 
-bool
-opal_ifislocal(const char *hostname)
+bool opal_ifislocal(const char *hostname)
 {
     return false;
 }
 
-int
-opal_iftupletoaddr(const char *inaddr, uint32_t *net, uint32_t *mask)
+int opal_iftupletoaddr(const char *inaddr, uint32_t *net, uint32_t *mask)
 {
     return 0;
 }
@@ -793,4 +772,3 @@ void opal_ifgetaliases(char ***aliases)
 }
 
 #endif /* HAVE_STRUCT_SOCKADDR_IN */
-
