@@ -1,3 +1,4 @@
+/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil -*- */
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
@@ -11,6 +12,8 @@
  *                         All rights reserved.
  * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
  *                         reserved.
+ * Copyright (c) 2018      Triad National Security, LLC. All rights
+ *                         reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -26,19 +29,19 @@
 #include "opal_config.h"
 
 #ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
+#    include <sys/types.h>
 #endif
 #ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
+#    include <sys/socket.h>
 #endif
 #ifdef HAVE_NETINET_IN_H
-#include <netinet/in.h>
+#    include <netinet/in.h>
 #endif
 
 BEGIN_C_DECLS
 
 /**
- * Intiailize the network helper subsystem
+ * Initialize the network helper subsystem
  *
  * Initialize the network helper subsystem.  Should be called exactly
  * once for any process that will use any function in the network
@@ -50,19 +53,6 @@ BEGIN_C_DECLS
  */
 OPAL_DECLSPEC int opal_net_init(void);
 
-
-/**
- * Finalize the network helper subsystem
- *
- * Finalize the network helper subsystem.  Should be called exactly
- * once for any process that will use any function in the network
- * helper subsystem.
- *
- * @retval OPAL_SUCCESS   Success
- */
-OPAL_DECLSPEC int opal_net_finalize(void);
-
-
 /**
  * Calculate netmask in network byte order from CIDR notation
  *
@@ -71,7 +61,6 @@ OPAL_DECLSPEC int opal_net_finalize(void);
  */
 OPAL_DECLSPEC uint32_t opal_net_prefix2netmask(uint32_t prefixlen);
 
-
 /**
  * Determine if given IP address is in the localhost range
  *
@@ -85,7 +74,6 @@ OPAL_DECLSPEC uint32_t opal_net_prefix2netmask(uint32_t prefixlen);
  */
 OPAL_DECLSPEC bool opal_net_islocalhost(const struct sockaddr *addr);
 
-
 /**
  * Are we on the same network?
  *
@@ -98,11 +86,9 @@ OPAL_DECLSPEC bool opal_net_islocalhost(const struct sockaddr *addr);
  * @return                  true if \c addr1 and \c addr2 are on the
  *                          same net, false otherwise.
  */
-OPAL_DECLSPEC bool opal_net_samenetwork(const struct sockaddr *addr1,
-                                        const struct sockaddr *addr2,
+OPAL_DECLSPEC bool opal_net_samenetwork(const struct sockaddr *addr1, const struct sockaddr *addr2,
                                         uint32_t prefixlen);
 
-
 /**
  * Is the given address a public IPv4 address?  Returns false for IPv6
  * address.
@@ -131,8 +117,7 @@ OPAL_DECLSPEC bool opal_net_addr_isipv6linklocal(const struct sockaddr *addr);
  * @param addr              struct sockaddr of address
  * @return                  literal representation of \c addr
  */
-OPAL_DECLSPEC char* opal_net_get_hostname(const struct sockaddr *addr);
-
+OPAL_DECLSPEC char *opal_net_get_hostname(const struct sockaddr *addr);
 
 /**
  * Get port number from struct sockaddr
