@@ -3,7 +3,7 @@
  * Copyright (c) 2004-2010 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2008 The University of Tennessee and The University
+ * Copyright (c) 2004-2021 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2006 High Performance Computing Center Stuttgart,
@@ -13,9 +13,16 @@
  * Copyright (c) 2007-2008 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2015      Los Alamos National Security, LLC. All rights
  *                         reserved.
- * Copyright (c) 2015      Research Organization for Information Science
+ * Copyright (c) 2015-2019 Research Organization for Information Science
  *                         and Technology (RIST). All rights reserved.
  * Copyright (c) 2017      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2017-2019 Intel, Inc.  All rights reserved.
+ * Copyright (c) 2018      Amazon.com, Inc. or its affiliates.  All Rights reserved.
+ * Copyright (c) 2018      Triad National Security, LLC. All rights
+ *                         reserved.
+ * Copyright (c) 2019      Triad National Security, LLC. All rights
+ *                         reserved.
+ * Copyright (c) 2024      Jeffrey M. Squyres.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -25,25 +32,28 @@
 
 #include "opal_config.h"
 
-#include <stdio.h>
 #include <stdarg.h>
+#include <stdio.h>
 #include <stdlib.h>
 #ifdef HAVE_SYSLOG_H
-#include <syslog.h>
+#    include <syslog.h>
 #endif
-#include <string.h>
 #include <fcntl.h>
+#include <string.h>
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+#    include <unistd.h>
 #endif
 #ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h>
+#    include <sys/param.h>
 #endif
 
+#include "opal/constants.h"
+#include "opal/mca/threads/mutex.h"
+#include "opal/runtime/opal.h"
 #include "opal/util/opal_environ.h"
 #include "opal/util/output.h"
-#include "opal/threads/mutex.h"
-#include "opal/constants.h"
+#include "opal/util/printf.h"
+#include "opal/util/string_copy.h"
 
 /*
  * Private data
@@ -53,6 +63,7 @@ static opal_output_stream_t verbose;
 static char *output_dir = NULL;
 static char *output_prefix = NULL;
 
+static opal_output_pmix_cleanup_fn_t output_pmix_cleanup_fn = NULL;
 
 /*
  * Internal data structures and helpers for the generalized output
@@ -81,25 +92,27 @@ typedef struct {
     char *ldi_file_suffix;
     int ldi_fd;
     int ldi_file_num_lines_lost;
+    char *ldi_filename;
 } output_desc_t;
 
 /*
  * Private functions
  */
 static void construct(opal_object_t *stream);
-static int do_open(int output_id, opal_output_stream_t * lds);
+static void destruct(opal_object_t *stream);
+static int do_open(int output_id, opal_output_stream_t *lds);
 static int open_file(int i);
 static void free_descriptor(int output_id);
-static int make_string(char **no_newline_string, output_desc_t *ldi,
-                       const char *format, va_list arglist);
+static int make_string(char **no_newline_string, output_desc_t *ldi, const char *format,
+                       va_list arglist);
 static int output(int output_id, const char *format, va_list arglist);
-
+static void opal_output_finalize(void);
 
 #define OPAL_OUTPUT_MAX_STREAMS 64
 #if defined(HAVE_SYSLOG)
-#define USE_SYSLOG 1
+#    define USE_SYSLOG 1
 #else
-#define USE_SYSLOG 0
+#    define USE_SYSLOG 0
 #endif
 
 /* global state */
@@ -120,7 +133,7 @@ static bool syslog_opened = false;
 #endif
 static char *redirect_syslog_ident = NULL;
 
-OBJ_CLASS_INSTANCE(opal_output_stream_t, opal_object_t, construct, NULL);
+OBJ_CLASS_INSTANCE(opal_output_stream_t, opal_object_t, construct, destruct);
 
 /*
  * Setup the output stream infrastructure
@@ -128,7 +141,7 @@ OBJ_CLASS_INSTANCE(opal_output_stream_t, opal_object_t, construct, NULL);
 bool opal_output_init(void)
 {
     int i;
-    char hostname[OPAL_MAXHOSTNAMELEN];
+    const char *hostname;
     char *str;
 
     if (initialized) {
@@ -146,6 +159,7 @@ bool opal_output_init(void)
         }
     }
     str = getenv("OPAL_OUTPUT_SYSLOG_PRI");
+#ifdef HAVE_SYSLOG_H
     if (NULL != str) {
         if (0 == strcasecmp(str, "info")) {
             opal_output_redirected_syslog_pri = LOG_INFO;
@@ -159,7 +173,7 @@ bool opal_output_init(void)
     } else {
         opal_output_redirected_syslog_pri = LOG_ERR;
     }
-
+#endif /* HAVE_SYSLOG_H */
     str = getenv("OPAL_OUTPUT_SYSLOG_IDENT");
     if (NULL != str) {
         redirect_syslog_ident = strdup(str);
@@ -178,13 +192,12 @@ bool opal_output_init(void)
         str = getenv("OPAL_OUTPUT_INTERNAL_TO_STDOUT");
         if (NULL != str && str[0] == '1') {
             verbose.lds_want_stdout = true;
-        }
-        else {
+        } else {
             verbose.lds_want_stderr = true;
         }
     }
-    gethostname(hostname, sizeof(hostname));
-    asprintf(&verbose.lds_prefix, "[%s:%05d] ", hostname, getpid());
+    hostname = opal_gethostname();
+    opal_asprintf(&verbose.lds_prefix, "[%s:%05d] ", hostname, getpid());
 
     for (i = 0; i < OPAL_OUTPUT_MAX_STREAMS; ++i) {
         info[i].ldi_used = false;
@@ -195,6 +208,7 @@ bool opal_output_init(void)
         info[i].ldi_file_suffix = NULL;
         info[i].ldi_file_want_append = false;
         info[i].ldi_fd = -1;
+        info[i].ldi_filename = NULL;
         info[i].ldi_file_num_lines_lost = 0;
     }
 
@@ -205,33 +219,34 @@ bool opal_output_init(void)
 
     /* Set some defaults */
 
-    asprintf(&output_prefix, "output-pid%d-", getpid());
+    opal_asprintf(&output_prefix, "output-pid%d-", getpid());
     output_dir = strdup(opal_tmp_directory());
 
     /* Open the default verbose stream */
     verbose_stream = opal_output_open(&verbose);
+
+    /* make sure opal output is cleaned up on finalize */
+    opal_finalize_register_cleanup(opal_output_finalize);
+
     return true;
 }
 
-
 /*
  * Open a stream
  */
-int opal_output_open(opal_output_stream_t * lds)
+int opal_output_open(opal_output_stream_t *lds)
 {
     return do_open(-1, lds);
 }
 
-
 /*
  * Reset the parameters on a stream
  */
-int opal_output_reopen(int output_id, opal_output_stream_t * lds)
+int opal_output_reopen(int output_id, opal_output_stream_t *lds)
 {
     return do_open(output_id, lds);
 }
 
-
 /*
  * Enable and disable output streams
  */
@@ -253,14 +268,13 @@ bool opal_output_switch(int output_id, bool enable)
     return ret;
 }
 
-
 /*
  * Reopen all the streams; used during checkpoint/restart.
  */
 void opal_output_reopen_all(void)
 {
     char *str;
-    char hostname[OPAL_MAXHOSTNAMELEN];
+    const char *hostname;
 
     str = getenv("OPAL_OUTPUT_STDERR_FD");
     if (NULL != str) {
@@ -269,12 +283,12 @@ void opal_output_reopen_all(void)
         default_stderr_fd = -1;
     }
 
-    gethostname(hostname, sizeof(hostname));
-    if( NULL != verbose.lds_prefix ) {
+    hostname = opal_gethostname();
+    if (NULL != verbose.lds_prefix) {
         free(verbose.lds_prefix);
         verbose.lds_prefix = NULL;
     }
-    asprintf(&verbose.lds_prefix, "[%s:%05d] ", hostname, getpid());
+    opal_asprintf(&verbose.lds_prefix, "[%s:%05d] ", hostname, getpid());
 #if 0
     int i;
     opal_output_stream_t lds;
@@ -293,13 +307,13 @@ void opal_output_reopen_all(void)
          */
         info[i].ldi_used = false;
 
-#if USE_SYSLOG
+#    if USE_SYSLOG
         lds.lds_want_syslog = info[i].ldi_syslog;
         lds.lds_syslog_priority = info[i].ldi_syslog_priority;
         lds.lds_syslog_ident = info[i].ldi_syslog_ident;
-#else
+#    else
         lds.lds_want_syslog = false;
-#endif
+#    endif
         lds.lds_prefix = info[i].ldi_prefix;
         lds.lds_suffix = info[i].ldi_suffix;
         lds.lds_want_stdout = info[i].ldi_stdout;
@@ -318,7 +332,6 @@ void opal_output_reopen_all(void)
 #endif
 }
 
-
 /*
  * Close a stream
  */
@@ -328,7 +341,7 @@ void opal_output_close(int output_id)
 
     /* Setup */
 
-    if (!initialized) {
+    if (!initialized || output_id < 0) {
         return;
     }
 
@@ -336,8 +349,8 @@ void opal_output_close(int output_id)
      * free the resources associated with the descriptor */
 
     OPAL_THREAD_LOCK(&mutex);
-    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS &&
-        info[output_id].ldi_used && info[output_id].ldi_enabled) {
+    if (output_id < OPAL_OUTPUT_MAX_STREAMS && info[output_id].ldi_used
+        && info[output_id].ldi_enabled) {
         free_descriptor(output_id);
 
         /* If no one has the syslog open, we should close it */
@@ -348,7 +361,7 @@ void opal_output_close(int output_id)
             }
         }
 
-#if defined(HAVE_SYSLOG)
+#if defined(HAVE_SYSLOG) && defined(HAVE_SYSLOG_H)
         if (i >= OPAL_OUTPUT_MAX_STREAMS && syslog_opened) {
             closelog();
         }
@@ -358,7 +371,6 @@ void opal_output_close(int output_id)
     OPAL_THREAD_UNLOCK(&mutex);
 }
 
-
 /*
  * Main function to send output to a stream
  */
@@ -372,35 +384,26 @@ void opal_output(int output_id, const char *format, ...)
     }
 }
 
+/*
+ * Check whether the verbose level is high enough for the given stream
+ */
+bool opal_output_check_verbosity(int level, int output_id)
+{
+    return (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS
+            && info[output_id].ldi_verbose_level >= level);
+}
 
 /*
  * Send a message to a stream if the verbose level is high enough
  */
-void opal_output_verbose(int level, int output_id, const char *format, ...)
+void opal_output_vverbose(int level, int output_id, const char *format, va_list arglist)
 {
-    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS &&
-        info[output_id].ldi_verbose_level >= level) {
-        va_list arglist;
-        va_start(arglist, format);
-        output(output_id, format, arglist);
-        va_end(arglist);
-    }
-}
-
-
-/*
- * Send a message to a stream if the verbose level is high enough
- */
-void opal_output_vverbose(int level, int output_id, const char *format,
-                          va_list arglist)
-{
-    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS &&
-        info[output_id].ldi_verbose_level >= level) {
+    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS
+        && info[output_id].ldi_verbose_level >= level) {
         output(output_id, format, arglist);
     }
 }
 
-
 /*
  * Send a message to a string if the verbose level is high enough
  */
@@ -409,8 +412,8 @@ char *opal_output_string(int level, int output_id, const char *format, ...)
     int rc;
     char *ret = NULL;
 
-    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS &&
-        info[output_id].ldi_verbose_level >= level) {
+    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS
+        && info[output_id].ldi_verbose_level >= level) {
         va_list arglist;
         va_start(arglist, format);
         rc = make_string(&ret, &info[output_id], format, arglist);
@@ -423,18 +426,16 @@ char *opal_output_string(int level, int output_id, const char *format, ...)
     return ret;
 }
 
-
 /*
  * Send a message to a string if the verbose level is high enough
  */
-char *opal_output_vstring(int level, int output_id, const char *format,
-                          va_list arglist)
+char *opal_output_vstring(int level, int output_id, const char *format, va_list arglist)
 {
     int rc;
     char *ret = NULL;
 
-    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS &&
-        info[output_id].ldi_verbose_level >= level) {
+    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS
+        && info[output_id].ldi_verbose_level >= level) {
         rc = make_string(&ret, &info[output_id], format, arglist);
         if (OPAL_SUCCESS != rc) {
             ret = NULL;
@@ -444,7 +445,6 @@ char *opal_output_vstring(int level, int output_id, const char *format,
     return ret;
 }
 
-
 /*
  * Set the verbosity level of a stream
  */
@@ -455,13 +455,10 @@ void opal_output_set_verbosity(int output_id, int level)
     }
 }
 
-
 /*
  * Control where output flies will go
  */
-void opal_output_set_output_file_info(const char *dir,
-                                      const char *prefix,
-                                      char **olddir,
+void opal_output_set_output_file_info(const char *dir, const char *prefix, char **olddir,
                                       char **oldprefix)
 {
     if (NULL != olddir) {
@@ -481,11 +478,26 @@ void opal_output_set_output_file_info(const char *dir,
     }
 }
 
+void opal_output_register_pmix_cleanup_fn(opal_output_pmix_cleanup_fn_t cleanup_fn)
+{
+    int n;
+
+    OPAL_THREAD_LOCK(&mutex);
+    /* Register any filenames setup prior to this call */
+    for (n = 0; n < OPAL_OUTPUT_MAX_STREAMS; n++) {
+        if (NULL != info[n].ldi_filename) {
+            output_pmix_cleanup_fn(info[n].ldi_filename, false, true, false);
+        }
+    }
+    /* Save the function pointer for later calls */
+    output_pmix_cleanup_fn = cleanup_fn;
+    OPAL_THREAD_UNLOCK(&mutex);
+}
 
 /*
  * Shut down the output stream system
  */
-void opal_output_finalize(void)
+static void opal_output_finalize(void)
 {
     if (initialized) {
         if (verbose_stream != -1) {
@@ -496,17 +508,17 @@ void opal_output_finalize(void)
 
         verbose_stream = -1;
 
-        free (output_prefix);
+        free(output_prefix);
         output_prefix = NULL;
 
-        free (output_dir);
+        free(output_dir);
         output_dir = NULL;
 
-        if(NULL != temp_str) {
-	    free(temp_str);
-	    temp_str = NULL;
-	    temp_str_len = 0;
-	}
+        if (NULL != temp_str) {
+            free(temp_str);
+            temp_str = NULL;
+            temp_str_len = 0;
+        }
         OBJ_DESTRUCT(&verbose);
         OBJ_DESTRUCT(&mutex);
     }
@@ -521,7 +533,7 @@ void opal_output_finalize(void)
  */
 static void construct(opal_object_t *obj)
 {
-    opal_output_stream_t *stream = (opal_output_stream_t*) obj;
+    opal_output_stream_t *stream = (opal_output_stream_t *) obj;
 
     stream->lds_verbose_level = 0;
     stream->lds_syslog_priority = 0;
@@ -536,13 +548,22 @@ static void construct(opal_object_t *obj)
     stream->lds_want_file_append = false;
     stream->lds_file_suffix = NULL;
 }
+static void destruct(opal_object_t *obj)
+{
+    opal_output_stream_t *stream = (opal_output_stream_t *) obj;
+
+    if (NULL != stream->lds_file_suffix) {
+        free(stream->lds_file_suffix);
+        stream->lds_file_suffix = NULL;
+    }
+}
 
 /*
  * Back-end of open() and reopen().  Necessary to have it as a
  * back-end function so that we can do the thread locking properly
  * (especially upon reopen).
  */
-static int do_open(int output_id, opal_output_stream_t * lds)
+static int do_open(int output_id, opal_output_stream_t *lds)
 {
     int i;
     bool redirect_to_file = false;
@@ -554,6 +575,13 @@ static int do_open(int output_id, opal_output_stream_t * lds)
         opal_output_init();
     }
 
+    /* Bozo check */
+
+    if (output_id >= OPAL_OUTPUT_MAX_STREAMS ||
+        output_id < -1) {
+        return OPAL_ERR_BAD_PARAM;
+    }
+
     str = getenv("OPAL_OUTPUT_REDIRECT");
     if (NULL != str && 0 == strcasecmp(str, "file")) {
         redirect_to_file = true;
@@ -597,12 +625,11 @@ static int do_open(int output_id, opal_output_stream_t * lds)
     if (-1 == output_id) {
         OPAL_THREAD_UNLOCK(&mutex);
     }
-    info[i].ldi_enabled = lds->lds_is_debugging ?
-        (bool) OPAL_ENABLE_DEBUG : true;
+    info[i].ldi_enabled = lds->lds_is_debugging ? (bool) OPAL_ENABLE_DEBUG : true;
     info[i].ldi_verbose_level = lds->lds_verbose_level;
 
 #if USE_SYSLOG
-#if defined(HAVE_SYSLOG)
+#    if defined(HAVE_SYSLOG) && defined(HAVE_SYSLOG_H)
     if (opal_output_redirected_to_syslog) {
         info[i].ldi_syslog = true;
         info[i].ldi_syslog_priority = opal_output_redirected_syslog_pri;
@@ -615,11 +642,11 @@ static int do_open(int output_id, opal_output_stream_t * lds)
         }
         syslog_opened = true;
     } else {
-#endif
+#    endif
         info[i].ldi_syslog = lds->lds_want_syslog;
         if (lds->lds_want_syslog) {
 
-#if defined(HAVE_SYSLOG)
+#    if defined(HAVE_SYSLOG) && defined(HAVE_SYSLOG_H)
             if (NULL != lds->lds_syslog_ident) {
                 info[i].ldi_syslog_ident = strdup(lds->lds_syslog_ident);
                 openlog(lds->lds_syslog_ident, LOG_PID, LOG_USER);
@@ -627,14 +654,14 @@ static int do_open(int output_id, opal_output_stream_t * lds)
                 info[i].ldi_syslog_ident = NULL;
                 openlog("opal", LOG_PID, LOG_USER);
             }
-#endif
+#    endif
             syslog_opened = true;
             info[i].ldi_syslog_priority = lds->lds_syslog_priority;
         }
 
-#if defined(HAVE_SYSLOG)
+#    if defined(HAVE_SYSLOG) && defined(HAVE_SYSLOG_H)
     }
-#endif
+#    endif
 
 #else
     info[i].ldi_syslog = false;
@@ -642,7 +669,7 @@ static int do_open(int output_id, opal_output_stream_t * lds)
 
     if (NULL != lds->lds_prefix) {
         info[i].ldi_prefix = strdup(lds->lds_prefix);
-        info[i].ldi_prefix_len = (int)strlen(lds->lds_prefix);
+        info[i].ldi_prefix_len = (int) strlen(lds->lds_prefix);
     } else {
         info[i].ldi_prefix = NULL;
         info[i].ldi_prefix_len = 0;
@@ -650,7 +677,7 @@ static int do_open(int output_id, opal_output_stream_t * lds)
 
     if (NULL != lds->lds_suffix) {
         info[i].ldi_suffix = strdup(lds->lds_suffix);
-        info[i].ldi_suffix_len = (int)strlen(lds->lds_suffix);
+        info[i].ldi_suffix_len = (int) strlen(lds->lds_suffix);
     } else {
         info[i].ldi_suffix = NULL;
         info[i].ldi_suffix_len = 0;
@@ -682,8 +709,8 @@ static int do_open(int output_id, opal_output_stream_t * lds)
         if (NULL != sfx) {
             info[i].ldi_file_suffix = strdup(sfx);
         } else {
-            info[i].ldi_file_suffix = (NULL == lds->lds_file_suffix) ? NULL :
-                strdup(lds->lds_file_suffix);
+            info[i].ldi_file_suffix = (NULL == lds->lds_file_suffix) ? NULL
+                                                                     : strdup(lds->lds_file_suffix);
         }
         info[i].ldi_file_want_append = lds->lds_want_file_append;
         info[i].ldi_file_num_lines_lost = 0;
@@ -693,12 +720,12 @@ static int do_open(int output_id, opal_output_stream_t * lds)
      * This is the verbose stream, so update the internal 'verbose_stream'
      * to match the parameters set in the info[i]
      */
-    if( verbose_stream == i ) {
-        verbose.lds_want_syslog     = info[i].ldi_syslog;
+    if (verbose_stream == i) {
+        verbose.lds_want_syslog = info[i].ldi_syslog;
         verbose.lds_syslog_priority = info[i].ldi_syslog_priority;
-        verbose.lds_syslog_ident    = info[i].ldi_syslog_ident;
-        verbose.lds_want_stdout     = info[i].ldi_stdout;
-        verbose.lds_want_stderr     = info[i].ldi_stderr;
+        verbose.lds_syslog_ident = info[i].ldi_syslog_ident;
+        verbose.lds_want_stdout = info[i].ldi_stdout;
+        verbose.lds_want_stderr = info[i].ldi_stderr;
     }
 
     /* Don't open a file in the session directory now -- do that lazily
@@ -707,18 +734,16 @@ static int do_open(int output_id, opal_output_stream_t * lds)
     return i;
 }
 
-
 static int open_file(int i)
 {
     int flags;
-    char *filename;
     int n;
 
     /* first check to see if this file is already open
      * on someone else's stream - if so, we don't want
      * to open it twice
      */
-    for (n=0; n < OPAL_OUTPUT_MAX_STREAMS; n++) {
+    for (n = 0; n < OPAL_OUTPUT_MAX_STREAMS; n++) {
         if (i == n) {
             continue;
         }
@@ -728,18 +753,15 @@ static int open_file(int i)
         if (!info[n].ldi_file) {
             continue;
         }
-        if (NULL != info[i].ldi_file_suffix &&
-            NULL != info[n].ldi_file_suffix) {
+        if (NULL != info[i].ldi_file_suffix && NULL != info[n].ldi_file_suffix) {
             if (0 != strcmp(info[i].ldi_file_suffix, info[n].ldi_file_suffix)) {
                 break;
             }
         }
-        if (NULL == info[i].ldi_file_suffix &&
-            NULL != info[n].ldi_file_suffix) {
+        if (NULL == info[i].ldi_file_suffix && NULL != info[n].ldi_file_suffix) {
             break;
         }
-        if (NULL != info[i].ldi_file_suffix &&
-            NULL == info[n].ldi_file_suffix) {
+        if (NULL != info[i].ldi_file_suffix && NULL == info[n].ldi_file_suffix) {
             break;
         }
         if (info[n].ldi_fd < 0) {
@@ -752,20 +774,23 @@ static int open_file(int i)
     /* Setup the filename and open flags */
 
     if (NULL != output_dir) {
-        filename = (char *) malloc(OPAL_PATH_MAX);
-        if (NULL == filename) {
+        if (NULL != info[i].ldi_filename) {
+            free(info[i].ldi_filename);
+        }
+        info[i].ldi_filename = (char *) malloc(OPAL_PATH_MAX);
+        if (NULL == info[i].ldi_filename) {
             return OPAL_ERR_OUT_OF_RESOURCE;
         }
-        strncpy(filename, output_dir, OPAL_PATH_MAX);
-        strcat(filename, "/");
+        opal_string_copy(info[i].ldi_filename, output_dir, OPAL_PATH_MAX);
+        strcat(info[i].ldi_filename, "/");
         if (NULL != output_prefix) {
-            strcat(filename, output_prefix);
+            strcat(info[i].ldi_filename, output_prefix);
         }
         if (info[i].ldi_file_suffix != NULL) {
-            strcat(filename, info[i].ldi_file_suffix);
+            strcat(info[i].ldi_filename, info[i].ldi_file_suffix);
         } else {
             info[i].ldi_file_suffix = NULL;
-            strcat(filename, "output.txt");
+            strcat(info[i].ldi_filename, "output.txt");
         }
         flags = O_CREAT | O_RDWR;
         if (!info[i].ldi_file_want_append) {
@@ -773,19 +798,28 @@ static int open_file(int i)
         }
 
         /* Actually open the file */
-        info[i].ldi_fd = open(filename, flags, 0644);
-        free(filename);  /* release the filename in all cases */
+        info[i].ldi_fd = open(info[i].ldi_filename, flags, 0644);
         if (-1 == info[i].ldi_fd) {
             info[i].ldi_used = false;
+            /* release the filename in all error cases */
+            free(info[i].ldi_filename);
+            info[i].ldi_filename = NULL;
             return OPAL_ERR_IN_ERRNO;
         }
 
         /* Make the file be close-on-exec to prevent child inheritance
          * problems */
         if (-1 == fcntl(info[i].ldi_fd, F_SETFD, 1)) {
-           return OPAL_ERR_IN_ERRNO;
+            /* release the filename in all error cases */
+            free(info[i].ldi_filename);
+            info[i].ldi_filename = NULL;
+            return OPAL_ERR_IN_ERRNO;
         }
 
+        /* register it to be ignored */
+        if (NULL != output_pmix_cleanup_fn) {
+            output_pmix_cleanup_fn(info[i].ldi_filename, false, true, false);
+        }
     }
 
     /* Return successfully even if the session dir did not exist yet;
@@ -794,7 +828,6 @@ static int open_file(int i)
     return OPAL_SUCCESS;
 }
 
-
 /*
  * Free all the resources associated with a descriptor.
  */
@@ -802,49 +835,52 @@ static void free_descriptor(int output_id)
 {
     output_desc_t *ldi;
 
-    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS &&
-	info[output_id].ldi_used && info[output_id].ldi_enabled) {
-	ldi = &info[output_id];
+    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS && info[output_id].ldi_used
+        && info[output_id].ldi_enabled) {
+        ldi = &info[output_id];
 
-	if (-1 != ldi->ldi_fd) {
-	    close(ldi->ldi_fd);
-	}
-	ldi->ldi_used = false;
+        if (-1 != ldi->ldi_fd) {
+            close(ldi->ldi_fd);
+        }
+        if (NULL != ldi->ldi_filename) {
+            free(ldi->ldi_filename);
+            ldi->ldi_filename = NULL;
+        }
+        ldi->ldi_used = false;
 
-	/* If we strduped a prefix, suffix, or syslog ident, free it */
+        /* If we strduped a prefix, suffix, or syslog ident, free it */
 
-	if (NULL != ldi->ldi_prefix) {
-	    free(ldi->ldi_prefix);
-	}
-	ldi->ldi_prefix = NULL;
+        if (NULL != ldi->ldi_prefix) {
+            free(ldi->ldi_prefix);
+        }
+        ldi->ldi_prefix = NULL;
 
-    if (NULL != ldi->ldi_suffix) {
-        free(ldi->ldi_suffix);
-    }
-    ldi->ldi_suffix = NULL;
+        if (NULL != ldi->ldi_suffix) {
+            free(ldi->ldi_suffix);
+        }
+        ldi->ldi_suffix = NULL;
 
-    if (NULL != ldi->ldi_file_suffix) {
-	    free(ldi->ldi_file_suffix);
-	}
-	ldi->ldi_file_suffix = NULL;
+        if (NULL != ldi->ldi_file_suffix) {
+            free(ldi->ldi_file_suffix);
+        }
+        ldi->ldi_file_suffix = NULL;
 
-	if (NULL != ldi->ldi_syslog_ident) {
-	    free(ldi->ldi_syslog_ident);
-	}
-	ldi->ldi_syslog_ident = NULL;
+        if (NULL != ldi->ldi_syslog_ident) {
+            free(ldi->ldi_syslog_ident);
+        }
+        ldi->ldi_syslog_ident = NULL;
     }
 }
 
-
-static int make_string(char **no_newline_string, output_desc_t *ldi,
-                       const char *format, va_list arglist)
+static int make_string(char **no_newline_string, output_desc_t *ldi, const char *format,
+                       va_list arglist)
 {
     size_t len, total_len;
     bool want_newline = false;
 
     /* Make the formatted string */
 
-    vasprintf(no_newline_string, format, arglist);
+    opal_vasprintf(no_newline_string, format, arglist);
     total_len = len = strlen(*no_newline_string);
     if ('\n' != (*no_newline_string)[len - 1]) {
         want_newline = true;
@@ -877,27 +913,23 @@ static int make_string(char **no_newline_string, output_desc_t *ldi,
     }
     if (NULL != ldi->ldi_prefix && NULL != ldi->ldi_suffix) {
         if (want_newline) {
-            snprintf(temp_str, temp_str_len, "%s%s%s\n",
-                     ldi->ldi_prefix, *no_newline_string, ldi->ldi_suffix);
+            snprintf(temp_str, temp_str_len, "%s%s%s\n", ldi->ldi_prefix, *no_newline_string,
+                     ldi->ldi_suffix);
         } else {
-            snprintf(temp_str, temp_str_len, "%s%s%s", ldi->ldi_prefix,
-                     *no_newline_string, ldi->ldi_suffix);
+            snprintf(temp_str, temp_str_len, "%s%s%s", ldi->ldi_prefix, *no_newline_string,
+                     ldi->ldi_suffix);
         }
     } else if (NULL != ldi->ldi_prefix) {
         if (want_newline) {
-            snprintf(temp_str, temp_str_len, "%s%s\n",
-                     ldi->ldi_prefix, *no_newline_string);
+            snprintf(temp_str, temp_str_len, "%s%s\n", ldi->ldi_prefix, *no_newline_string);
         } else {
-            snprintf(temp_str, temp_str_len, "%s%s", ldi->ldi_prefix,
-                     *no_newline_string);
+            snprintf(temp_str, temp_str_len, "%s%s", ldi->ldi_prefix, *no_newline_string);
         }
     } else if (NULL != ldi->ldi_suffix) {
         if (want_newline) {
-            snprintf(temp_str, temp_str_len, "%s%s\n",
-                     *no_newline_string, ldi->ldi_suffix);
+            snprintf(temp_str, temp_str_len, "%s%s\n", *no_newline_string, ldi->ldi_suffix);
         } else {
-            snprintf(temp_str, temp_str_len, "%s%s",
-                     *no_newline_string, ldi->ldi_suffix);
+            snprintf(temp_str, temp_str_len, "%s%s", *no_newline_string, ldi->ldi_suffix);
         }
     } else {
         if (want_newline) {
@@ -929,8 +961,8 @@ static int output(int output_id, const char *format, va_list arglist)
 
     /* If it's valid, used, and enabled, output */
 
-    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS &&
-        info[output_id].ldi_used && info[output_id].ldi_enabled) {
+    if (output_id >= 0 && output_id < OPAL_OUTPUT_MAX_STREAMS && info[output_id].ldi_used
+        && info[output_id].ldi_enabled) {
         OPAL_THREAD_LOCK(&mutex);
         ldi = &info[output_id];
 
@@ -941,7 +973,7 @@ static int output(int output_id, const char *format, va_list arglist)
         }
 
         /* Syslog output -- does not use the newline-appended string */
-#if defined(HAVE_SYSLOG)
+#if defined(HAVE_SYSLOG) && defined(HAVE_SYSLOG_H)
         if (ldi->ldi_syslog) {
             syslog(ldi->ldi_syslog_priority, "%s", str);
         }
@@ -954,15 +986,20 @@ static int output(int output_id, const char *format, va_list arglist)
 
         /* stdout output */
         if (ldi->ldi_stdout) {
-            write(fileno(stdout), out, (int)strlen(out));
+            int tmp = opal_best_effort_write(fileno(stdout), out, strlen(out));
+            if (OPAL_SUCCESS != tmp) {
+                rc = tmp;
+            }
             fflush(stdout);
         }
 
         /* stderr output */
         if (ldi->ldi_stderr) {
-            write((-1 == default_stderr_fd) ?
-                  fileno(stderr) : default_stderr_fd,
-                  out, (int)strlen(out));
+            int tmp = opal_best_effort_write((-1 == default_stderr_fd) ? fileno(stderr) : default_stderr_fd,
+                                             out, strlen(out));
+            if (OPAL_SUCCESS != tmp) {
+                rc = tmp;
+            }
             fflush(stderr);
         }
 
@@ -977,20 +1014,24 @@ static int output(int output_id, const char *format, va_list arglist)
                     ++ldi->ldi_file_num_lines_lost;
                 } else if (ldi->ldi_file_num_lines_lost > 0) {
                     char buffer[BUFSIZ];
-                    char *out = buffer;
+                    int tmp;
                     memset(buffer, 0, BUFSIZ);
                     snprintf(buffer, BUFSIZ - 1,
-                             "[WARNING: %d lines lost because the Open MPI process session directory did\n not exist when opal_output() was invoked]\n",
+                             "[WARNING: %d lines lost because the Open MPI process session "
+                             "directory did\n not exist when opal_output() was invoked]\n",
                              ldi->ldi_file_num_lines_lost);
-                   write(ldi->ldi_fd, buffer, (int)strlen(buffer));
-                    ldi->ldi_file_num_lines_lost = 0;
-                    if (out != buffer) {
-                        free(out);
+                    tmp = opal_best_effort_write(ldi->ldi_fd, buffer, strlen(buffer));
+                    if (OPAL_SUCCESS != tmp) {
+                        rc = tmp;
                     }
+                    ldi->ldi_file_num_lines_lost = 0;
                 }
             }
             if (ldi->ldi_fd != -1) {
-                write(ldi->ldi_fd, out, (int)strlen(out));
+                int tmp = opal_best_effort_write(ldi->ldi_fd, out, strlen(out));
+                if (OPAL_SUCCESS != tmp) {
+                    rc = tmp;
+                }
             }
         }
         OPAL_THREAD_UNLOCK(&mutex);
