@@ -19,7 +19,7 @@ BEGIN_C_DECLS
 /*
  * MEMHEAP module functions.
  */
-OSHMEM_MODULE_DECLSPEC extern mca_memheap_base_component_2_0_0_t mca_memheap_buddy_component;
+OSHMEM_DECLSPEC extern mca_memheap_base_component_2_0_0_t mca_memheap_buddy_component;
 
 END_C_DECLS
 
