@@ -1,8 +1,8 @@
 /*
  * Copyright (c) 2013      Mellanox Technologies, Inc.
  *                         All rights reserved.
- * Copyright (c) 2014-2018 Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2014-2019 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * Copyright (c) 2016      ARM, Inc. All rights reserved.
  * $COPYRIGHT$
  *
@@ -21,8 +21,8 @@
 #include "oshmem_config.h"
 #include "opal/datatype/opal_convertor.h"
 #include "opal/mca/common/ucx/common_ucx.h"
-#include "orte/include/orte/types.h"
-#include "orte/runtime/orte_globals.h"
+#include "opal/util/opal_environ.h"
+#include "opal/util/minmax.h"
 #include "ompi/datatype/ompi_datatype.h"
 #include "ompi/mca/pml/pml.h"
 
@@ -36,7 +36,6 @@
 #include "oshmem/mca/spml/base/spml_base_putreq.h"
 #include "oshmem/mca/atomic/atomic.h"
 #include "oshmem/runtime/runtime.h"
-#include "orte/util/show_help.h"
 
 #include "oshmem/mca/spml/ucx/spml_ucx_component.h"
 #include "oshmem/mca/sshmem/ucx/sshmem_ucx.h"
@@ -59,13 +58,12 @@ mca_spml_ucx_t mca_spml_ucx = {
         .spml_ctx_destroy   = mca_spml_ucx_ctx_destroy,
         .spml_put           = mca_spml_ucx_put,
         .spml_put_nb        = mca_spml_ucx_put_nb,
+        .spml_put_signal    = mca_spml_ucx_put_signal,
+        .spml_put_signal_nb = mca_spml_ucx_put_signal_nb,
         .spml_get           = mca_spml_ucx_get,
         .spml_get_nb        = mca_spml_ucx_get_nb,
         .spml_recv          = mca_spml_ucx_recv,
         .spml_send          = mca_spml_ucx_send,
-        .spml_wait          = mca_spml_base_wait,
-        .spml_wait_nb       = mca_spml_base_wait_nb,
-        .spml_test          = mca_spml_base_test,
         .spml_fence         = mca_spml_ucx_fence,
         .spml_quiet         = mca_spml_ucx_quiet,
         .spml_rmkey_unpack  = mca_spml_ucx_rmkey_unpack,
@@ -73,7 +71,38 @@ mca_spml_ucx_t mca_spml_ucx = {
         .spml_rmkey_ptr     = mca_spml_ucx_rmkey_ptr,
         .spml_memuse_hook   = mca_spml_ucx_memuse_hook,
         .spml_put_all_nb    = mca_spml_ucx_put_all_nb,
-        .self               = (void*)&mca_spml_ucx
+        .spml_wait                      = mca_spml_base_wait,
+        .spml_wait_nb                   = mca_spml_base_wait_nb,
+        .spml_wait_until_all            = mca_spml_ucx_wait_until_all,
+        .spml_wait_until_any            = mca_spml_ucx_wait_until_any,
+        .spml_wait_until_some           = mca_spml_ucx_wait_until_some,
+        .spml_wait_until_all_vector     = mca_spml_ucx_wait_until_all_vector,
+        .spml_wait_until_any_vector     = mca_spml_ucx_wait_until_any_vector,
+        .spml_wait_until_some_vector    = mca_spml_ucx_wait_until_some_vector,
+        .spml_test                      = mca_spml_base_test,
+        .spml_test_all          	= mca_spml_ucx_test_all,
+        .spml_test_any          	= mca_spml_ucx_test_any,
+        .spml_test_some         	= mca_spml_ucx_test_some,
+        .spml_test_all_vector   	= mca_spml_ucx_test_all_vector,
+        .spml_test_any_vector   	= mca_spml_ucx_test_any_vector,
+        .spml_test_some_vector  	= mca_spml_ucx_test_some_vector,
+        .spml_team_sync                 = mca_spml_ucx_team_sync,
+        .spml_team_my_pe                = mca_spml_ucx_team_my_pe,
+        .spml_team_n_pes                = mca_spml_ucx_team_n_pes,
+        .spml_team_get_config           = mca_spml_ucx_team_get_config,
+        .spml_team_translate_pe         = mca_spml_ucx_team_translate_pe,
+        .spml_team_split_strided        = mca_spml_ucx_team_split_strided,
+        .spml_team_split_2d             = mca_spml_ucx_team_split_2d,
+        .spml_team_destroy              = mca_spml_ucx_team_destroy,
+        .spml_team_get                  = mca_spml_ucx_team_get,
+        .spml_team_create_ctx           = mca_spml_ucx_team_create_ctx,
+        .spml_team_alltoall             = mca_spml_ucx_team_alltoall,
+        .spml_team_alltoalls            = mca_spml_ucx_team_alltoalls,
+        .spml_team_broadcast            = mca_spml_ucx_team_broadcast,
+        .spml_team_collect              = mca_spml_ucx_team_collect,
+        .spml_team_fcollect             = mca_spml_ucx_team_fcollect,
+        .spml_team_reduce               = mca_spml_ucx_team_reduce,
+        .self                           = (void*)&mca_spml_ucx
     },
 
     .ucp_context            = NULL,
@@ -91,13 +120,178 @@ mca_spml_ucx_ctx_t mca_spml_ucx_ctx_default = {
     .strong_sync        = SPML_UCX_STRONG_ORDERING_NONE
 };
 
-#if HAVE_DECL_UCP_ATOMIC_OP_NBX
+#ifdef HAVE_UCP_REQUEST_PARAM_T
 static ucp_request_param_t mca_spml_ucx_request_param = {0};
 static ucp_request_param_t mca_spml_ucx_request_param_b = {
     .op_attr_mask = UCP_OP_ATTR_FLAG_FAST_CMPL
 };
 #endif
 
+unsigned
+mca_spml_ucx_mem_map_flags_symmetric_rkey(struct mca_spml_ucx *spml_ucx)
+{
+#if HAVE_DECL_UCP_MEM_MAP_SYMMETRIC_RKEY
+    if (spml_ucx->symmetric_rkey_max_count > 0) {
+        return UCP_MEM_MAP_SYMMETRIC_RKEY;
+    }
+#endif
+
+    return 0;
+}
+
+void mca_spml_ucx_rkey_store_init(mca_spml_ucx_rkey_store_t *store)
+{
+    store->array = NULL;
+    store->count = 0;
+    store->size  = 0;
+}
+
+void mca_spml_ucx_rkey_store_cleanup(mca_spml_ucx_rkey_store_t *store)
+{
+    int i;
+
+    for (i = 0; i < store->count; i++) {
+        if (store->array[i].refcnt != 0) {
+            SPML_UCX_ERROR("rkey store destroy: %d/%d has refcnt %d > 0",
+                           i, store->count, store->array[i].refcnt);
+        }
+
+        ucp_rkey_destroy(store->array[i].rkey);
+    }
+
+    free(store->array);
+}
+
+/**
+ * Find position in sorted array for existing or future entry
+ *
+ * @param[in]  store  Store of the entries
+ * @param[in]  worker Common worker for rkeys used
+ * @param[in]  rkey   Remote key to search for
+ * @param[out] index  Index of entry
+ *
+ * @return
+ *   OSHMEM_ERR_NOT_FOUND: index contains the position where future element
+ *                         should be inserted to keep array sorted
+ *   OSHMEM_SUCCESS      : index contains the position of the element
+ *   Other error         : index is not valid
+ */
+static int mca_spml_ucx_rkey_store_find(const mca_spml_ucx_rkey_store_t *store,
+                                        const ucp_worker_h worker,
+                                        const ucp_rkey_h rkey,
+                                        int *index)
+{
+#if HAVE_DECL_UCP_RKEY_COMPARE
+    ucp_rkey_compare_params_t params;
+    int i, result, m, end;
+    ucs_status_t status;
+
+    for (i = 0, end = store->count; i < end;) {
+        m = (i + end) / 2;
+
+        params.field_mask = 0;
+        status = ucp_rkey_compare(worker, store->array[m].rkey,
+                                  rkey, &params, &result);
+        if (status != UCS_OK) {
+            return OSHMEM_ERROR;
+        } else if (result == 0) {
+            *index = m;
+            return OSHMEM_SUCCESS;
+        } else if (result > 0) {
+            end = m;
+        } else {
+            i = m + 1;
+        }
+    }
+
+    *index = i;
+    return OSHMEM_ERR_NOT_FOUND;
+#else
+    return OSHMEM_ERROR;
+#endif
+}
+
+static void mca_spml_ucx_rkey_store_insert(mca_spml_ucx_rkey_store_t *store,
+                                           int i, ucp_rkey_h rkey)
+{
+    int size;
+    mca_spml_ucx_rkey_t *tmp;
+
+    if (store->count >= mca_spml_ucx.symmetric_rkey_max_count) {
+        return;
+    }
+
+    if (store->count >= store->size) {
+        size = opal_min(opal_max(store->size, 8) * 2,
+                        mca_spml_ucx.symmetric_rkey_max_count);
+        tmp  = realloc(store->array, size * sizeof(*store->array));
+        if (tmp == NULL) {
+            return;
+        }
+
+        store->array = tmp;
+        store->size  = size;
+    }
+
+    memmove(&store->array[i + 1], &store->array[i],
+            (store->count - i) * sizeof(*store->array));
+    store->array[i].rkey   = rkey;
+    store->array[i].refcnt = 1;
+    store->count++;
+    return;
+}
+
+/* Takes ownership of input ucp remote key */
+static ucp_rkey_h mca_spml_ucx_rkey_store_get(mca_spml_ucx_rkey_store_t *store,
+                                              ucp_worker_h worker,
+                                              ucp_rkey_h rkey)
+{
+    int ret, i;
+
+    if (mca_spml_ucx.symmetric_rkey_max_count == 0) {
+        return rkey;
+    }
+
+    ret = mca_spml_ucx_rkey_store_find(store, worker, rkey, &i);
+    if (ret == OSHMEM_SUCCESS) {
+        ucp_rkey_destroy(rkey);
+        store->array[i].refcnt++;
+        return store->array[i].rkey;
+    }
+
+    if (ret == OSHMEM_ERR_NOT_FOUND) {
+        mca_spml_ucx_rkey_store_insert(store, i, rkey);
+    }
+
+    return rkey;
+}
+
+static void mca_spml_ucx_rkey_store_put(mca_spml_ucx_rkey_store_t *store,
+                                        ucp_worker_h worker,
+                                        ucp_rkey_h rkey)
+{
+    mca_spml_ucx_rkey_t *entry;
+    int ret, i;
+
+    ret = mca_spml_ucx_rkey_store_find(store, worker, rkey, &i);
+    if (ret != OSHMEM_SUCCESS) {
+        goto out;
+    }
+
+    entry = &store->array[i];
+    assert(entry->rkey == rkey);
+    if (--entry->refcnt > 0) {
+        return;
+    }
+
+    memmove(&store->array[i], &store->array[i + 1],
+            (store->count - (i + 1)) * sizeof(*store->array));
+    store->count--;
+
+out:
+    ucp_rkey_destroy(rkey);
+}
+
 int mca_spml_ucx_enable(bool enable)
 {
     SPML_UCX_VERBOSE(50, "*** ucx ENABLED ****");
@@ -121,13 +315,8 @@ void mca_spml_ucx_peer_mkey_cache_init(mca_spml_ucx_ctx_t *ucx_ctx, int pe)
 int mca_spml_ucx_peer_mkey_cache_add(ucp_peer_t *ucp_peer, int index)
 {
     /* Allocate an array to hold the pointers to the ucx_cached_mkey */
-    if (index >= (int)ucp_peer->mkeys_cnt) {
+    if (index >= (int)ucp_peer->mkeys_cnt){
         int old_size = ucp_peer->mkeys_cnt;
-        if (MCA_MEMHEAP_MAX_SEGMENTS <= (index + 1)) {
-            SPML_UCX_ERROR("Failed to get new mkey for segment: max number (%d) of segment descriptor is exhausted",
-                        MCA_MEMHEAP_MAX_SEGMENTS);
-            return OSHMEM_ERROR;
-        }
         ucp_peer->mkeys_cnt = index + 1;
         ucp_peer->mkeys = realloc(ucp_peer->mkeys, sizeof(ucp_peer->mkeys[0]) * ucp_peer->mkeys_cnt);
         if (NULL == ucp_peer->mkeys) {
@@ -217,6 +406,7 @@ int mca_spml_ucx_ctx_mkey_add(mca_spml_ucx_ctx_t *ucx_ctx, int pe, uint32_t segn
 {
     int rc;
     ucs_status_t err;
+    ucp_rkey_h rkey;
 
     rc = mca_spml_ucx_ctx_mkey_new(ucx_ctx, pe, segno, ucx_mkey);
     if (OSHMEM_SUCCESS != rc) {
@@ -225,11 +415,18 @@ int mca_spml_ucx_ctx_mkey_add(mca_spml_ucx_ctx_t *ucx_ctx, int pe, uint32_t segn
     }
 
     if (mkey->u.data) {
-        err = ucp_ep_rkey_unpack(ucx_ctx->ucp_peers[pe].ucp_conn, mkey->u.data, &((*ucx_mkey)->rkey));
+        err = ucp_ep_rkey_unpack(ucx_ctx->ucp_peers[pe].ucp_conn, mkey->u.data, &rkey);
         if (UCS_OK != err) {
             SPML_UCX_ERROR("failed to unpack rkey: %s", ucs_status_string(err));
             return OSHMEM_ERROR;
         }
+
+        if (!oshmem_proc_on_local_node(pe)) {
+            rkey = mca_spml_ucx_rkey_store_get(&ucx_ctx->rkey_store, ucx_ctx->ucp_worker[0], rkey);
+        }
+
+        (*ucx_mkey)->rkey = rkey;
+
         rc = mca_spml_ucx_ctx_mkey_cache(ucx_ctx, mkey, segno, pe);
         if (OSHMEM_SUCCESS != rc) {
             SPML_UCX_ERROR("mca_spml_ucx_ctx_mkey_cache failed");
@@ -244,7 +441,7 @@ int mca_spml_ucx_ctx_mkey_del(mca_spml_ucx_ctx_t *ucx_ctx, int pe, uint32_t segn
     ucp_peer_t *ucp_peer;
     int rc;
     ucp_peer = &(ucx_ctx->ucp_peers[pe]);
-    ucp_rkey_destroy(ucx_mkey->rkey);
+    mca_spml_ucx_rkey_store_put(&ucx_ctx->rkey_store, ucx_ctx->ucp_worker[0], ucx_mkey->rkey);
     ucx_mkey->rkey = NULL;
     rc = mca_spml_ucx_peer_mkey_cache_del(ucp_peer, segno);
     if(OSHMEM_SUCCESS != rc){
@@ -256,10 +453,10 @@ int mca_spml_ucx_ctx_mkey_del(mca_spml_ucx_ctx_t *ucx_ctx, int pe, uint32_t segn
 
 int mca_spml_ucx_del_procs(oshmem_group_t* group, size_t nprocs)
 {
+    size_t ucp_workers = mca_spml_ucx.ucp_workers;
     opal_common_ucx_del_proc_t *del_procs;
     size_t i, w, n;
     int ret;
-    size_t ucp_workers = mca_spml_ucx.ucp_workers;
 
     oshmem_shmem_barrier();
 
@@ -308,7 +505,7 @@ int mca_spml_ucx_del_procs(oshmem_group_t* group, size_t nprocs)
     return ret;
 }
 
-/* TODO: move func into common place, use it with rkey exchng too */
+/* TODO: move func into common place, use it with rkey exchange too */
 static int oshmem_shmem_xchng(
         void **local_data, unsigned int *local_size, int nprocs, int ucp_workers,
         void **rdata_p, unsigned int **roffsets_p, unsigned int **rsizes_p)
@@ -436,16 +633,16 @@ int mca_spml_ucx_clear_put_op_mask(mca_spml_ucx_ctx_t *ctx)
 
 int mca_spml_ucx_add_procs(oshmem_group_t* group, size_t nprocs)
 {
+    int rc                  = OSHMEM_ERROR;
+    int my_rank             = oshmem_my_proc_id();
+    size_t ucp_workers      = mca_spml_ucx.ucp_workers;
+    unsigned int *wk_roffs  = NULL;
+    unsigned int *wk_rsizes = NULL;
+    char *wk_raddrs         = NULL;
     size_t i, w, n;
-    int rc = OSHMEM_ERROR;
-    int my_rank = oshmem_my_proc_id();
-    size_t ucp_workers = mca_spml_ucx.ucp_workers;
     ucs_status_t err;
     ucp_address_t **wk_local_addr;
     unsigned int *wk_addr_len;
-    unsigned int *wk_roffs = NULL;
-    unsigned int *wk_rsizes = NULL;
-    char *wk_raddrs = NULL;
     ucp_ep_params_t ep_params;
 
     wk_local_addr = calloc(mca_spml_ucx.ucp_workers, sizeof(ucp_address_t *));
@@ -666,7 +863,7 @@ sshmem_mkey_t *mca_spml_ucx_register(void* addr,
 {
     sshmem_mkey_t *mkeys;
     ucs_status_t status;
-    spml_ucx_mkey_t   *ucx_mkey;
+    spml_ucx_mkey_t   *ucx_mkey = NULL;
     size_t len;
     ucp_mem_map_params_t mem_map_params;
     uint32_t segno;
@@ -686,7 +883,7 @@ sshmem_mkey_t *mca_spml_ucx_register(void* addr,
     if (MEMHEAP_SEG_INVALID == segno) {
         SPML_UCX_ERROR("mca_spml_ucx_register failed because of invalid "
             "segment number: %d\n", segno);
-        return NULL;
+        goto error_out;
     }
     mem_seg = memheap_find_seg(segno);
 
@@ -702,7 +899,8 @@ sshmem_mkey_t *mca_spml_ucx_register(void* addr,
                                     UCP_MEM_MAP_PARAM_FIELD_FLAGS;
         mem_map_params.address    = addr;
         mem_map_params.length     = size;
-        mem_map_params.flags      = flags;
+        mem_map_params.flags      = flags |
+            mca_spml_ucx_mem_map_flags_symmetric_rkey(&mca_spml_ucx);
 
         status = ucp_mem_map(mca_spml_ucx.ucp_context, &mem_map_params, &mem_h);
         if (UCS_OK != status) {
@@ -717,7 +915,7 @@ sshmem_mkey_t *mca_spml_ucx_register(void* addr,
     status = ucp_rkey_pack(mca_spml_ucx.ucp_context, mem_h,
                            &mkeys[SPML_UCX_TRANSP_IDX].u.data, &len);
     if (UCS_OK != status) {
-        goto error_unmap;
+        goto error_out;
     }
     if (len >= 0xffff) {
         SPML_UCX_ERROR("packed rkey is too long: %llu >= %d",
@@ -738,7 +936,9 @@ sshmem_mkey_t *mca_spml_ucx_register(void* addr,
     return mkeys;
 
 error_unmap:
-    ucp_mem_unmap(mca_spml_ucx.ucp_context, ucx_mkey->mem_h);
+    if (NULL != ucx_mkey) {
+        ucp_mem_unmap(mca_spml_ucx.ucp_context, ucx_mkey->mem_h);
+    }
 error_out:
     free(mkeys);
 
@@ -818,11 +1018,11 @@ static inline void _ctx_remove(mca_spml_ucx_ctx_array_t *array, mca_spml_ucx_ctx
         if (array->ctxs[i] == ctx) {
             array->ctxs[i] = array->ctxs[array->ctxs_count-1];
             array->ctxs[array->ctxs_count-1] = NULL;
+            array->ctxs_count--;
             break;
         }
     }
 
-    array->ctxs_count--;
     opal_atomic_wmb ();
 }
 
@@ -847,8 +1047,11 @@ static int mca_spml_ucx_ctx_create_common(long options, mca_spml_ucx_ctx_t **ucx
     ucx_ctx->strong_sync = mca_spml_ucx_ctx_default.strong_sync;      
 
     params.field_mask  = UCP_WORKER_PARAM_FIELD_THREAD_MODE;
-    if (oshmem_mpi_thread_provided == SHMEM_THREAD_SINGLE || options & SHMEM_CTX_PRIVATE || options & SHMEM_CTX_SERIALIZED) {
+    if (oshmem_mpi_thread_provided == SHMEM_THREAD_SINGLE ||
+        oshmem_mpi_thread_provided == SHMEM_THREAD_FUNNELED || options & SHMEM_CTX_PRIVATE) {
         params.thread_mode = UCS_THREAD_MODE_SINGLE;
+    } else if (oshmem_mpi_thread_provided == SHMEM_THREAD_SERIALIZED || options & SHMEM_CTX_SERIALIZED) {
+        params.thread_mode = UCS_THREAD_MODE_SERIALIZED;
     } else {
         params.thread_mode = UCS_THREAD_MODE_MULTI;
     }
@@ -892,6 +1095,8 @@ static int mca_spml_ucx_ctx_create_common(long options, mca_spml_ucx_ctx_t **ucx
         }
     }
 
+    mca_spml_ucx_rkey_store_init(&ucx_ctx->rkey_store);
+
     *ucx_ctx_p = ucx_ctx;
 
     return OSHMEM_SUCCESS;
@@ -920,40 +1125,42 @@ static int mca_spml_ucx_ctx_create_common(long options, mca_spml_ucx_ctx_t **ucx
 
 int mca_spml_ucx_ctx_create(long options, shmem_ctx_t *ctx)
 {
-    mca_spml_ucx_ctx_t *ucx_ctx = NULL;
-    mca_spml_ucx_ctx_array_t *idle_array = &mca_spml_ucx.idle_array;
-    int i, rc = OSHMEM_SUCCESS;
+    mca_spml_ucx_ctx_t *ucx_ctx            = NULL;
+    mca_spml_ucx_ctx_array_t *active_array = &mca_spml_ucx.active_array;
+    mca_spml_ucx_ctx_array_t *idle_array   = &mca_spml_ucx.idle_array;
+    int i = 0, rc = OSHMEM_SUCCESS;
 
     /* Take a lock controlling context creation. AUX context may set specific
      * UCX parameters affecting worker creation, which are not needed for
      * regular contexts. */
-    pthread_mutex_lock(&mca_spml_ucx.ctx_create_mutex);
 
     /* Check if we have an idle context to reuse */
+    SHMEM_MUTEX_LOCK(mca_spml_ucx.internal_mutex);
     for (i = 0; i < idle_array->ctxs_count; i++) {
         if (idle_array->ctxs[i]->options & options) {
             ucx_ctx = idle_array->ctxs[i];
             _ctx_remove(idle_array, ucx_ctx, i);
+            break;
         }
     }
+    SHMEM_MUTEX_UNLOCK(mca_spml_ucx.internal_mutex);
 
     /* If we cannot reuse, create new ctx */
     if (ucx_ctx == NULL) {
+        pthread_mutex_lock(&mca_spml_ucx.ctx_create_mutex);
         rc = mca_spml_ucx_ctx_create_common(options, &ucx_ctx);
+        pthread_mutex_unlock(&mca_spml_ucx.ctx_create_mutex);
+        if (rc != OSHMEM_SUCCESS) {
+            return rc;
+        }
     }
     
-    pthread_mutex_unlock(&mca_spml_ucx.ctx_create_mutex);
-    if (rc != OSHMEM_SUCCESS) {
-        return rc;
-    }
-
-    if (mca_spml_ucx.active_array.ctxs_count == 0) {
-        opal_progress_register(spml_ucx_ctx_progress);
-    }
-
     if (!(options & SHMEM_CTX_PRIVATE)) {
         SHMEM_MUTEX_LOCK(mca_spml_ucx.internal_mutex);
-        _ctx_add(&mca_spml_ucx.active_array, ucx_ctx);
+        _ctx_add(active_array, ucx_ctx);
+        if (active_array->ctxs_count == 0) {
+            opal_progress_register(spml_ucx_ctx_progress);
+        }
         SHMEM_MUTEX_UNLOCK(mca_spml_ucx.internal_mutex);
     }
 
@@ -970,11 +1177,10 @@ void mca_spml_ucx_ctx_destroy(shmem_ctx_t ctx)
         _ctx_remove(&mca_spml_ucx.active_array, (mca_spml_ucx_ctx_t *)ctx, 0);
     }
     _ctx_add(&mca_spml_ucx.idle_array, (mca_spml_ucx_ctx_t *)ctx);
-    SHMEM_MUTEX_UNLOCK(mca_spml_ucx.internal_mutex);
-
     if (!mca_spml_ucx.active_array.ctxs_count) {
         opal_progress_unregister(spml_ucx_ctx_progress);
     }
+    SHMEM_MUTEX_UNLOCK(mca_spml_ucx.internal_mutex);
 }
 
 int mca_spml_ucx_get(shmem_ctx_t ctx, void *src_addr, size_t size, void *dst_addr, int src)
@@ -1437,3 +1643,218 @@ int mca_spml_ucx_put_all_nb(void *dest, const void *source, size_t size, long *c
 
     return OSHMEM_SUCCESS;
 }
+
+/* This routine is not implemented */
+int mca_spml_ucx_put_signal(shmem_ctx_t ctx, void* dst_addr, size_t size, void*
+        src_addr, uint64_t *sig_addr, uint64_t signal, int sig_op, int dst)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_put_signal_nb(shmem_ctx_t ctx, void* dst_addr, size_t size,
+        void* src_addr, uint64_t *sig_addr, uint64_t signal, int sig_op, int
+        dst)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+void mca_spml_ucx_wait_until_all(void *ivars, int cmp, void
+        *cmp_value, size_t nelems, const int *status, int datatype)
+{
+    RUNTIME_SHMEM_NOT_IMPLEMENTED_API_ABORT();
+}
+
+/* This routine is not implemented */
+size_t mca_spml_ucx_wait_until_any(void *ivars, int cmp, void
+        *cmp_value, size_t nelems, const int *status, int datatype)
+{
+    RUNTIME_SHMEM_NOT_IMPLEMENTED_API_ABORT_RET_SIZE_T();
+}
+
+/* This routine is not implemented */
+size_t mca_spml_ucx_wait_until_some(void *ivars, int cmp, void
+        *cmp_value, size_t nelems, size_t *indices, const int *status, int
+        datatype)
+{
+    RUNTIME_SHMEM_NOT_IMPLEMENTED_API_ABORT_RET_SIZE_T();
+}
+
+/* This routine is not implemented */ 
+void mca_spml_ucx_wait_until_all_vector(void *ivars, int cmp, void
+        *cmp_values, size_t nelems, const int *status, int datatype)
+{
+    RUNTIME_SHMEM_NOT_IMPLEMENTED_API_ABORT();
+}
+
+/* This routine is not implemented */
+size_t mca_spml_ucx_wait_until_any_vector(void *ivars, int cmp, void
+        *cmp_value, size_t nelems, const int *status, int datatype)
+{
+    RUNTIME_SHMEM_NOT_IMPLEMENTED_API_ABORT_RET_SIZE_T();
+}
+
+/* This routine is not implemented */
+size_t mca_spml_ucx_wait_until_some_vector(void *ivars, int cmp, void
+        *cmp_value, size_t nelems, size_t *indices, const int *status, int
+        datatype)
+{
+    RUNTIME_SHMEM_NOT_IMPLEMENTED_API_ABORT_RET_SIZE_T();
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_test_all(void *ivars, int cmp, void *cmp_value,
+        size_t nelems, const int *status, int datatype)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+size_t mca_spml_ucx_test_any(void *ivars, int cmp, void *cmp_value,
+        size_t nelems, const int *status, int datatype)
+{
+    RUNTIME_SHMEM_NOT_IMPLEMENTED_API_ABORT_RET_SIZE_T();
+}
+
+/* This routine is not implemented */
+size_t mca_spml_ucx_test_some(void *ivars, int cmp, void *cmp_value,
+        size_t nelems, size_t *indices, const int *status, int datatype)
+{
+    RUNTIME_SHMEM_NOT_IMPLEMENTED_API_ABORT_RET_SIZE_T();
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_test_all_vector(void *ivars, int cmp, void *cmp_values,
+                                 size_t nelems, const int *status,
+                                 int datatype)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+size_t mca_spml_ucx_test_any_vector(void *ivars, int cmp,
+                                    void *cmp_values, size_t nelems,
+                                    const int *status, int datatype)
+{
+    RUNTIME_SHMEM_NOT_IMPLEMENTED_API_ABORT_RET_SIZE_T();
+}
+
+/* This routine is not implemented */
+size_t mca_spml_ucx_test_some_vector(void *ivars, int cmp,
+                                     void *cmp_values, size_t nelems,
+                                     size_t *indices, const int *status,
+                                     int datatype)
+{
+    RUNTIME_SHMEM_NOT_IMPLEMENTED_API_ABORT_RET_SIZE_T();
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_sync(shmem_team_t team)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_my_pe(shmem_team_t team)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_n_pes(shmem_team_t team)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_get_config(shmem_team_t team, long config_mask,
+        shmem_team_config_t *config)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_translate_pe(shmem_team_t src_team, int src_pe,
+        shmem_team_t dest_team)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_split_strided(shmem_team_t parent_team, int start, int
+        stride, int size, const shmem_team_config_t *config, long config_mask,
+        shmem_team_t *new_team)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_split_2d(shmem_team_t parent_team, int xrange, const
+        shmem_team_config_t *xaxis_config, long xaxis_mask, shmem_team_t
+        *xaxis_team, const shmem_team_config_t *yaxis_config, long yaxis_mask,
+        shmem_team_t *yaxis_team)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_destroy(shmem_team_t team)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_get(shmem_ctx_t ctx, shmem_team_t *team)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_create_ctx(shmem_team_t team, long options, shmem_ctx_t *ctx)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_alltoall(shmem_team_t team, void
+        *dest, const void *source, size_t nelems, int datatype)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_alltoalls(shmem_team_t team, void
+        *dest, const void *source, ptrdiff_t dst, ptrdiff_t sst, size_t nelems,
+        int datatype)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_broadcast(shmem_team_t team, void
+        *dest, const void *source, size_t nelems, int PE_root, int datatype)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_collect(shmem_team_t team, void
+        *dest, const void *source, size_t nelems, int datatype)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_fcollect(shmem_team_t team, void
+        *dest, const void *source, size_t nelems, int datatype)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
+
+/* This routine is not implemented */
+int mca_spml_ucx_team_reduce(shmem_team_t team, void
+        *dest, const void *source, size_t nreduce, int operation, int datatype)
+{
+    return OSHMEM_ERR_NOT_IMPLEMENTED;
+}
