@@ -21,7 +21,7 @@
 #include "oshmem/include/pshmem.h"
 #pragma weak shmem_align = pshmem_align
 #pragma weak shmemalign = pshmemalign
-#include "oshmem/shmem/c/profile/defines.h"
+#include "oshmem/shmem/c/profile-defines.h"
 #endif
 
 static inline void* _shmemalign(size_t align, size_t size);
@@ -43,8 +43,12 @@ static inline void* _shmemalign(size_t align, size_t size)
 
     RUNTIME_CHECK_INIT();
 
+    SHMEM_MUTEX_LOCK(shmem_internal_mutex_alloc);
+
     rc = MCA_MEMHEAP_CALL(memalign(align, size, &pBuff));
 
+    SHMEM_MUTEX_UNLOCK(shmem_internal_mutex_alloc);
+
     if (OSHMEM_SUCCESS != rc) {
         SHMEM_API_VERBOSE(1,
                           "Allocation with shmemalign(align=%lu, size=%lu) failed.",
