@@ -1,8 +1,8 @@
 /*
  * Copyright (c) 2013-2015 Mellanox Technologies, Inc.
  *                         All rights reserved.
- * Copyright (c) 2016      Research Organization for Information Science
- *                         and Technology (RIST). All rights reserved.
+ * Copyright (c) 2016-2019 Research Organization for Information Science
+ *                         and Technology (RIST).  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -14,9 +14,6 @@
 
 #include <stdlib.h>
 
-#include "orte/util/show_help.h"
-
-#include "opal/runtime/opal_cr.h"
 #include "opal/util/output.h"
 
 #include "oshmem/constants.h"
@@ -28,24 +25,33 @@
 #if OSHMEM_PROFILING
 #include "oshmem/include/pshmem.h"
 #pragma weak shmem_init = pshmem_init
+#pragma weak shmem_init_thread = pshmem_init_thread
 #pragma weak start_pes = pstart_pes
-#include "oshmem/shmem/c/profile/defines.h"
+#include "oshmem/shmem/c/profile-defines.h"
 #endif
 
 extern int oshmem_shmem_globalexit_status;
 
-static inline void _shmem_init(void);
+static inline void _shmem_init(int required, int *provided);
 
 void shmem_init(void)
 {
+    int provided;
     /* spec says that npes are ignored for now */
-    _shmem_init();
+    _shmem_init(SHMEM_THREAD_SINGLE, &provided);
+}
+
+int shmem_init_thread(int requested, int *provided)
+{
+    _shmem_init(requested, provided);
+    return 0;
 }
 
 void start_pes(int npes)
 {
+    int provided;
     /* spec says that npes are ignored for now */
-    _shmem_init();
+    _shmem_init(SHMEM_THREAD_SINGLE, &provided);
 }
 
 static void shmem_onexit(int exitcode, void *arg)
@@ -54,11 +60,9 @@ static void shmem_onexit(int exitcode, void *arg)
     shmem_finalize();
 }
 
-static inline void _shmem_init(void)
+static inline void _shmem_init(int required, int *provided)
 {
     int err = OSHMEM_SUCCESS;
-    int provided;
-    int required = SHMEM_THREAD_SINGLE;
 
     if (oshmem_shmem_initialized) {
         /*
@@ -67,14 +71,13 @@ static inline void _shmem_init(void)
         return;
     }
 
-    err = oshmem_shmem_init(0, NULL, required, &provided);
+    err = oshmem_shmem_init(0, NULL, required, provided);
     if (OSHMEM_SUCCESS != err) {
         /* since spec does not propagete error to user we can only abort */
         SHMEM_API_ERROR("SHMEM failed to initialize - aborting");
         oshmem_shmem_abort(-1);
     }
 
-    OPAL_CR_INIT_LIBRARY();
 #if HAVE_ON_EXIT
     on_exit(shmem_onexit, NULL);
 #endif
