@@ -3,6 +3,7 @@
  * Copyright (c) 2017      The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
+ * Copyright (c) 2024      Jeffrey M. Squyres.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -15,7 +16,7 @@
 
   To be run as:
 
-  mpirun -np 4 --mca pml_monitoring_enable 2 ./check_monitoring
+  mpirun -n 4 --mca pml_monitoring_enable 2 ./check_monitoring
 */
 
 #include <mpi.h>
@@ -43,6 +44,7 @@
     static inline int pvar_prefix ## _init(MPI_T_pvar_session session)  \
     {                                                                   \
         int MPIT_result;                                                \
+        MPI_Comm comm = MPI_COMM_WORLD;                                 \
         /* Get index */                                                 \
         MPIT_result = MPI_T_pvar_get_index(pvar_prefix ## _pvar_name,   \
                                            pvar_class,                  \
@@ -57,7 +59,7 @@
         /* Allocating a new PVAR in a session will reset the counters */ \
         int count;                                                      \
         MPIT_result = MPI_T_pvar_handle_alloc(session, pvar_prefix ## _pvar_idx, \
-                                              MPI_COMM_WORLD, &(pvar_prefix ## _handle), \
+                                              &comm, &(pvar_prefix ## _handle), \
                                               &count);                  \
         if( MPI_SUCCESS != MPIT_result ) {                              \
             fprintf(stderr, "Failed to allocate handle on \"%s\" pvar, check that you have " \
@@ -147,7 +149,7 @@ static inline void pvar_all_init(MPI_T_pvar_session*session, int world_size)
     int MPIT_result, provided;
     MPIT_result = MPI_T_init_thread(MPI_THREAD_SINGLE, &provided);
     if (MPIT_result != MPI_SUCCESS) {
-        fprintf(stderr, "Failed to initialiaze MPI_Tools sub-system.\n");
+        fprintf(stderr, "Failed to initialize MPI_Tools sub-system.\n");
         MPI_Abort(MPI_COMM_WORLD, MPIT_result);
     }
     MPIT_result = MPI_T_pvar_session_create(session);
@@ -477,7 +479,7 @@ int main(int argc, char* argv[])
     if( -1 == pvar_pml_check(session, size, world_rank) ) MPI_Abort(MPI_COMM_WORLD, -1);
 
     /* third phase: exchange size times data with everyone, including self, in
-       MPI_COMM_WORLD with RMA opertations */
+       MPI_COMM_WORLD with RMA operations */
     char win_buff[20];
     MPI_Win win;
     MPI_Win_create(win_buff, 20, sizeof(char), MPI_INFO_NULL, MPI_COMM_WORLD, &win);
